/**************************************************************

cat_file.h (C-Munipack project)
Catalog files - private header file
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CMPACK_CAT_FILE_H
#define CMPACK_CAT_FILE_H

#include "cmpack_catfile.h"
#include "header.h"

/* Table of stars */
typedef struct _CmpackObjTab
{
	int nx, ny;					/**< Chart size */
	int count;					/**< Number of stars */
	int capacity;				/**< Number of allocated records */
	CmpackCatObject *list;		/**< List of stars */
} CmpackObjTab;

/* Selection record */
typedef struct _CmpackSelItem
{
	int star_id;
	CmpackSelectionType type;
} CmpackSelItem;

/* Selection list */
typedef struct _CmpackSelection
{
	int count;					/**< Number of stars */
	int capacity;				/**< Number of allocated records */
	CmpackSelItem *list;		/**< List of stars */
} CmpackSelection;

/* Selection record */
typedef struct _CmpackTag
{
	int star_id;
	char *tag;
} CmpackTag;

/* Selection list */
typedef struct _CmpackTagList
{
	int count;					/**< Number of tags */
	int capacity;				/**< Number of allocated records */
	CmpackTag *list;			/**< List of tags */
} CmpackTagList;

/* Catalog file context */
struct _CmpackCatFile 
{
	int				refcnt;		/**< Reference counter */
	FILE			*f;			/**< Associated resource */
	char			*fpath;		/**< File path */
	int				readonly;	/**< Nonzero if changes are not allowed */
	int				changed;	/**< Nonzero if changes were made */
	unsigned		flags;		/**< Read/write flags */
	CmpackHeader	head;		/**< File header */
	CmpackObjTab	stars;		/**< Table of stars */
	CmpackSelection	sel;		/**< Selection list */
	CmpackTagList	tags;		/**< Table of tags */
};

#endif
