/**************************************************************

amasslist_dlg.cpp (C-Munipack project)
The 'Plot deviations' dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>

#include "project.h"
#include "configuration.h"
#include "progress_dlg.h"
#include "amasscurve_dlg.h"
#include "main.h"
#include "utils.h"
#include "proc_classes.h"
#include "frameinfo_dlg.h"
#include "ctxhelp.h"
#include "object_dlg.h"
#include "observer_dlg.h"
#include "chartexport_dlg.h"

//-------------------------   LOCAL MENU   --------------------------------

enum tMenuId
{
	MENU_FILE = 1,
	MENU_EDIT,
	MENU_VIEW,
	MENU_TOOLS,
	MENU_HELP
};

enum tCommandId
{
	CMD_CLOSE = 100,
	CMD_EXPORT,
	CMD_REBUILD,
	CMD_PROPERTIES,
	CMD_PRINT,
	CMD_SAVE,
	CMD_HELP,
	CMD_HIDE,
	CMD_DELETE,
	CMD_PREVIEW,
	CMD_FRAMEINFO,
	CMD_GRID,
	CMD_STATISTICS,
	CMD_MEASUREMENT
};

static const CMenuBar::tMenuItem FileMenu[] = {
	{ CMenuBar::MB_ITEM,		CMD_SAVE,		"_Save" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM,		CMD_EXPORT,		"_Export as image" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM,		CMD_REBUILD,	"_Rebuild" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM,		CMD_CLOSE,		"_Close" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem EditMenu[] = {
	{ CMenuBar::MB_ITEM,		CMD_PROPERTIES,	"Edit _properties" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem ViewMenu[] = {
	{ CMenuBar::MB_CHECKBTN,	CMD_GRID,		"_Grid" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem ToolsMenu[] = {
	{ CMenuBar::MB_CHECKBTN,	CMD_STATISTICS,	"_Statistics" },
	{ CMenuBar::MB_CHECKBTN,	CMD_MEASUREMENT, "_Measurement" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem HelpMenu[] = {
	{ CMenuBar::MB_ITEM,		CMD_HELP,		"_Show help", "help" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenu AMassCurveMenu[] = {
	{ "_File",	MENU_FILE,	FileMenu },
	{ "_Edit",	MENU_EDIT,	EditMenu },
	{ "_View",	MENU_VIEW,	ViewMenu },
	{ "_Tools", MENU_TOOLS,	ToolsMenu },
	{ "_Help",	MENU_HELP,	HelpMenu,	CMenuBar::MBF_RIGHT },
	{ NULL }
};

static const CPopupMenu::tPopupMenuItem GraphContextMenu[] = {
	{ CPopupMenu::MB_ITEM, CMD_PREVIEW,		"_Show frame" },
	{ CPopupMenu::MB_ITEM, CMD_FRAMEINFO,	"_Show properties" },
	{ CPopupMenu::MB_SEPARATOR },
	{ CPopupMenu::MB_ITEM, CMD_HIDE,		"_Delete from data set" },
	{ CPopupMenu::MB_ITEM, CMD_DELETE,		"_Remove from project" },
	{ CPopupMenu::MB_END }
};

//------------------------  TOOLBAR HELPERS   ---------------------------------

static GtkToolItem *toolbar_new_button_from_stock(GtkWidget *toolbar, const gchar *stock_id, 
	const gchar *tooltip)
{
	GtkToolItem *tbtn = gtk_tool_button_new_from_stock(stock_id);
	gtk_widget_set_tooltip_text(GTK_WIDGET(tbtn), tooltip);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tbtn, -1);
	return tbtn;
}

static GtkToolItem *toolbar_new_widget(GtkWidget *toolbar, GtkWidget *widget)
{
	GtkToolItem *item = gtk_tool_item_new();
	gtk_container_add(GTK_CONTAINER(item), widget);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
	return item;
}

static GtkWidget *toolbar_new_combo(GtkWidget *toolbar, const gchar *tooltip)
{
	GtkWidget *entry = gtk_combo_box_new();
	gtk_widget_set_size_request(entry, 120, -1);
	gtk_widget_set_tooltip_text(entry, tooltip);
	toolbar_new_widget(toolbar, entry);
	return entry;
}

static GtkWidget *toolbar_new_label(GtkWidget *toolbar, const char *caption)
{
	GtkWidget *entry = gtk_label_new(caption);
	gtk_misc_set_padding(GTK_MISC(entry), 8, 0);
	gtk_misc_set_alignment(GTK_MISC(entry), 0, 0.5);
	toolbar_new_widget(toolbar, entry);
	return entry;
}

static GtkToolItem *toolbar_new_separator(GtkWidget *toolbar)
{
	GtkToolItem *tbtn = gtk_separator_tool_item_new();
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tbtn, -1);
	return tbtn;
}

//-------------------------   AIR MASS CURVE DIALOG   --------------------------------

CAMassCurveDlg::CAMassCurveDlg(void):m_UpdatePos(true), m_LastPosValid(false), 
	m_LastPosX(0), m_LastPosY(0), m_LastFocus(-1), m_InFiles(0), m_OutFiles(0), 
	m_ChannelY(-1), m_InfoMode(INFO_NONE)
{
	gchar buf[512];
	GtkWidget *tbox, *scrwnd;
	GtkCellRenderer *renderer;

	m_RawFiles = true;

	m_ShowGrid = CConfig::GetBool("AirMass", "Grid", false);

	// Dialog caption
	sprintf(buf, "%s - %s", "Air mass curve", g_AppTitle);
	gtk_window_set_title(GTK_WINDOW(m_pDlg), buf);

	// Menu bar
	m_Menu.Create(AMassCurveMenu, false);
	m_Menu.RegisterCallback(MenuCallback, this);
	gtk_box_pack_start(GTK_BOX(m_MainBox), m_Menu.Handle(), FALSE, FALSE, 0);

	// Toolbar
	tbox = gtk_toolbar_new();
	gtk_toolbar_set_style(GTK_TOOLBAR(tbox), GTK_TOOLBAR_ICONS);
	gtk_toolbar_set_orientation(GTK_TOOLBAR(tbox), GTK_ORIENTATION_HORIZONTAL);
	gtk_box_pack_start(GTK_BOX(m_MainBox), tbox, false, false, 0);

	m_YLabel = toolbar_new_label(tbox, "Y axis");
	m_YChannels = gtk_list_store_new(2, GTK_TYPE_INT, GTK_TYPE_STRING);
	m_YCombo = toolbar_new_combo(tbox, "Column shown on the vertical axis of the graph");
	g_signal_connect(G_OBJECT(m_YCombo), "changed", G_CALLBACK(changed), this);
	renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(m_YCombo), renderer, TRUE);
	gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(m_YCombo), renderer, "text", 1);

	toolbar_new_separator(tbox);

	toolbar_new_label(tbox, "Zoom");
	m_ZoomFit = toolbar_new_button_from_stock(tbox, GTK_STOCK_ZOOM_FIT, "Zoom to all points");
	g_signal_connect(G_OBJECT(m_ZoomFit), "clicked", G_CALLBACK(button_clicked), this);
	m_ZoomOut = toolbar_new_button_from_stock(tbox, GTK_STOCK_ZOOM_OUT, "Zoom out");
	g_signal_connect(G_OBJECT(m_ZoomOut), "clicked", G_CALLBACK(button_clicked), this);
	m_ZoomIn = toolbar_new_button_from_stock(tbox, GTK_STOCK_ZOOM_IN, "Zoom in");
	g_signal_connect(G_OBJECT(m_ZoomIn), "clicked", G_CALLBACK(button_clicked), this);

	GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(m_MainBox), hbox, TRUE, TRUE, 0);

	// Graph data
	m_GraphData = cmpack_graph_data_new();
	m_Graph = cmpack_graph_view_new_with_model(m_GraphData);
	cmpack_graph_view_set_mouse_control(CMPACK_GRAPH_VIEW(m_Graph), TRUE);
	cmpack_graph_view_set_selection_mode(CMPACK_GRAPH_VIEW(m_Graph), GTK_SELECTION_MULTIPLE);
	cmpack_graph_view_set_scales(CMPACK_GRAPH_VIEW(m_Graph), TRUE, TRUE);
	g_signal_connect(G_OBJECT(m_Graph), "mouse-moved", G_CALLBACK(mouse_moved), this);
	g_signal_connect(G_OBJECT(m_Graph), "button_press_event", G_CALLBACK(button_press_event), this);
	g_signal_connect(G_OBJECT(m_Graph), "selection-changed", G_CALLBACK(selection_changed), this);
	gtk_widget_set_size_request(m_Graph, 300, 200);
	scrwnd = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrwnd), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrwnd), GTK_SHADOW_ETCHED_IN);
	gtk_container_add(GTK_CONTAINER(scrwnd), m_Graph);
	gtk_box_pack_start(GTK_BOX(hbox), scrwnd, TRUE, TRUE, 0);

	// Info box
	m_InfoBox.RegisterCallback(InfoBoxCallback, this);
	gtk_box_pack_start(GTK_BOX(hbox), m_InfoBox.Handle(), FALSE, FALSE, 0);
	m_InfoBox.SetCaption("Statistics");

	// Measurement tool
	m_MeasBox.RegisterCallback(InfoBoxCallback, this);
	gtk_box_pack_start(GTK_BOX(hbox), m_MeasBox.Handle(), FALSE, FALSE, 0);
	m_MeasBox.SetGraphView(CMPACK_GRAPH_VIEW(m_Graph));

	// Timers
	m_TimerId = g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE, 100, GSourceFunc(timer_cb), this, NULL);

	// Popup menu
	m_GraphMenu.Create(GraphContextMenu);

	// Show the dialog
	gtk_widget_show_all(m_MainBox);
	m_InfoBox.Show(false);
	m_MeasBox.Show(false);
}

CAMassCurveDlg::~CAMassCurveDlg()
{
	// Disconnect graph signals
	g_signal_handlers_disconnect_by_func(G_OBJECT(m_Graph), (gpointer)mouse_moved, this);
	g_signal_handlers_disconnect_by_func(G_OBJECT(m_Graph), (gpointer)button_press_event, this);

	// Release objects, free allocated memory
	g_source_remove(m_TimerId);
	g_object_unref(m_YChannels);
	g_object_unref(m_GraphData);
}

bool CAMassCurveDlg::Make(GtkWindow *parent, bool selected_files, 
	const CObjectCoords &obj, const CLocation &loc)
{
	m_MeasBox.SetTable(NULL);
	m_Table.Clear();
	m_FrameSet.Clear();
	m_ChannelY = -1;
	m_DataSaved = false;
	m_ObjCoords = obj;
	m_Location = loc;

	if (!InitFileList(parent, selected_files))
		return false;

	if (!RebuildData(parent))
		return false;

	UpdateAirMassCurve();
	UpdateChannels();
	UpdateGraph(TRUE, TRUE);
	SetInfoMode(INFO_NONE);
	SetStatus(NULL);
	UpdateControls();
	return true;
}

void CAMassCurveDlg::OnFrameSetChanged(void)
{
	UpdateAirMassCurve();
	UpdateGraph(TRUE, TRUE);
	UpdateStatus();
	UpdateTools();
	UpdateControls();
}

void CAMassCurveDlg::UpdateAirMassCurve(void)
{
	int res;

	m_MeasBox.SetTable(NULL);
	m_Table.Clear();
	res = CmpackAirMassCurve(NULL, m_Table, m_FrameSet, m_ObjCoords, m_Location);
	if (res==0) {
		m_MeasBox.SetTable(&m_Table);
		m_MeasBox.SetChannel(CMPACK_AXIS_X, 0);
	}
}

void CAMassCurveDlg::UpdateControls()
{
	m_Menu.Check(CMD_GRID, m_ShowGrid);
	m_Menu.Check(CMD_STATISTICS, m_InfoMode == INFO_STATISTICS);
	m_Menu.Check(CMD_MEASUREMENT, m_InfoMode == INFO_MEASUREMENT);
}

void CAMassCurveDlg::UpdateGraph(gboolean autozoom_x, gboolean autozoom_y)
{
	cmpack_graph_view_set_model(CMPACK_GRAPH_VIEW(m_Graph), NULL);
	cmpack_graph_data_clear(m_GraphData);
	m_Table.Render(m_GraphData, 0, m_ChannelY, false);
	m_Table.SetView(CMPACK_GRAPH_VIEW(m_Graph), 0, m_ChannelY, false);
	cmpack_graph_view_set_grid(CMPACK_GRAPH_VIEW(m_Graph), m_ShowGrid, m_ShowGrid);
	cmpack_graph_view_set_model(CMPACK_GRAPH_VIEW(m_Graph), m_GraphData);
	cmpack_graph_view_reset_zoom(CMPACK_GRAPH_VIEW(m_Graph), autozoom_x, autozoom_y);
}

void CAMassCurveDlg::UpdateChannels(void)
{
	int i;
	GtkTreeIter iter;
	CChannels *cy = m_Table.ChannelsY();

	// Update list of dependent channels
	gtk_combo_box_set_model(GTK_COMBO_BOX(m_YCombo), NULL);
	gtk_list_store_clear(m_YChannels);
	for (i=0; i<cy->Count(); i++) {
		gtk_list_store_append(m_YChannels, &iter);
		gtk_list_store_set(m_YChannels, &iter, 0, i, 1, cy->GetName(i), -1);
	}
	gtk_combo_box_set_model(GTK_COMBO_BOX(m_YCombo), GTK_TREE_MODEL(m_YChannels));
	if (m_ChannelY<0 || m_ChannelY>=cy->Count()) 
		m_ChannelY = (cy->Count()>0 ? 0 : -1);
	if (m_ChannelY>=0) {
		m_MeasBox.SetChannel(CMPACK_AXIS_Y, m_ChannelY);
		gtk_combo_box_set_active(GTK_COMBO_BOX(m_YCombo), m_ChannelY);
	}
	gtk_widget_set_sensitive(m_YCombo, cy->Count()>1);
	gtk_widget_set_sensitive(m_YLabel, cy->Count()>1);
}

void CAMassCurveDlg::MenuCallback(CCBObject *sender, int message, int wparam, void *lparam, void *cb_data)
{
	CAMassCurveDlg *pMe = (CAMassCurveDlg*)cb_data;

	switch (message)
	{
	case CMenuBar::CB_ACTIVATE:
		pMe->OnCommand(wparam);
		break;
	}
}

void CAMassCurveDlg::OnCommand(int cmd_id)
{
	switch (cmd_id)
	{
	// File menu
	case CMD_CLOSE:
		Close();
		break;
	case CMD_SAVE:
		SaveData();
		break;
	case CMD_EXPORT:
		ExportAsImage();
		break;
	case CMD_REBUILD:
		RebuildData(GTK_WINDOW(m_pDlg));
		break;

	// Edit menu
	case CMD_PROPERTIES:
		EditProperties();
		break;

	// View menu
	case CMD_GRID:
		m_ShowGrid = m_Menu.IsChecked(CMD_GRID);
		UpdateGraph(FALSE, FALSE);
		CConfig::SetBool("AirMass", "Grid", m_ShowGrid);
		break;

	// Tools menu
	case CMD_STATISTICS:
		if (m_Menu.IsChecked(CMD_STATISTICS)) 
			SetInfoMode(INFO_STATISTICS);
		else if (m_InfoMode == INFO_STATISTICS)
			SetInfoMode(INFO_NONE);
		break;
	case CMD_MEASUREMENT:
		if (m_Menu.IsChecked(CMD_MEASUREMENT)) 
			SetInfoMode(INFO_MEASUREMENT);
		else if (m_InfoMode == INFO_MEASUREMENT)
			SetInfoMode(INFO_NONE);
		break;

	// Help menu
	case CMD_HELP:
		g_MainWnd->ShowHelp(GTK_WINDOW(m_pDlg), IDH_AIR_MASS_CURVE);
		break;
	}
}

void CAMassCurveDlg::changed(GtkWidget *pBtn, CAMassCurveDlg *pMe)
{
	pMe->OnChanged(pBtn);
}

void CAMassCurveDlg::OnChanged(GtkWidget *pBtn)
{
	int ch;
	GtkTreeIter iter;

	if (pBtn == m_YCombo) {
		if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(m_YCombo), &iter)) {
			gtk_tree_model_get(GTK_TREE_MODEL(m_YChannels), &iter, 0, &ch, -1);
			if (ch!=m_ChannelY) {
				m_ChannelY = ch;
				UpdateGraph(FALSE, TRUE);
				UpdateTools();
			}
		}
	}
}

void CAMassCurveDlg::mouse_moved(GtkWidget *button, CAMassCurveDlg *pDlg)
{
	pDlg->m_UpdatePos = true;
}

//
// Mouse button handler
//
gint CAMassCurveDlg::button_press_event(GtkWidget *widget, GdkEventButton *event, CAMassCurveDlg *pMe)
{
	int focused;

	if (event->type==GDK_BUTTON_PRESS && event->button==3) {
		gtk_widget_grab_focus(widget);
		if (widget==pMe->m_Graph) {
			focused = cmpack_graph_view_get_focused(CMPACK_GRAPH_VIEW(widget));
			if (focused>=0 && !cmpack_graph_view_is_selected(CMPACK_GRAPH_VIEW(widget), focused)) {
				cmpack_graph_view_unselect_all(CMPACK_GRAPH_VIEW(widget));
				cmpack_graph_view_select(CMPACK_GRAPH_VIEW(widget), focused);
			}
		}
		pMe->OnContextMenu(widget, event);
		return TRUE;
	}
	return FALSE;
}

//
// Show context menu
//
void CAMassCurveDlg::OnContextMenu(GtkWidget *widget, GdkEventButton *event)
{
	int res, selected;

	if (widget==m_Graph) {
		selected = cmpack_graph_view_get_selected_count(CMPACK_GRAPH_VIEW(m_Graph));
		m_GraphMenu.Enable(CMD_DELETE, selected>0);
		m_GraphMenu.Enable(CMD_HIDE, selected>0);
		m_GraphMenu.Enable(CMD_FRAMEINFO, selected==1);
		m_GraphMenu.Enable(CMD_PREVIEW, selected==1);
		res = m_GraphMenu.Execute(event);
		switch (res) 
		{
		case CMD_PREVIEW:
			// Show frame preview
			ShowFramePreview();
			break;
		case CMD_FRAMEINFO:
			// Show frame properties
			ShowFrameInfo();
			break;
		case CMD_HIDE:
			// Remove selected points from data set
			RemoveFromDataSet();
			break;
		case CMD_DELETE:
			// Delete selected points from project
			DeleteFromProject();
			break;
		}
	}
}

gboolean CAMassCurveDlg::timer_cb(CAMassCurveDlg *pDlg)
{
	if (pDlg->m_UpdatePos) {
		pDlg->m_UpdatePos = false;
		pDlg->UpdateStatus();
	}
	return TRUE;
}

void CAMassCurveDlg::selection_changed(GtkWidget *pChart, CAMassCurveDlg *pDlg)
{
	pDlg->OnSelectionChanged();
}

void CAMassCurveDlg::OnSelectionChanged(void)
{
	UpdateTools();
}

void CAMassCurveDlg::UpdateStatus(void)
{
	char	msg[256], buf1[256], buf2[256];
	gdouble	dx, dy;

	int item = cmpack_graph_view_get_focused(CMPACK_GRAPH_VIEW(m_Graph));
	if (item>=0) {
		if (m_LastFocus!=item) {
			m_LastFocus = item;
			int frame_id = (int)cmpack_graph_data_get_param(m_GraphData, item),
				xcol = m_Table.ChannelsX()->GetColumn(0),
				ycol = m_Table.ChannelsY()->GetColumn(m_ChannelY);
			if (m_Table.Find(frame_id) && m_Table.GetDbl(xcol, &dx) && m_Table.GetDbl(ycol, &dy)) {
				PrintKeyValue(buf1, dx, m_Table.ChannelsX()->Get(0));
				PrintKeyValue(buf2, dy, m_Table.ChannelsY()->Get(m_ChannelY));
				sprintf(msg, "Frame #%d: %s, %s", frame_id, buf1, buf2);
			} else {
				sprintf(msg, "Frame #%d", frame_id);
			}
			SetStatus(msg);
		}
	} else {
		m_LastFocus = -1;
		if (m_ChannelY>=0 && cmpack_graph_view_mouse_pos(CMPACK_GRAPH_VIEW(m_Graph), &dx, &dy)) {
			if (!m_LastPosValid || dx!=m_LastPosX || dy!=m_LastPosY) {
				m_LastPosValid = true;
				m_LastPosX = dx;
				m_LastPosY = dy;
				PrintKeyValue(buf1, dx, m_Table.ChannelsX()->Get(0));
				PrintKeyValue(buf2, dy, m_Table.ChannelsY()->Get(m_ChannelY));
				sprintf(msg, "Cursor: %s, %s", buf1, buf2);
				SetStatus(msg);
			}
		} else {
			if (m_LastPosValid) {
				m_LastPosValid = false;
				SetStatus(NULL);
			}
		}
	}
}

void CAMassCurveDlg::PrintKeyValue(char *buf, double value, const CChannel *channel)
{
	const gchar *name = channel->Name(), *unit = channel->Unit();
	int prec = channel->Precision();

	if (unit)
		sprintf(buf, "%s = %.*f %s", name, prec, value, unit);
	else
		sprintf(buf, "%s = %.*f", name, prec, value);
}

void CAMassCurveDlg::PrintValue(char *buf, double value, const CChannel *channel)
{
	const gchar *unit = channel->Unit();
	int prec = channel->Precision();

	if (unit)
		sprintf(buf, "%.*f %s", prec, value, unit);
	else
		sprintf(buf, "%.*f", prec, value);
}

void CAMassCurveDlg::UpdateTools(void)
{
	char	buf[256], msg[256];
	double	dx, dy, dmin, dmax;
	int		xcol, ycol;

	if (m_InfoMode == INFO_MEASUREMENT) {
		// Update measurement tool
		m_MeasBox.SetChannel(CMPACK_AXIS_Y, m_ChannelY);
	} else 
	if (m_InfoMode == INFO_STATISTICS) {
		m_InfoBox.BeginUpdate();
		m_InfoBox.Clear();
		// Show information about selected star
		xcol = m_Table.ChannelsX()->GetColumn(0);
		ycol = m_Table.ChannelsY()->GetColumn(m_ChannelY);
		int selected_count = cmpack_graph_view_get_selected_count(CMPACK_GRAPH_VIEW(m_Graph));
		if (selected_count==1) {
			int row = cmpack_graph_view_get_selected(CMPACK_GRAPH_VIEW(m_Graph));
			int frame_id = (int)cmpack_graph_data_get_param(m_GraphData, row);
			if (frame_id>=0) {
				sprintf(buf, "Frame #%d", frame_id);
				m_InfoBox.AddTitle(1, buf);
				if (m_Table.Find(frame_id) && m_Table.GetDbl(xcol, &dx) && m_Table.GetDbl(ycol, &dy)) {
					PrintKeyValue(buf, dx, m_Table.ChannelsX()->Get(0));
					m_InfoBox.AddText(buf);
					PrintKeyValue(buf, dy, m_Table.ChannelsY()->Get(m_ChannelY));
					m_InfoBox.AddText(buf);
				}
			}
		} else {
			// Compute statistics
			int count = 0, length = (selected_count>0 ? selected_count : m_Table.Rows());
			double *x = new double[length], *y = new double[length];
			if (selected_count>0) {
				GList *list = cmpack_graph_view_get_selected_rows(CMPACK_GRAPH_VIEW(m_Graph));
				for (GList *ptr=list; ptr!=NULL; ptr=ptr->next) {
					int frame_id = (int)cmpack_graph_data_get_param(m_GraphData, (gint)(intptr_t)ptr->data);
					if (m_Table.Find(frame_id) && m_Table.GetDbl(xcol, x+count) && m_Table.GetDbl(ycol, y+count))
						count++;
				}
				g_list_free(list);
			} else {
				bool ok = m_Table.Rewind();
				while (ok) {
					if (m_Table.GetDbl(xcol, x+count) && m_Table.GetDbl(ycol, y+count))
						count++;
					ok = m_Table.Next();
				}
			}
			if (selected_count>0)
				sprintf(buf, "There are %d selected points.", count);
			else
				sprintf(buf, "There are %d points in total.", count);
			m_InfoBox.AddText(buf);
			if (ComputeMinMax(count, x, &dmin, &dmax)) {
				m_InfoBox.AddText("");
				sprintf(msg, "Data %s:", m_Table.ChannelsX()->GetName(0));
				m_InfoBox.AddTitle(1, msg);
				PrintValue(buf, dmin, m_Table.ChannelsX()->Get(0));
				sprintf(msg, "%s: %s", "Min.", buf);
				m_InfoBox.AddText(msg);
				PrintValue(buf, dmax, m_Table.ChannelsX()->Get(0));
				sprintf(msg, "%s: %s", "Max.", buf);
				m_InfoBox.AddText(msg);
			}
			if (ComputeMinMax(count, y, &dmin, &dmax)) {
				m_InfoBox.AddText("");
				sprintf(msg, "Data %s:", m_Table.ChannelsY()->GetName(m_ChannelY));
				m_InfoBox.AddTitle(1, msg);
				PrintValue(buf, dmin, m_Table.ChannelsY()->Get(m_ChannelY));
				sprintf(msg, "%s: %s", "Min.", buf);
				m_InfoBox.AddText(msg);
				PrintValue(buf, dmax, m_Table.ChannelsY()->Get(m_ChannelY));
				sprintf(msg, "%s: %s", "Max.", buf);
				m_InfoBox.AddText(msg);
			}
			delete[] x;
			delete[] y;
		}
		m_InfoBox.EndUpdate();
	} else {
		// No tool is open
		cmpack_graph_view_set_cursors(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_X, 0);
		cmpack_graph_view_set_cursors(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_Y, 0);
	}
}

void CAMassCurveDlg::SetInfoMode(tInfoMode mode)
{
	m_InfoMode = mode;
	switch (m_InfoMode) 
	{
	case INFO_STATISTICS:
		m_MeasBox.Show(false);
		m_InfoBox.Show(true);
		break;
	case INFO_MEASUREMENT:
		m_InfoBox.Show(false);
		m_MeasBox.Show(true);
		break;
	default:
		m_MeasBox.Show(false);
		m_InfoBox.Show(false);
		break;
	}
	UpdateTools();
	UpdateControls();
}

void CAMassCurveDlg::InfoBoxCallback(CCBObject *sender, int message, int wparam, void* lparam, void* cb_data)
{
	CAMassCurveDlg *pMe = (CAMassCurveDlg*)cb_data;

	switch (message)
	{
	case CInfoBox::CB_CLOSED:
		pMe->OnInfoBoxClosed();
		break;
	}
}

void CAMassCurveDlg::OnInfoBoxClosed(void)
{
	m_InfoMode = INFO_NONE;
	UpdateControls();
}

void CAMassCurveDlg::button_clicked(GtkWidget *button, CAMassCurveDlg *pDlg)
{
	pDlg->OnButtonClicked(button);
}

void CAMassCurveDlg::OnButtonClicked(GtkWidget *pBtn)
{
	double zoom;

	if (pBtn==GTK_WIDGET(m_ZoomIn)) {
		zoom = cmpack_graph_view_get_zoom(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_X);
		cmpack_graph_view_set_zoom(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_X, zoom + 5.0);
		zoom = cmpack_graph_view_get_zoom(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_Y);
		cmpack_graph_view_set_zoom(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_Y, zoom + 5.0);
	} else 
	if (pBtn==GTK_WIDGET(m_ZoomOut)) {
		zoom = cmpack_graph_view_get_zoom(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_X);
		cmpack_graph_view_set_zoom(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_X, zoom - 5.0);
		zoom = cmpack_graph_view_get_zoom(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_Y);
		cmpack_graph_view_set_zoom(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_Y, zoom - 5.0);
	} else 
	if (pBtn==GTK_WIDGET(m_ZoomFit)) {
		cmpack_graph_view_reset_zoom(CMPACK_GRAPH_VIEW(m_Graph), TRUE, TRUE);
	}
}

void CAMassCurveDlg::EditProperties(void)
{
	CAMassCurveOptionsDlg dlg(GTK_WINDOW(m_pDlg));
	if (dlg.Execute(m_ObjCoords, m_Location)) {
		m_DataSaved = false;
		g_Project->SetObjectCoords(m_ObjCoords);
		CConfig::SetLocation(m_Location);
		UpdateAirMassCurve();
		UpdateChannels();
		UpdateGraph(TRUE, TRUE);
		SetStatus(NULL);
		UpdateControls();
	}
}

void CAMassCurveDlg::SaveData(void)
{
	int res;
	char *filename, *folder, *fpath;
	
	GtkWidget *pSaveDlg = gtk_file_chooser_dialog_new("Save air mass curve",
		GTK_WINDOW(m_pDlg), GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, 
		GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_standard_tooltips(GTK_FILE_CHOOSER(pSaveDlg));
	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(pSaveDlg), true);

	folder = g_Project->GetStr("Output", "Folder", NULL);
	if (!folder)
		folder = CConfig::GetStr("Output", "Folder", NULL);
	if (!folder)
		folder = g_path_get_dirname(g_Project->Path());
	if (folder && g_file_test(folder, G_FILE_TEST_IS_DIR)) 
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(pSaveDlg), folder);
	g_free(folder);
	filename = g_Project->GetStr("AirMassCurveDlg", "File", "airmass.txt");
	if (filename) 
		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(pSaveDlg), filename);
	g_free(filename);

	if (gtk_dialog_run(GTK_DIALOG(pSaveDlg))==GTK_RESPONSE_ACCEPT) {
		folder = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(pSaveDlg));
		g_Project->SetStr("Output", "Folder", folder);
		CConfig::GetStr("Output", "Folder", folder);
		g_free(folder);
		fpath = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(pSaveDlg));
		g_Project->SetStr("AirMassCurveDlg", "File", g_basename(fpath));
		m_Table.Export(fpath, true, false, false, &res);
		g_free(fpath);
		g_Project->Save();
		m_DataSaved = true;
	}
	gtk_widget_destroy(pSaveDlg);
}

void CAMassCurveDlg::ExportAsImage(void)
{
	CGraphExportDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute(CMPACK_GRAPH_VIEW(m_Graph));
}

GList *CAMassCurveDlg::GetSelectedFrames(void)
{
	GList *frames = NULL;

	GList *rows = cmpack_graph_view_get_selected_rows(CMPACK_GRAPH_VIEW(m_Graph));
	if (rows) {
		for (GList *ptr=rows; ptr!=NULL; ptr=ptr->next) {
			int frame = (gint)cmpack_graph_data_get_param(m_GraphData, (gint)(intptr_t)ptr->data);
			GtkTreePath *path = g_Project->GetFilePath(frame);
			if (path)
				frames = g_list_prepend(frames, path);
		}
		g_list_free(rows);
	}
	return g_list_reverse(frames);
}

GtkTreePath *CAMassCurveDlg::GetSelectedFrame(void)
{
	int row = cmpack_graph_view_get_selected(CMPACK_GRAPH_VIEW(m_Graph));
	if (row>=0) {
		int frame = (gint)cmpack_graph_data_get_param(m_GraphData, row);
		return g_Project->GetFilePath(frame);
	}
	return NULL;
}

//------------------------   AIR MASS CURVE OPTIONS   ---------------------------------

CAMassCurveOptionsDlg::CAMassCurveOptionsDlg(GtkWindow *pParent)
{
	GtkWidget *tbox;

	// Dialog with buttons
	m_pDlg = gtk_dialog_new_with_buttons("Air mass curve options", pParent, 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR),
		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_APPLY, GTK_RESPONSE_ACCEPT, 
		GTK_STOCK_HELP, GTK_RESPONSE_HELP, NULL);
	gtk_dialog_widget_standard_tooltips(GTK_DIALOG(m_pDlg));
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);

	// Dialog icon
	gchar *icon = get_icon_file("airmasscurve");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Dialog layout
	tbox = gtk_table_new(6, 4, FALSE);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(m_pDlg)->vbox), tbox);
	gtk_container_set_border_width(GTK_CONTAINER(tbox), 4);
	gtk_table_set_row_spacings(GTK_TABLE(tbox), 4);
	gtk_table_set_row_spacing(GTK_TABLE(tbox), 2, 12);
	gtk_table_set_col_spacings(GTK_TABLE(tbox), 4);

	// Object coordinates
	m_ObjLabel = gtk_label_new("Object - designation");
	gtk_misc_set_alignment(GTK_MISC(m_ObjLabel), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_ObjLabel, 0, 1, 0, 1);
	m_ObjName = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_ObjName, "Object's designation, e.g. RT And");
	gtk_entry_set_max_length(GTK_ENTRY(m_ObjName), MAX_OBJECT_LEN);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_ObjName, 1, 2, 0, 1);
	m_ObjBtn = gtk_button_new_with_label("More");
	gtk_widget_set_tooltip_text(m_ObjBtn, "Load object coordinates from presets or variable star catalogs");
	g_signal_connect(G_OBJECT(m_ObjBtn), "clicked", G_CALLBACK(button_clicked), this);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_ObjBtn, 2, 3, 0, 1);
	m_RALabel = gtk_label_new("- right ascension");
	gtk_misc_set_alignment(GTK_MISC(m_RALabel), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_RALabel, 0, 1, 1, 2);
	m_RA = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_RA, "Object's right ascension in hours, minutes and seconds, e.g. 12 34 56");
	gtk_entry_set_max_length(GTK_ENTRY(m_RA), 64);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_RA, 1, 2, 1, 2);
	m_RAUnit = gtk_label_new("[h m s]");
	gtk_misc_set_alignment(GTK_MISC(m_RAUnit), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_RAUnit, 2, 3, 1, 2);
	m_DecLabel = gtk_label_new("- declination");
	gtk_misc_set_alignment(GTK_MISC(m_DecLabel), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_DecLabel, 0, 1, 2, 3);
	m_Dec = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_Dec, "Object's declination in degrees, minutes and seconds, e.g. -12 34 56");
	gtk_entry_set_max_length(GTK_ENTRY(m_Dec), 64);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Dec, 1, 2, 2, 3);
	m_DecUnit = gtk_label_new("[\xC2\xB1""d m s]");
	gtk_misc_set_alignment(GTK_MISC(m_DecUnit), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_DecUnit, 2, 3, 2, 3);

	// Location
	m_LocLabel = gtk_label_new("Location - name");
	gtk_misc_set_alignment(GTK_MISC(m_LocLabel), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_LocLabel, 0, 1, 3, 4);
	m_LocName = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_LocName, "Observer's location designation, e.g. Brno, Czech Republic");
	gtk_entry_set_max_length(GTK_ENTRY(m_LocName), MAX_LOCATION_LEN);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_LocName, 1, 2, 3, 4);
	m_LocBtn = gtk_button_new_with_label("More");
	gtk_widget_set_tooltip_text(m_LocBtn, "Load observer's coordinates from presets");
	g_signal_connect(G_OBJECT(m_LocBtn), "clicked", G_CALLBACK(button_clicked), this);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_LocBtn, 2, 3, 3, 4);
	m_LonLabel = gtk_label_new("- longitude");
	gtk_misc_set_alignment(GTK_MISC(m_LonLabel), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_LonLabel, 0, 1, 4, 5);
	m_Lon = gtk_entry_new_with_max_length(32);
	gtk_widget_set_tooltip_text(m_Lon, "Observer's longitude in degrees, minutes and seconds, use E or W suffix to indicate hemisphere, e.g. 12 34 56 E");
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Lon, 1, 2, 4, 5);
	m_LonUnit = gtk_label_new("[d m s E/W]");
	gtk_misc_set_alignment(GTK_MISC(m_LonUnit), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_LonUnit, 2, 3, 4, 5);
	m_LatLabel = gtk_label_new("- latitude");
	gtk_misc_set_alignment(GTK_MISC(m_LatLabel), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_LatLabel, 0, 1, 5, 6);
	m_Lat = gtk_entry_new_with_max_length(32);
	gtk_widget_set_tooltip_text(m_Lat, "Observer's latitude in degrees, minutes and seconds, use N or S suffix to indicate hemisphere, e.g. 56 34 12 N");
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Lat, 1, 2, 5, 6);
	m_LatUnit = gtk_label_new("[d m s N/S]");
	gtk_misc_set_alignment(GTK_MISC(m_LatUnit), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_LatUnit, 2, 3, 5, 6);

	gtk_widget_show_all(GTK_DIALOG(m_pDlg)->vbox);
}

CAMassCurveOptionsDlg::~CAMassCurveOptionsDlg()
{
	gtk_widget_destroy(m_pDlg);
}

void CAMassCurveOptionsDlg::response_dialog(GtkDialog *pDlg, gint response_id, CAMassCurveOptionsDlg *pMe)
{
	if (!pMe->OnResponseDialog(response_id))
		g_signal_stop_emission_by_name(pDlg, "response");
}

bool CAMassCurveOptionsDlg::OnResponseDialog(gint response_id)
{
	switch (response_id)
	{
	case GTK_RESPONSE_ACCEPT:
		// Commit changes
		return OnCloseQuery();

	case GTK_RESPONSE_HELP:
		// Show context help
		g_MainWnd->ShowHelp(GTK_WINDOW(m_pDlg), IDH_AIR_MASS_CURVE);
		return false;
	}
	return true;
}

bool CAMassCurveOptionsDlg::Execute(CObjectCoords &obj, CLocation &loc)
{
	m_ObjCoords = obj;
	m_Location = loc;

	UpdateObjectCoords();
	UpdateLocation();
	if (gtk_dialog_run(GTK_DIALOG(m_pDlg)) != GTK_RESPONSE_ACCEPT) 
		return false;

	obj = m_ObjCoords;
	loc = m_Location;
	return true;
}

//
// Set parameters
//
void CAMassCurveOptionsDlg::UpdateObjectCoords(void)
{
	const CObjectCoords *obj = &m_ObjCoords;
	if (obj->Name())
		gtk_entry_set_text(GTK_ENTRY(m_ObjName), obj->Name());
	else
		gtk_entry_set_text(GTK_ENTRY(m_ObjName), "");
	if (obj->RA())
		gtk_entry_set_text(GTK_ENTRY(m_RA), obj->RA());
	else
		gtk_entry_set_text(GTK_ENTRY(m_RA), "");
	if (obj->Dec())
		gtk_entry_set_text(GTK_ENTRY(m_Dec), obj->Dec());
	else
		gtk_entry_set_text(GTK_ENTRY(m_Dec), "");
}

void CAMassCurveOptionsDlg::UpdateLocation(void)
{
	const CLocation *obs = &m_Location;
	if (obs->Name())
		gtk_entry_set_text(GTK_ENTRY(m_LocName), obs->Name());
	else
		gtk_entry_set_text(GTK_ENTRY(m_LocName), "");
	if (obs->Lon())
		gtk_entry_set_text(GTK_ENTRY(m_Lon), obs->Lon());
	else
		gtk_entry_set_text(GTK_ENTRY(m_Lon), "");
	if (obs->Lat())
		gtk_entry_set_text(GTK_ENTRY(m_Lat), obs->Lat());
	else
		gtk_entry_set_text(GTK_ENTRY(m_Lat), "");
}

bool CAMassCurveOptionsDlg::OnCloseQuery()
{
	const gchar *ra, *dec, *lon, *lat;
	gchar buf[64];
	gdouble x, y;

	ra = gtk_entry_get_text(GTK_ENTRY(m_RA));
	dec = gtk_entry_get_text(GTK_ENTRY(m_Dec));
	if (*ra=='\0' || *dec=='\0') {
		ShowError(GTK_WINDOW(m_pDlg), "Please, enter object coordinates.");
		return false;
	}
	if (cmpack_strtora(ra, &x)!=0) {
		ShowError(GTK_WINDOW(m_pDlg), "Invalid value of the right ascension.");
		return false;
	}
	cmpack_ratostr(x, buf, 64);
	gtk_entry_set_text(GTK_ENTRY(m_RA), buf);
		
	if (cmpack_strtodec(dec, &y)!=0) {
		ShowError(GTK_WINDOW(m_pDlg), "Invalid value of the declination.");
		return false;
	}
	cmpack_dectostr(y, buf, 64);
	gtk_entry_set_text(GTK_ENTRY(m_Dec), buf);
	m_ObjCoords.SetName(gtk_entry_get_text(GTK_ENTRY(m_ObjName)));
	m_ObjCoords.SetRA(gtk_entry_get_text(GTK_ENTRY(m_RA)));
	m_ObjCoords.SetDec(gtk_entry_get_text(GTK_ENTRY(m_Dec)));

	lon = gtk_entry_get_text(GTK_ENTRY(m_Lon));
	lat = gtk_entry_get_text(GTK_ENTRY(m_Lat));
	if (*lon=='\0' || *lat=='\0') {
		ShowError(GTK_WINDOW(m_pDlg), "Please, enter the geographic coordinates.");
		return false;
	}
	if (cmpack_strtolon(lon, &x)!=0) {
		ShowError(GTK_WINDOW(m_pDlg), "Invalid value of the longitude.");
		return false;
	}
	cmpack_lontostr(x, buf, 64);
	gtk_entry_set_text(GTK_ENTRY(m_Lon), buf);
		
	if (cmpack_strtolat(lat, &y)!=0) {
		ShowError(GTK_WINDOW(m_pDlg), "Invalid value of the latitude.");
		return false;
	}
	cmpack_lattostr(y, buf, 64);
	gtk_entry_set_text(GTK_ENTRY(m_Lat), buf);
	m_Location.SetName(gtk_entry_get_text(GTK_ENTRY(m_LocName)));
	m_Location.SetLon(gtk_entry_get_text(GTK_ENTRY(m_Lon)));
	m_Location.SetLat(gtk_entry_get_text(GTK_ENTRY(m_Lat)));

	return true;
}

void CAMassCurveOptionsDlg::button_clicked(GtkWidget *pButton, CAMassCurveOptionsDlg *pMe)
{
	pMe->OnButtonClicked(pButton);
}

void CAMassCurveOptionsDlg::OnButtonClicked(GtkWidget *pButton)
{
	if (pButton == m_ObjBtn) 
		EditObjectCoords();
	else if (pButton == m_LocBtn) 
		EditLocation();
}

void CAMassCurveOptionsDlg::EditObjectCoords(void)
{
	CObjectDlg dlg(GTK_WINDOW(m_pDlg));

	m_ObjCoords.SetName(gtk_entry_get_text(GTK_ENTRY(m_ObjName)));
	m_ObjCoords.SetRA(gtk_entry_get_text(GTK_ENTRY(m_RA)));
	m_ObjCoords.SetDec(gtk_entry_get_text(GTK_ENTRY(m_Dec)));
	if (dlg.Execute(&m_ObjCoords)) 
		UpdateObjectCoords();
}

void CAMassCurveOptionsDlg::EditLocation(void)
{
	CLocationDlg dlg(GTK_WINDOW(m_pDlg));

	m_Location.SetName(gtk_entry_get_text(GTK_ENTRY(m_LocName)));
	m_Location.SetLon(gtk_entry_get_text(GTK_ENTRY(m_Lon)));
	m_Location.SetLat(gtk_entry_get_text(GTK_ENTRY(m_Lat)));
	if (dlg.Execute(&m_Location)) 
		UpdateLocation();
}
