/**************************************************************

chart_dlg.cpp (C-Munipack project)
The 'Plot chart' dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>

#include "utils.h"
#include "main.h"
#include "ctxhelp.h"
#include "configuration.h"
#include "chartexport_dlg.h"

#define MIN_WIDTH			1
#define MIN_HEIGHT			1
#define MAX_WIDTH			8192
#define MAX_HEIGHT			8192

//-------------------------   SAVE CHART DIALOG   --------------------------------

//
// Constructor
//
CChartExportDlg::CChartExportDlg(GtkWindow *pParent):m_Parent(pParent), m_DstWidth(0), m_DstHeight(0), 
	m_JpegQuality(80), m_FilePath(NULL), m_ChartData(NULL), m_ImageData(NULL)
{
	GtkFileFilter *filters[2];

	// Dialog with buttons
	m_pDlg = gtk_file_chooser_dialog_new(NULL, GTK_WINDOW(m_Parent),
		GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, 
		GTK_RESPONSE_ACCEPT, GTK_STOCK_HELP, GTK_RESPONSE_HELP, NULL);
	gtk_file_chooser_standard_tooltips(GTK_FILE_CHOOSER(m_pDlg));
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);
	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(m_pDlg), true);

	// File filters
	filters[0] = gtk_file_filter_new();
	gtk_file_filter_add_pixbuf_formats(filters[0]);
	gtk_file_filter_set_name(filters[0], "Images");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(m_pDlg), filters[0]);
	filters[1] = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filters[1], "*");
	gtk_file_filter_set_name(filters[1], "All files");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(m_pDlg), filters[1]);
	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(m_pDlg), filters[0]);

	// Dialog icon
	gchar *icon = get_icon_file("muniwin");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Chart size widget
	gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(m_pDlg), m_Widget.Handle());
	m_Widget.RegisterCallback(WidgetCallback, this);
}

//
// Destructor
//
CChartExportDlg::~CChartExportDlg()
{
	g_free(m_FilePath);
	gtk_widget_destroy(m_pDlg);
}

//
// Execute the dialog
//
bool CChartExportDlg::Execute(CmpackChartData *chart, CmpackImageData *image)
{
	gchar *folder, *filename;

	g_free(m_FilePath);
	m_FilePath = NULL;
	m_ChartData = chart;
	m_ImageData = image;

	if (chart) {
		m_DstWidth = cmpack_chart_data_width(chart);
		m_DstHeight = cmpack_chart_data_height(chart);
	} else if (image) {
		m_DstWidth = cmpack_image_data_width(image);
		m_DstHeight = cmpack_image_data_height(image);
	} else
		return false;

	m_Widget.Init(m_DstWidth, m_DstHeight,
		(CImageSizeWidget::tSizeMode)CConfig::GetInt("ChartExport", "Mode", CImageSizeWidget::RELATIVE),
		(tImageFile)CConfig::GetInt("ChartExport", "Type", IMAGE_UNKNOWN),
		CConfig::GetDbl("ChartExport", "Zoom", 1.0),
		CConfig::GetInt("ChartExport", "Width", m_DstWidth),
		CConfig::GetInt("ChartExport", "JpegQuality", 80));

	// Restore last folder and filename
	folder = g_Project->GetStr("Output", "Folder", NULL);
	if (!folder)
		folder = CConfig::GetStr("Output", "Folder", NULL);
	if (!folder)
		folder = g_path_get_dirname(g_Project->Path());
	if (folder && g_file_test(folder, G_FILE_TEST_IS_DIR)) 
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(m_pDlg), folder);
	g_free(folder);
	filename = g_Project->GetStr("Chart", "File", "chart.png");
	if (filename) {
		const gchar *ext = ImageFileExtension(m_Widget.Type());
		if (ext) {
			gchar *newname = SetFileExtension(filename, ext);
			g_free(filename);
			filename = newname;
		}
		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(m_pDlg), g_basename(filename));
	}
	g_free(filename);

	gtk_window_set_title(GTK_WINDOW(m_pDlg), "Export chart as image");
	if (gtk_dialog_run(GTK_DIALOG(m_pDlg)) != GTK_RESPONSE_ACCEPT)
		return false;
	gtk_widget_hide(m_pDlg);

	m_FilePath = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(m_pDlg));
	m_DstWidth = m_Widget.OutputWidth();
	m_DstHeight = m_Widget.OutputHeight();
	m_JpegQuality = m_Widget.JpegQuality();

	// Save configuration
	CConfig::SetInt("ChartExport", "Mode", (int)m_Widget.Mode());
	CConfig::SetInt("ChartExport", "Type", (int)m_Widget.Type());
	CConfig::SetDbl("ChartExport", "Zoom", m_Widget.Zoom());
	CConfig::SetInt("ChartExport", "Width", m_Widget.OutputWidth());
	CConfig::SetInt("ChartExport", "JpegQuality", m_JpegQuality);
	folder = g_path_get_dirname(m_FilePath);
	g_Project->SetStr("Output", "Folder", folder);
	CConfig::GetStr("Output", "Folder", folder);
	g_free(folder);
	filename = g_path_get_basename(m_FilePath);
	g_Project->SetStr("Chart", "File", filename);
	g_free(filename);

	// Save in separate thread
	CProgressDlg pdlg(m_Parent, "Saving image");
	pdlg.SetFileName(g_basename(m_FilePath));
	if (!pdlg.Execute(ExportProc, this)) {
		ShowError(m_Parent, "Failed to write image to a file");
		return false;
	}
	return true;
}

void CChartExportDlg::response_dialog(GtkWidget *pDlg, gint response_id, CChartExportDlg *pMe)
{
	if (!pMe->OnResponseDialog(response_id)) 
		g_signal_stop_emission_by_name(pDlg, "response");
}

bool CChartExportDlg::OnResponseDialog(gint response_id)
{
	switch (response_id)
	{
	case GTK_RESPONSE_ACCEPT:
		// Check values
		if (!CheckFormat()) {
			ShowError(GTK_WINDOW(m_pDlg), "Invalid file format.\n\n"
				"If you specify the file format by an extension, check that the file name contains a valid extension. "
				"Current version of the software supports the following formats: PNG and JPEG");
			return false;
		}
		if (m_Widget.OutputWidth()<MIN_WIDTH || m_Widget.OutputHeight()<MIN_HEIGHT) {
			ShowError(GTK_WINDOW(m_pDlg), "Invalid image size");
			return false;
		}
		if (m_Widget.OutputWidth()>MAX_WIDTH || m_Widget.OutputHeight()>MAX_HEIGHT) {
			ShowError(GTK_WINDOW(m_pDlg), "The image is too large.");
			return false;
		}
		break;

	case GTK_RESPONSE_HELP:
		// Show context help
		g_MainWnd->ShowHelp(GTK_WINDOW(m_pDlg), IDH_SAVE_CHART);
		return false;
	}
	return true;
}

void CChartExportDlg::WidgetCallback(CCBObject *sender, int message, int wparam, void* lparam, void* cb_data)
{
	CChartExportDlg *pMe = (CChartExportDlg*)cb_data;

	switch (message)
	{
	case CImageSizeWidget::CB_TYPE_CHANGED:
		pMe->OnTypeChanged();
		break;
	}
}

void CChartExportDlg::OnTypeChanged(void)
{
	const gchar *ext = ImageFileExtension(m_Widget.Type());
	if (ext) {
		gchar *oldname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(m_pDlg));
		if (oldname) {
			gchar *newname = SetFileExtension(oldname, ext);
			gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(m_pDlg), g_basename(newname));
			g_free(newname);
			g_free(oldname);
		}
	}
}

bool CChartExportDlg::CheckFormat(void)
{
	tImageFile type = m_Widget.Type();
	if (type==IMAGE_UNKNOWN) {
		gchar *fname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(m_pDlg));
		if (fname) {
			type = ImageTypeByExtension(fname);
			g_free(fname);
		}
	}
	switch (type)
	{
	case IMAGE_JPEG:
		m_DstFormat = "jpeg";
		return true;
	case IMAGE_PNG:
		m_DstFormat = "png";
		return true;
	default:
		return false;
	}
}

int CChartExportDlg::ExportProc(CProgressDlg *sender, void *user_data)
{
	return ((CChartExportDlg*)user_data)->OnExport(sender);
}

int CChartExportDlg::OnExport(CProgressDlg *sender)
{
	bool negative = CConfig::GetBool(CConfig::NEGATIVE_CHARTS);

	if (m_ImageData && !m_ChartData) {
		return (int)cmpack_image_data_write_to_file(m_ImageData, m_FilePath, 
			m_DstFormat, m_DstWidth, m_DstHeight, m_JpegQuality);
	} else
	if (m_ChartData && m_ImageData) {
		return (int)cmpack_chart_data_write_to_file(m_ChartData, m_ImageData,
			m_FilePath, m_DstFormat, m_DstWidth, m_DstHeight, negative, m_JpegQuality);
	} else 
	if (m_ChartData && !m_ImageData) {
		return (int)cmpack_chart_data_write_to_file(m_ChartData, NULL,
			m_FilePath, m_DstFormat, m_DstWidth, m_DstHeight, negative, m_JpegQuality);
	}

	return 0;
}

//---------------------------   GRAPH EXPORT DIALOG   ------------------------

CGraphExportDlg::CGraphExportDlg(GtkWindow *pParent):m_JpegQuality(80), m_FilePath(NULL), 
	m_Graph(NULL)
{
	GtkFileFilter *filters[2];

	// Dialog with buttons
	m_pDlg = gtk_file_chooser_dialog_new(NULL, GTK_WINDOW(pParent),
		GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, 
		GTK_RESPONSE_ACCEPT, GTK_STOCK_HELP, GTK_RESPONSE_HELP, NULL);
	gtk_file_chooser_standard_tooltips(GTK_FILE_CHOOSER(m_pDlg));
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);
	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(m_pDlg), true);

	// File filters
	filters[0] = gtk_file_filter_new();
	gtk_file_filter_add_pixbuf_formats(filters[0]);
	gtk_file_filter_set_name(filters[0], "Images");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(m_pDlg), filters[0]);
	filters[1] = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filters[1], "*");
	gtk_file_filter_set_name(filters[1], "All files");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(m_pDlg), filters[1]);
	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(m_pDlg), filters[0]);

	// Dialog icon
	gchar *icon = get_icon_file("muniwin");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Chart size widget
	gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(m_pDlg), m_Widget.Handle());
	m_Widget.RegisterCallback(WidgetCallback, this);
}

CGraphExportDlg::~CGraphExportDlg()
{
	g_free(m_FilePath);
	gtk_widget_destroy(m_pDlg);
}

bool CGraphExportDlg::Execute(CmpackGraphView *widget)
{
	gchar *folder, *filename;

	g_free(m_FilePath);
	m_FilePath = NULL;

	m_Graph = widget;

	m_Widget.Init((tImageFile)CConfig::GetInt("GraphExport", "Type", IMAGE_UNKNOWN),
		CConfig::GetInt("GraphExport", "JpegQuality", 80));

	// Restore last folder and filename
	folder = g_Project->GetStr("Output", "Folder", NULL);
	if (!folder)
		folder = CConfig::GetStr("Output", "Folder", NULL);
	if (!folder)
		folder = g_path_get_dirname(g_Project->Path());
	if (folder && g_file_test(folder, G_FILE_TEST_IS_DIR)) 
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(m_pDlg), folder);
	g_free(folder);
	filename = g_Project->GetStr("Graph", "File", "graph.png");
	if (filename) {
		const gchar *ext = ImageFileExtension(m_Widget.Type());
		if (ext) {
			gchar *newname = SetFileExtension(filename, ext);
			g_free(filename);
			filename = newname;
		}
		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(m_pDlg), g_basename(filename));
	}
	g_free(filename);

	gtk_window_set_title(GTK_WINDOW(m_pDlg), "Export graph as image");
	if (gtk_dialog_run(GTK_DIALOG(m_pDlg)) != GTK_RESPONSE_ACCEPT)
		return false;
	gtk_widget_hide(m_pDlg);

	m_FilePath = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(m_pDlg));
	m_JpegQuality = m_Widget.JpegQuality();

	// Save configuration
	CConfig::SetInt("GraphExport", "Type", (int)m_Widget.Type());
	CConfig::SetInt("GraphExport", "JpegQuality", m_JpegQuality);
	folder = g_path_get_dirname(m_FilePath);
	g_Project->SetStr("Output", "Folder", folder);
	CConfig::GetStr("Output", "Folder", folder);
	g_free(folder);
	filename = g_path_get_basename(m_FilePath);
	g_Project->SetStr("Graph", "File", filename);
	g_free(filename);

	return cmpack_graph_view_write_to_file(CMPACK_GRAPH_VIEW(m_Graph), 
		m_FilePath, m_DstFormat, m_JpegQuality)!=0;
}

void CGraphExportDlg::response_dialog(GtkWidget *pDlg, gint response_id, CGraphExportDlg *pMe)
{
	if (!pMe->OnResponseDialog(response_id)) 
		g_signal_stop_emission_by_name(pDlg, "response");
}

bool CGraphExportDlg::OnResponseDialog(gint response_id)
{
	switch (response_id)
	{
	case GTK_RESPONSE_ACCEPT:
		// Check values
		if (!CheckFormat()) {
			ShowError(GTK_WINDOW(m_pDlg), "Invalid file format.\n\n"
				"If you specify the file format by an extension, check that the file name contains a valid extension. "
				"Current version of the software supports the following formats: PNG and JPEG");
			return false;
		}
		break;

	case GTK_RESPONSE_HELP:
		// Show context help
		g_MainWnd->ShowHelp(GTK_WINDOW(m_pDlg), IDH_SAVE_CHART);
		return false;
	}
	return true;
}

void CGraphExportDlg::WidgetCallback(CCBObject *sender, int message, int wparam, void* lparam, void* cb_data)
{
	CGraphExportDlg *pMe = (CGraphExportDlg*)cb_data;

	switch (message)
	{
	case CImageSizeWidget::CB_TYPE_CHANGED:
		pMe->OnTypeChanged();
		break;
	}
}

void CGraphExportDlg::OnTypeChanged(void)
{
	const gchar *ext = ImageFileExtension(m_Widget.Type());
	if (ext) {
		gchar *oldname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(m_pDlg));
		if (oldname) {
			gchar *newname = SetFileExtension(oldname, ext);
			gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(m_pDlg), g_basename(newname));
			g_free(newname);
			g_free(oldname);
		}
	}
}

bool CGraphExportDlg::CheckFormat(void)
{
	tImageFile type = m_Widget.Type();
	if (type==IMAGE_UNKNOWN) {
		gchar *fname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(m_pDlg));
		if (fname) {
			type = ImageTypeByExtension(fname);
			g_free(fname);
		}
	}
	switch (type)
	{
	case IMAGE_JPEG:
		m_DstFormat = "jpeg";
		return true;
	case IMAGE_PNG:
		m_DstFormat = "png";
		return true;
	default:
		return false;
	}
}
