/**************************************************************

stars_dlg.cpp (C-Munipack project)
The 'Choose stars' dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>

#include "project.h"
#include "choosestars_dlg.h"
#include "main.h"
#include "utils.h"
#include "configuration.h"
#include "configuration.h"
#include "profile.h"
#include "ctxhelp.h"

//-------------------------   POPUP MENU   ---------------------------

enum tPopupCommand
{
	CMD_SET_VARIABLE,
	CMD_SET_COMPARISON,
	CMD_SET_CHECK,
	CMD_UNSET,
	CMD_CLEAR_ALL,
	CMD_EDIT_TAG,
	CMD_REMOVE_TAG,
	CMD_CLEAR_TAGS
};

static const CPopupMenu::tPopupMenuItem SelectMenu[] = {
	{ CPopupMenu::MB_ITEM, CMD_SET_VARIABLE,	"_Variable" },
	{ CPopupMenu::MB_ITEM, CMD_SET_COMPARISON,	"_Comparison" },
	{ CPopupMenu::MB_ITEM, CMD_SET_CHECK,		"Chec_k" },
	{ CPopupMenu::MB_SEPARATOR },
	{ CPopupMenu::MB_ITEM, CMD_UNSET,			"_Unselect" },
	{ CPopupMenu::MB_ITEM, CMD_CLEAR_ALL,		"Clear _selection" },
	{ CPopupMenu::MB_SEPARATOR },
	{ CPopupMenu::MB_ITEM, CMD_EDIT_TAG,		"_Edit tag" },
	{ CPopupMenu::MB_ITEM, CMD_REMOVE_TAG,		"_Remove tag" },
	{ CPopupMenu::MB_ITEM, CMD_CLEAR_TAGS,		"Clear _all tags" },
	{ CPopupMenu::MB_END }
};

static const CPopupMenu::tPopupMenuItem ContextMenu[] = {
	{ CPopupMenu::MB_ITEM, CMD_CLEAR_ALL,		"Clear _selection" },
	{ CPopupMenu::MB_ITEM, CMD_CLEAR_TAGS,		"Clear _all tags" },
	{ CPopupMenu::MB_END }
};

//-------------------------   PRIVATE DATA   ---------------------------

static const struct {
	const gchar *label;
	CmpackColor color;
} Types[CMPACK_SELECT_COUNT] = {
	{ NULL, CMPACK_COLOR_DEFAULT },
	{ "var", CMPACK_COLOR_RED },
	{ "comp", CMPACK_COLOR_GREEN },
	{ "check", CMPACK_COLOR_BLUE }
};

//------------------------  TOOLBAR HELPERS   ---------------------------------

static GtkToolItem *toolbar_new_button_from_stock(GtkWidget *toolbar, const gchar *stock_id,
	const gchar *tooltip)
{
	GtkToolItem *tbtn = gtk_tool_button_new_from_stock(stock_id);
	gtk_widget_set_tooltip_text(GTK_WIDGET(tbtn), tooltip);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tbtn, -1);
	return tbtn;
}

static GtkToolItem *toolbar_new_button(GtkWidget *toolbar, const gchar *label,
	const gchar *tooltip)
{
	GtkToolItem *tbtn = gtk_tool_button_new(NULL, label);
	gtk_widget_set_tooltip_text(GTK_WIDGET(tbtn), tooltip);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tbtn, -1);
	return tbtn;
}

static GtkToolItem *toolbar_new_radio(GtkWidget *toolbar, GtkToolItem *first_in_group, 
	const gchar *label, const gchar *tooltip)
{
	GSList *group;
	if (first_in_group)
		group = gtk_radio_tool_button_get_group(GTK_RADIO_TOOL_BUTTON(first_in_group));
	else
		group = NULL;

	GtkToolItem *tbtn = gtk_radio_tool_button_new(group);
	gtk_tool_button_set_label(GTK_TOOL_BUTTON(tbtn), label);
	gtk_widget_set_tooltip_text(GTK_WIDGET(tbtn), tooltip);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tbtn, -1);
	return tbtn;
}

static GtkToolItem *toolbar_new_widget(GtkWidget *toolbar, GtkWidget *widget)
{
	GtkToolItem *item = gtk_tool_item_new();
	gtk_container_add(GTK_CONTAINER(item), widget);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
	return item;
}

static GtkWidget *toolbar_new_label(GtkWidget *toolbar, const char *caption)
{
	GtkWidget *entry = gtk_label_new(caption);
	gtk_misc_set_padding(GTK_MISC(entry), 8, 0);
	gtk_misc_set_alignment(GTK_MISC(entry), 0, 0.5);
	toolbar_new_widget(toolbar, entry);
	return entry;
}

static GtkToolItem *toolbar_new_separator(GtkWidget *toolbar)
{
	GtkToolItem *tbtn = gtk_separator_tool_item_new();
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tbtn, -1);
	return tbtn;
}

//-------------------------   CHOOSE STARS DIALOG   --------------------------------

CChooseStarsDlg::CChooseStarsDlg(GtkWindow *pParent):m_ImageData(NULL), 
	m_RefType(REF_UNDEFINED), m_ApertureIndex(-1), m_InstMagnitudes(false), 
	m_SingleComparison(false)
{
	GtkWidget *tbox, *scrwnd;
	GdkRectangle rc;

	m_DisplayMode = (tDisplayMode)CConfig::GetInt("ChooseStarsDlg", "Mode");

	// Dialog with buttons
	m_pDlg = gtk_dialog_new_with_buttons("Choose stars", pParent, 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR),
		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, 
		GTK_STOCK_HELP, GTK_RESPONSE_HELP, NULL);
	gtk_dialog_widget_standard_tooltips(GTK_DIALOG(m_pDlg));
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);

	// Dialog icon
	gchar *icon = get_icon_file("muniwin");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Dialog size
	GdkScreen *scr = gdk_screen_get_default();
	gdk_screen_get_monitor_geometry(scr, 0, &rc);
	if (rc.width>0 && rc.height>0)
		gtk_window_set_default_size(GTK_WINDOW(m_pDlg), RoundToInt(0.9*rc.width), RoundToInt(0.8*rc.height));
	else
		gtk_window_set_default_size(GTK_WINDOW(m_pDlg), 640, 320);
	gtk_window_set_position(GTK_WINDOW(m_pDlg), GTK_WIN_POS_CENTER);

	// Toolbar
	tbox = gtk_toolbar_new();
	gtk_toolbar_set_style(GTK_TOOLBAR(tbox), GTK_TOOLBAR_ICONS);
	gtk_toolbar_set_orientation(GTK_TOOLBAR(tbox), GTK_ORIENTATION_HORIZONTAL);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(m_pDlg)->vbox), tbox, false, false, 0);

	m_ViewLabel = toolbar_new_label(tbox, "View");
	m_ShowImage = toolbar_new_radio(tbox, NULL, "Image", "Display an image only");
	g_signal_connect(G_OBJECT(m_ShowImage), "toggled", G_CALLBACK(button_clicked), this);
	m_ShowChart = toolbar_new_radio(tbox, m_ShowImage, "Chart", "Display objects on a flat background");
	g_signal_connect(G_OBJECT(m_ShowChart), "toggled", G_CALLBACK(button_clicked), this);
	m_ShowMixed = toolbar_new_radio(tbox, m_ShowImage, "Mixed", "Display objects over an image");
	g_signal_connect(G_OBJECT(m_ShowMixed), "toggled", G_CALLBACK(button_clicked), this);

	toolbar_new_separator(tbox);

	toolbar_new_label(tbox, "Selection");
	m_ClearBtn = toolbar_new_button(tbox, "Clear", "Unselect all objects");
	g_signal_connect(G_OBJECT(m_ClearBtn), "clicked", G_CALLBACK(button_clicked), this);
	m_RestoreBtn = toolbar_new_button(tbox, "Restore", "Restore selection");
	g_signal_connect(G_OBJECT(m_RestoreBtn), "clicked", G_CALLBACK(button_clicked), this);

	toolbar_new_separator(tbox);

	toolbar_new_label(tbox, "Zoom");
	m_ZoomFit = toolbar_new_button_from_stock(tbox, GTK_STOCK_ZOOM_FIT, "Fit the frame to the window");
	g_signal_connect(G_OBJECT(m_ZoomFit), "clicked", G_CALLBACK(button_clicked), this);
	m_ZoomOut = toolbar_new_button_from_stock(tbox, GTK_STOCK_ZOOM_OUT, "Zoom out");
	g_signal_connect(G_OBJECT(m_ZoomOut), "clicked", G_CALLBACK(button_clicked), this);
	m_ZoomIn = toolbar_new_button_from_stock(tbox, GTK_STOCK_ZOOM_IN, "Zoom in");
	g_signal_connect(G_OBJECT(m_ZoomIn), "clicked", G_CALLBACK(button_clicked), this);

	// Chart
	m_ChartData = cmpack_chart_data_new();
	m_Chart = cmpack_chart_view_new();
	cmpack_chart_view_set_mouse_control(CMPACK_CHART_VIEW(m_Chart), TRUE);
	cmpack_chart_view_set_activation_mode(CMPACK_CHART_VIEW(m_Chart), CMPACK_ACTIVATION_CLICK);
	g_signal_connect(G_OBJECT(m_Chart), "item-activated", G_CALLBACK(item_activated), this);
	g_signal_connect(G_OBJECT(m_Chart), "button_press_event", G_CALLBACK(button_press_event), this);
	gtk_widget_set_size_request(m_Chart, 320, 200);
	scrwnd = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrwnd), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrwnd), GTK_SHADOW_ETCHED_IN);
	gtk_container_add(GTK_CONTAINER(scrwnd), m_Chart);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(m_pDlg)->vbox), scrwnd, TRUE, TRUE, 0);

	// Make popup menus
	m_SelectMenu.Create(SelectMenu);
	m_ContextMenu.Create(ContextMenu);

	// Signals
	gtk_widget_show_all(GTK_DIALOG(m_pDlg)->vbox);
}

CChooseStarsDlg::~CChooseStarsDlg()
{
	g_object_unref(m_ChartData);
	if (m_ImageData)
		g_object_unref(m_ImageData);
	gtk_widget_destroy(m_pDlg);
}

//
// Left button click
//
void CChooseStarsDlg::item_activated(GtkWidget *widget, gint item, CChooseStarsDlg *pMe)
{
	GdkEventButton ev;
	ev.button = 1;
	ev.time = gtk_get_current_event_time();
	pMe->OnSelectMenu(&ev, item);
}

//
// Right mouse click
//
gint CChooseStarsDlg::button_press_event(GtkWidget *widget, GdkEventButton *event, CChooseStarsDlg *pMe)
{
	int focused;

	if (event->type==GDK_BUTTON_PRESS && event->button==3) {
		gtk_widget_grab_focus(widget);
		if (widget==pMe->m_Chart) {
			focused = cmpack_chart_view_get_focused(CMPACK_CHART_VIEW(widget));
			if (focused>=0) 
				pMe->OnSelectMenu(event, focused);
			else
				pMe->OnContextMenu(event);
		}
		return TRUE;
	}
	return FALSE;
}

//
// Object's context menu
//
void CChooseStarsDlg::OnSelectMenu(GdkEventButton *event, gint row)
{
	int star_id, index;
	CmpackSelectionType type;

	if (row>=0) {
		star_id = (int)cmpack_chart_data_get_param(m_ChartData, row);
		index = m_Selection.IndexOf(star_id);
		if (index>=0)
			type = m_Selection.GetType(index);
		else
			type = CMPACK_SELECT_NONE;
		m_SelectMenu.Enable(CMD_SET_VARIABLE, type!=CMPACK_SELECT_VAR);
		m_SelectMenu.Enable(CMD_SET_COMPARISON, type!=CMPACK_SELECT_COMP);
		m_SelectMenu.Enable(CMD_SET_CHECK, type!=CMPACK_SELECT_CHECK);
		m_SelectMenu.Enable(CMD_UNSET, type!=CMPACK_SELECT_NONE);
		m_SelectMenu.Enable(CMD_CLEAR_ALL, m_Selection.Count()>0);
		m_SelectMenu.Enable(CMD_REMOVE_TAG, m_Tags.Get(star_id)!=0);
		m_SelectMenu.Enable(CMD_CLEAR_TAGS, m_Tags.Count()>0);
		switch (m_SelectMenu.Execute(event))
		{
		case CMD_SET_VARIABLE:
			if (!m_InstMagnitudes) 
				UnselectType(CMPACK_SELECT_VAR);
			Select(row, star_id, CMPACK_SELECT_VAR);
			break;
		case CMD_SET_COMPARISON:
			if (m_SingleComparison && !m_InstMagnitudes)
				UnselectType(CMPACK_SELECT_COMP);
			Select(row, star_id, CMPACK_SELECT_COMP);
			break;
		case CMD_SET_CHECK:
			Select(row, star_id, CMPACK_SELECT_CHECK);
			break;
		case CMD_UNSET:
			Unselect(row, star_id);
			break;
		case CMD_CLEAR_ALL:
			ClearSelection();
			break;
		case CMD_EDIT_TAG:
			EditTag(row, star_id);
			break;
		case CMD_REMOVE_TAG:
			RemoveTag(row, star_id);
			break;
		case CMD_CLEAR_TAGS:
			ClearTags();
			break;
		}
	}
}

//
// Context menu (no object focused)
//
void CChooseStarsDlg::OnContextMenu(GdkEventButton *event)
{
	m_ContextMenu.Enable(CMD_CLEAR_ALL, m_Selection.Count()>0);
	m_ContextMenu.Enable(CMD_CLEAR_TAGS, m_Tags.Count()>0);
	switch (m_ContextMenu.Execute(event))
	{
	case CMD_CLEAR_ALL:
		ClearSelection();
		break;
	case CMD_CLEAR_TAGS:
		ClearTags();
		break;
	}
}

void CChooseStarsDlg::Select(int row, int star_id, CmpackSelectionType type)
{
	if (row>=0 && star_id>=0) {
		m_Selection.Select(star_id, type);
		UpdateObject(row, star_id);
	}
}

void CChooseStarsDlg::Unselect(int row, int star_id)
{
	if (row>=0 && star_id>=0) {
		m_Selection.Select(star_id, CMPACK_SELECT_NONE);
		UpdateObject(row, star_id);
	}
}

void CChooseStarsDlg::ClearSelection(void)
{
	int count = m_Selection.Count();
	int *star_id = (int*)g_malloc(count*sizeof(int));
	for (int i=0; i<count; i++)
		star_id[i] = m_Selection.GetId(i);
	m_Selection.Clear();
	for (int i=0; i<count; i++) {
		int row = cmpack_chart_data_find_item(m_ChartData, star_id[i]);
		UpdateObject(row, star_id[i]);
	}
	g_free(star_id);
}

void CChooseStarsDlg::EditTag(int row, int star_id)
{
	char obj[256], buf[256];

	if (row>=0 && star_id>=0) {
		CTextQueryDlg dlg(GTK_WINDOW(m_pDlg), "Edit tag");
		int i = m_Selection.IndexOf(star_id);
		if (i>=0) {
			int index = m_Selection.GetIndex(i);
			CmpackSelectionType type = m_Selection.GetType(i);
			if (index==1)
				strcpy(obj, Types[type].label);
			else
				sprintf(obj, "%s #%d", Types[type].label, index);
		} else 
			sprintf(obj, "object #%d", star_id);
		sprintf(buf, "Enter a tag for '%s':", obj);
		gchar *value = dlg.Execute(buf, MAX_TAG_SIZE, m_Tags.Get(star_id));
		if (value) {
			SetTag(row, star_id, value);
			UpdateObject(row, star_id);
		}
	}
}

void CChooseStarsDlg::SetTag(int row, int star_id, const gchar *value)
{
	if (row>=0 && star_id>=0) {
		if (value && value[0]!='\0') 
			m_Tags.Set(star_id, value);
		else
			m_Tags.Remove(star_id);
		UpdateObject(row, star_id);
	}
}

void CChooseStarsDlg::RemoveTag(int row, int star_id)
{
	if (row>=0 && m_Tags.Get(star_id)) {
		m_Tags.Remove(star_id);
		UpdateObject(row, star_id);
	}
}

void CChooseStarsDlg::ClearTags(void)
{
	int id, i = 0, count = m_Tags.Count();
	int *star_id = (int*)g_malloc(count*sizeof(int));
	intptr_t ptr = m_Tags.First(&id, NULL);
	while (ptr) {
		star_id[i++] = id;
		ptr = m_Tags.Next(ptr, &id, NULL);
	}
	m_Tags.Clear();
	for (int i=0; i<count; i++) {
		int row = cmpack_chart_data_find_item(m_ChartData, star_id[i]);
		UpdateObject(row, star_id[i]);
	}
	g_free(star_id);
}

void CChooseStarsDlg::UnselectType(CmpackSelectionType type)
{
	int i, count;

	count = m_Selection.CountStars(type);
	if (count>0) {
		int *stars = (int*)g_malloc(count*sizeof(int));
		m_Selection.GetStarList(type, stars, count);
		for (i=0; i<count; i++) {
			int row = cmpack_chart_data_find_item(m_ChartData, stars[i]);
			Unselect(row, stars[i]);
		}
		g_free(stars);
	}
}

void CChooseStarsDlg::SetSelection(const CSelection &sel)
{
	CSelection old = m_Selection;
	m_Selection = sel;

	int count = cmpack_chart_data_count(m_ChartData);
	for (int row=0; row<count; row++) {
		int star_id = (int)cmpack_chart_data_get_param(m_ChartData, row);
		if (old.IndexOf(star_id)>=0 || sel.IndexOf(star_id)>=0)
			UpdateObject(row, star_id);
	}
}

void CChooseStarsDlg::SetTags(const CTags &tags)
{
	CTags old = m_Tags;
	m_Tags = tags;

	int count = cmpack_chart_data_count(m_ChartData);
	for (int row=0; row<count; row++) {
		int star_id = (int)cmpack_chart_data_get_param(m_ChartData, row);
		if (old.Get(star_id)!=NULL || tags.Get(star_id)!=NULL)
			UpdateObject(row, star_id);
	}
}

void CChooseStarsDlg::button_clicked(GtkWidget *pButton, CChooseStarsDlg *pMe)
{
	pMe->OnButtonClicked(pButton);
}

void CChooseStarsDlg::OnButtonClicked(GtkWidget *pBtn)
{
	double zoom;

	if (pBtn==GTK_WIDGET(m_ShowChart)) {
		m_DisplayMode = DISPLAY_CHART;
		CConfig::SetInt("ChooseStarsDlg", "Mode", m_DisplayMode);
		cmpack_chart_view_set_image(CMPACK_CHART_VIEW(m_Chart), NULL);
		UpdateChart();
	} else
	if (pBtn==GTK_WIDGET(m_ShowImage)) {
		m_DisplayMode = DISPLAY_IMAGE;
		CConfig::SetInt("ChooseStarsDlg", "Mode", m_DisplayMode);
		cmpack_chart_view_set_image(CMPACK_CHART_VIEW(m_Chart), m_ImageData);
		UpdateChart();
	} else
	if (pBtn==GTK_WIDGET(m_ShowMixed)) {
		m_DisplayMode = DISPLAY_FULL;
		CConfig::SetInt("ChooseStarsDlg", "Mode", m_DisplayMode);
		cmpack_chart_view_set_image(CMPACK_CHART_VIEW(m_Chart), m_ImageData);
		UpdateChart();
	} else
	if (pBtn==GTK_WIDGET(m_ClearBtn)) {
		SetSelection(CSelection());
		SetTags(CTags());
	} else
	if (pBtn==GTK_WIDGET(m_RestoreBtn)) {
		RestoreSelection();
	} else
	if (pBtn==GTK_WIDGET(m_ZoomIn)) {
		zoom = cmpack_chart_view_get_zoom(CMPACK_CHART_VIEW(m_Chart));
		cmpack_chart_view_set_zoom(CMPACK_CHART_VIEW(m_Chart), zoom + 5.0);
	} else 
	if (pBtn==GTK_WIDGET(m_ZoomOut)) {
		zoom = cmpack_chart_view_get_zoom(CMPACK_CHART_VIEW(m_Chart));
		cmpack_chart_view_set_zoom(CMPACK_CHART_VIEW(m_Chart), zoom - 5.0);
	} else 
	if (pBtn==GTK_WIDGET(m_ZoomFit)) {
		cmpack_chart_view_set_auto_zoom(CMPACK_CHART_VIEW(m_Chart), TRUE);
	}
}

bool CChooseStarsDlg::OnCloseQuery()
{
	if (!m_InstMagnitudes) {
		if (m_Selection.CountStars(CMPACK_SELECT_VAR)==0) {
			ShowError(GTK_WINDOW(m_pDlg), "Please, select a variable star.");
			return false;
		}
		if (m_Selection.CountStars(CMPACK_SELECT_VAR)>1) {
			ShowError(GTK_WINDOW(m_pDlg), "It is not allowed to select more than one variable star.");
			return false;
		}
		if (m_Selection.CountStars(CMPACK_SELECT_COMP)==0) {
			ShowError(GTK_WINDOW(m_pDlg), "Please, select a comparison star.");
			return false;
		} 
		if (m_SingleComparison && m_Selection.CountStars(CMPACK_SELECT_COMP)>1) {
			ShowError(GTK_WINDOW(m_pDlg), "It is not allowed to select more than comparison star.");
			return false;
		}
	}
	return true;
}

bool CChooseStarsDlg::Execute(CSelection &sel, CTags &tags, bool inst_mag)
{
	int	res;
	gchar *pht_file, *fts_file;
	const gchar *tmp_file;
	GtkTreePath *refpath;

	m_Selection = sel;
	m_Tags = tags;
	m_InstMagnitudes = inst_mag;
	m_SingleComparison = !g_Project->Profile()->GetBool(CProfile::MULTIPLE_COMP_STARS);

	cmpack_chart_view_set_model(CMPACK_CHART_VIEW(m_Chart), NULL);
	cmpack_chart_view_set_image(CMPACK_CHART_VIEW(m_Chart), NULL);
	cmpack_chart_data_clear(m_ChartData);
	if (m_ImageData) {
		g_object_unref(m_ImageData);
		m_ImageData = NULL;
	}
	m_Phot.Close();
	m_Catalog.Close();
	m_Image.Clear();
	m_ApertureIndex = 0;

	m_RefType = g_Project->GetReferenceType();
	switch(m_RefType)
	{
	case REF_FRAME:
		refpath = g_Project->GetReferencePath();
		if (refpath) {
			pht_file = g_Project->GetPhotFile(refpath);
			if (m_Phot.Open(pht_file, CMPACK_OPEN_READONLY, 0, &res)) {
				UpdateChart();
				fts_file = g_Project->GetImageFile(refpath);
				if (m_Image.Load(fts_file, CMPACK_BITPIX_AUTO, &res)) 
					UpdateImage();
				g_free(fts_file);
			}
			g_free(pht_file);
			gtk_tree_path_free(refpath);
		}
		break;

	case REF_CATALOG_FILE:
		// Load catalog file
		tmp_file = g_Project->GetTempCatFile()->FullPath();
		if (m_Catalog.Open(tmp_file, CMPACK_OPEN_READONLY, &res)) {
			UpdateChart();
			fts_file = SetFileExtension(tmp_file, FILE_EXTENSION_FITS);
			if (m_Image.Load(fts_file, CMPACK_BITPIX_AUTO, &res)) 
				UpdateImage();
			g_free(fts_file);
		}
		break;

	default:
		return false;
	}

	gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(m_ShowChart),
		m_DisplayMode == DISPLAY_CHART);
	gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(m_ShowImage),
		m_DisplayMode == DISPLAY_IMAGE);
	gtk_toggle_tool_button_set_active(GTK_TOGGLE_TOOL_BUTTON(m_ShowMixed),
		m_DisplayMode == DISPLAY_FULL);
	gtk_widget_set_sensitive(GTK_WIDGET(m_ShowImage), 
		m_Image.Width()>0 && m_Image.Height()>0);
	gtk_widget_set_sensitive(GTK_WIDGET(m_ShowMixed), 
		m_Image.Width()>0 && m_Image.Height()>0);

	gtk_widget_set_sensitive(GTK_WIDGET(m_RestoreBtn), 
		g_Project->Selection()->Count()>0 ||
		g_Project->RefSelection()->Count()>0);

	if (gtk_dialog_run(GTK_DIALOG(m_pDlg)) == GTK_RESPONSE_ACCEPT) {
		sel = m_Selection;
		tags = m_Tags;
		return true;
	}
	return false;
}

void CChooseStarsDlg::UpdateImage(void)
{
	bool image_invert = CConfig::GetBool(CConfig::NEGATIVE_CHARTS);

	cmpack_chart_view_set_image(CMPACK_CHART_VIEW(m_Chart), NULL);
	if (m_ImageData)
		g_object_unref(m_ImageData);
	m_ImageData = m_Image.ToImageData(image_invert, false, false);
	if (m_DisplayMode != DISPLAY_CHART) 
		cmpack_chart_view_set_image(CMPACK_CHART_VIEW(m_Chart), m_ImageData);
}

//
// Update chart
//
void CChooseStarsDlg::UpdateChart(void)
{
	cmpack_chart_view_set_model(CMPACK_CHART_VIEW(m_Chart), NULL);
	cmpack_chart_data_clear(m_ChartData);
	if (m_RefType==REF_FRAME)
		m_Phot.Render(m_ChartData, m_ApertureIndex, false, m_DisplayMode==DISPLAY_IMAGE);
	else
		m_Catalog.Render(m_ChartData, false, m_DisplayMode==DISPLAY_IMAGE);

	int count = cmpack_chart_data_count(m_ChartData);
	for (int row=0; row<count; row++) {
		int star_id = cmpack_chart_data_get_param(m_ChartData, row);
		if (m_Selection.IndexOf(star_id)>=0 || m_Tags.Get(star_id)) 
			UpdateObject(row, star_id);
	}

	bool chart_invert = CConfig::GetBool(CConfig::NEGATIVE_CHARTS);
	cmpack_chart_view_set_negative(CMPACK_CHART_VIEW(m_Chart), chart_invert);
	cmpack_chart_view_set_model(CMPACK_CHART_VIEW(m_Chart), m_ChartData);
}

//
// Update displayed object
//
void CChooseStarsDlg::UpdateObject(int row, int star_id) 
{
	int i = m_Selection.IndexOf(star_id);
	const gchar *tag = m_Tags.Get(star_id);
	if (i>=0) {
		// Selected object
		int index = m_Selection.GetIndex(i);
		CmpackSelectionType type = m_Selection.GetType(i);
		gchar *buf = (gchar*)g_malloc((256+(tag ? strlen(tag)+1 : 0))*sizeof(gchar));
		if (index==1)
			strcpy(buf, Types[type].label);
		else
			sprintf(buf, "%s #%d", Types[type].label, index);
		if (tag) {
			strcat(buf, "\n");
			strcat(buf, tag);
		}
		cmpack_chart_data_set_tag(m_ChartData, row, buf);
		cmpack_chart_data_set_color(m_ChartData, row, Types[type].color);
		cmpack_chart_data_set_topmost(m_ChartData, row, TRUE);
		if (m_DisplayMode==DISPLAY_IMAGE)
			cmpack_chart_data_set_diameter(m_ChartData, row, 4.0);
		g_free(buf);
	} else 
	if (tag) {
		// Not selected, with tag
		cmpack_chart_data_set_tag(m_ChartData, row, tag);
		cmpack_chart_data_set_color(m_ChartData, row, CMPACK_COLOR_YELLOW);
		cmpack_chart_data_set_topmost(m_ChartData, row, TRUE);
		if (m_DisplayMode==DISPLAY_IMAGE)
			cmpack_chart_data_set_diameter(m_ChartData, row, 4.0);
	} else {
		// Not selected, no tag
		cmpack_chart_data_set_tag(m_ChartData, row, NULL);
		cmpack_chart_data_set_color(m_ChartData, row, CMPACK_COLOR_DEFAULT);
		cmpack_chart_data_set_topmost(m_ChartData, row, FALSE);
		if (m_DisplayMode==DISPLAY_IMAGE)
			cmpack_chart_data_set_diameter(m_ChartData, row, 0);
	}
}

void CChooseStarsDlg::response_dialog(GtkDialog *pDlg, gint response_id, CChooseStarsDlg *pMe)
{
	if (!pMe->OnResponseDialog(response_id)) 
		g_signal_stop_emission_by_name(pDlg, "response");
}

bool CChooseStarsDlg::OnResponseDialog(gint response_id)
{
	switch (response_id)
	{
	case GTK_RESPONSE_ACCEPT:
		// Check input
		if (!OnCloseQuery())
			return false;
		break;

	case GTK_RESPONSE_HELP:
		// Show context help
		g_MainWnd->ShowHelp(GTK_WINDOW(m_pDlg), IDH_CHOOSE_STARS);
		return false;
	}
	return true;
}

void CChooseStarsDlg::RestoreSelection(void)
{
	GtkWidget *pDlg, *frame, *fbox, *last, *catfile;
	GSList *group;
	
	if (g_Project->Selection()->Count()==0 &&
		g_Project->RefSelection()->Count()==0)
		return;

	if (g_Project->Selection()->Count()>0 &&
		g_Project->RefSelection()->Count()==0) {
		SetSelection(*g_Project->Selection());
		SetTags(*g_Project->Tags());
		return;
	}
	if (g_Project->Selection()->Count()==0 &&
		g_Project->RefSelection()->Count()>0) {
		SetSelection(*g_Project->RefSelection());
		SetTags(*g_Project->RefTags());
		return;
	}

	// Dialog with buttons
	pDlg = gtk_dialog_new_with_buttons("Restore selection", GTK_WINDOW(m_pDlg), 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR),
		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
	gtk_dialog_widget_standard_tooltips(GTK_DIALOG(pDlg));

	// Sidebar
	frame = gtk_frame_new("Show");
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pDlg)->vbox), frame, FALSE, FALSE, 0);
	fbox = gtk_vbox_new(FALSE, 8);
	gtk_container_set_border_width(GTK_CONTAINER(fbox), 4);
	gtk_container_add(GTK_CONTAINER(frame), fbox);
	last = gtk_radio_button_new_with_label(NULL, "last selection used");
	gtk_widget_set_sensitive(last, g_Project->Selection()->Count()>0);
	gtk_box_pack_start(GTK_BOX(fbox), last, FALSE, TRUE, 0);
	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(last));
	catfile = gtk_radio_button_new_with_label(group, "stored in the reference file");
	gtk_widget_set_sensitive(catfile, g_Project->RefSelection()->Count()>0);
	gtk_box_pack_start(GTK_BOX(fbox), catfile, FALSE, TRUE, 0);

	if (CConfig::GetInt("StarsDlg", "RestoreMode", 0)==0) {
		if (g_Project->Selection()->Count()>0)
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(last), true);
		else if (g_Project->RefSelection()->Count()>0)
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(catfile), true);
	} else {
		if (g_Project->RefSelection()->Count()>0)
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(catfile), true);
		else if (g_Project->Selection()->Count()>0)
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(last), true);
	}

	gtk_widget_show_all(frame);

	if (gtk_dialog_run(GTK_DIALOG(pDlg))==GTK_RESPONSE_ACCEPT) {
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(last))) {
			CConfig::GetInt("StarsDlg", "RestoreMode", 0);
			SetSelection(*g_Project->Selection());
			SetTags(*g_Project->Tags());
		} else
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(catfile))) {
			CConfig::GetInt("StarsDlg", "RestoreMode", 1);
			SetSelection(*g_Project->RefSelection());
			SetTags(*g_Project->RefTags());
		}
	}

	gtk_widget_destroy(pDlg);
}

//-------------------------   TEXT QUERY DIALOG   ----------------------------

CTextQueryDlg::CTextQueryDlg(GtkWindow *pParent, const gchar *caption)
{
	// Dialog with buttons
	m_pDlg = gtk_dialog_new_with_buttons(caption, pParent, 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR),
		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
	gtk_dialog_widget_standard_tooltips(GTK_DIALOG(m_pDlg));

	// Dialog icon
	gchar *icon = get_icon_file("muniwin");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	GtkWidget *vbox = gtk_vbox_new(FALSE, 4);
	gtk_widget_set_size_request(vbox, 360, -1);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(m_pDlg)->vbox), vbox, FALSE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 4);

	m_Query = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(m_Query), 0, 0.5);
	gtk_box_pack_start(GTK_BOX(vbox), m_Query, TRUE, TRUE, 0);
	m_Entry = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(vbox), m_Entry, TRUE, TRUE, 0);

	gtk_widget_show_all(GTK_DIALOG(m_pDlg)->vbox);
}

CTextQueryDlg::~CTextQueryDlg()
{
	gtk_widget_destroy(m_pDlg);
}

gchar *CTextQueryDlg::Execute(const gchar *query, int maxsize, const gchar *defval)
{
	gtk_label_set_text(GTK_LABEL(m_Query), query);
	gtk_entry_set_text(GTK_ENTRY(m_Entry), (defval ? defval : ""));
	gtk_entry_set_max_length(GTK_ENTRY(m_Entry), maxsize);
	if (gtk_dialog_run(GTK_DIALOG(m_pDlg))==GTK_RESPONSE_ACCEPT)
		return g_strdup(gtk_entry_get_text(GTK_ENTRY(m_Entry)));
	return NULL;	
}
