/**************************************************************

stars_dlg.h (C-Munipack project)
The 'Choose stars' dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CMPACK_STARS_DLG_H
#define CMPACK_STARS_DLG_H

#include <gtk/gtk.h>

#include "cmpack_widgets.h"
#include "popup.h"
#include "project.h"
#include "lightcurve_dlg.h"
#include "image_class.h"
#include "phot_class.h"
#include "catalog_class.h"

class CChooseStarsDlg
{
public:
	// Constructor
	CChooseStarsDlg(GtkWindow *pParent);

	// Destructor
	~CChooseStarsDlg();

	// Execute the dialog
	bool Execute(CSelection &sel, CTags &tags, bool inst_mag);

private:
	// Display modes
	enum tDisplayMode {
		DISPLAY_IMAGE,
		DISPLAY_CHART,
		DISPLAY_FULL
	};

	GtkWidget		*m_pDlg, *m_Chart, *m_ViewLabel;
	GtkToolItem		*m_ShowImage, *m_ShowChart, *m_ShowMixed;
	GtkToolItem		*m_ClearBtn, *m_RestoreBtn;
	GtkToolItem		*m_ZoomIn, *m_ZoomOut, *m_ZoomFit;
	tDisplayMode	m_DisplayMode;
	CSelection		m_Selection;
	CTags			m_Tags;
	CPhot			m_Phot;
	CCatalog		m_Catalog;
	CImage			m_Image;
	CmpackImageData	*m_ImageData;
	CmpackChartData	*m_ChartData;
	CPopupMenu		m_SelectMenu, m_ContextMenu;
	tReferenceType	m_RefType;
	int				m_ApertureIndex;
	bool			m_InstMagnitudes, m_SingleComparison;

	void UpdateImage(void);
	void UpdateChart(void);
	void RestoreSelection(void);
	void Select(int row, int star_id, CmpackSelectionType type);
	void Unselect(int row, int star_id);
	void UnselectType(CmpackSelectionType type);
	void SetSelection(const CSelection &sel);
	void SetTags(const CTags &tags);
	void ClearSelection(void);
	void EditTag(int row, int star_id);
	void SetTag(int row, int star_id, const char *tag);
	void RemoveTag(int row, int star_id);
	void ClearTags(void);

	void UpdateObject(int row, int star_id);

	void OnButtonClicked(GtkWidget *pButton);
	void OnSelectMenu(GdkEventButton *event, gint item);
	void OnContextMenu(GdkEventButton *event);
	bool OnResponseDialog(gint response_id);
	bool OnCloseQuery(void);

	static void response_dialog(GtkDialog *pDlg, gint response_id, CChooseStarsDlg *pMe);
	static void button_clicked(GtkWidget *pButton, CChooseStarsDlg *pDlg);
	static void item_activated(GtkWidget *pChart, gint item, CChooseStarsDlg *pMe);
	static gint button_press_event(GtkWidget *widget, GdkEventButton *event, CChooseStarsDlg *pMe);
};

class CTextQueryDlg
{
public:
	// Constructor
	CTextQueryDlg(GtkWindow *pParent, const gchar *caption);

	// Destructor
	virtual ~CTextQueryDlg();

	// Execute the dialog
	gchar *Execute(const gchar *query, int maxsize, const gchar *defval);

private:
	GtkWidget	*m_pDlg, *m_Query, *m_Entry;
};

#endif
