/**************************************************************

ccd_image_size.h (C-Munipack project)
Widget that allows an user to define a size of exported image
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CMPACK_IMAGE_SIZE_H
#define CMPACK_IMAGE_SIZE_H

#include <gtk/gtk.h>

#include "callback.h"
#include "helper_classes.h"

class CImageSizeWidget:public CCBObject
{
public:
	enum tMessageId
	{
		CB_TYPE_CHANGED
	};

	enum tSizeMode
	{
		ABSOLUTE, RELATIVE
	};

	CImageSizeWidget();

	virtual ~CImageSizeWidget();

	GtkWidget *Handle(void)
	{ return m_Frame; }

	void Init(int img_width, int img_height, tSizeMode mode, 
		tImageFile type, double zoom, int dst_width, int jpeg_quality);

	int SourceWidth(void) const
	{ return m_ChartWidth; }

	int SourceHeight(void) const
	{ return m_ChartHeight; }

	tSizeMode Mode(void) const
	{ return m_Mode; }

	tImageFile Type(void) const
	{ return m_Type; }

	double Zoom(void);

	int OutputWidth(void);

	int OutputHeight(void);

	int JpegQuality(void) const;

private:
	GtkWidget		*m_Relative, *m_Absolute, *m_Frame, *m_TypeCombo;
	GtkWidget		*m_ZoomAdj, *m_AbsHeight, *m_AbsWidth, *m_Percents;
	GtkWidget		*m_HeightLabel, *m_Pixels, *m_JpegQLabel, *m_JpegQuality;
	GtkListStore	*m_FileTypes;
	tImageFile		m_Type;
	tSizeMode		m_Mode;
	int				m_ChartWidth, m_ChartHeight;
	int				m_ImageWidth, m_ImageHeight;
	bool			m_Changed, m_Updating;
	double			m_Zoom, m_ZoomMin, m_ZoomMax;

	void SetMode(tSizeMode mode);
	void OnValueChanged(GtkSpinButton *pButton);
	void OnSelectionChanged(GtkComboBox *pWidget);
	void OnToggled(GtkToggleButton *pButton);
	void UpdateData(void);

	static void value_changed(GtkSpinButton *spinbutton, CImageSizeWidget *pMe);
	static void selection_changed(GtkComboBox *pWidget, CImageSizeWidget *pMe);
	static void toggled(GtkToggleButton *togglebutton, CImageSizeWidget *pMe);
};

class CGraphSizeWidget:public CCBObject
{
public:
	enum tMessageId
	{
		CB_TYPE_CHANGED
	};

	CGraphSizeWidget();

	virtual ~CGraphSizeWidget();

	GtkWidget *Handle(void)
	{ return m_Frame; }

	void Init(tImageFile type, int jpeg_quality);

	tImageFile Type(void) const
	{ return m_Type; }

	int JpegQuality(void) const;

private:
	GtkWidget		*m_Frame, *m_TypeCombo, *m_JpegQLabel, *m_JpegQuality;
	GtkListStore	*m_FileTypes;
	tImageFile		m_Type;
	bool			m_Changed, m_Updating;

	void OnSelectionChanged(GtkComboBox *pWidget);

	static void selection_changed(GtkComboBox *pWidget, CGraphSizeWidget *pMe);
};

#endif
