/**************************************************************

newproject_dlg.cpp (C-Munipack project)
New project dialog
Copyright (C) 2012 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>

#include "loadprofile_dlg.h"
#include "utils.h"
#include "main.h"
#include "ctxhelp.h"
#include "configuration.h"

enum tColumnId
{
	COL_DISPLAY_NAME,
	COL_PATH,
	NCOLS
};

static gboolean has_group(const GtkRecentFilterInfo *filter_info, const gchar *group)
{
	if (filter_info->groups) {
		for (int i=0; filter_info->groups[i]; i++) {
			if (strcmp(filter_info->groups[i], group) == 0)
				return TRUE;
		}
	}
	return FALSE;
}

//---------------------------   LOAD PROFILE DIALOG   ---------------------------------

CLoadProfileDlg::CLoadProfileDlg(GtkWindow *pParent, tProjectType type, bool no_default):m_Type(type), 
	m_Updating(false)
{
	GtkWidget *vbox, *fbox;
	GSList *group;

	m_FileList = gtk_list_store_new(NCOLS, G_TYPE_STRING, G_TYPE_STRING);
	m_SelectMode = (tSelectMode)CConfig::GetInt("ChooseProfileDlg", "Mode", PROJECT);
	if (no_default && m_SelectMode==DEFAULTS)
		m_SelectMode = PROJECT;

	// Dialog with buttons
	m_pDlg = gtk_dialog_new_with_buttons("Import project settings", pParent, 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, 
		GTK_STOCK_HELP, GTK_RESPONSE_HELP, NULL);
	gtk_dialog_widget_standard_tooltips(GTK_DIALOG(m_pDlg));
	gtk_dialog_set_tooltip_by_response(GTK_DIALOG(m_pDlg), GTK_RESPONSE_ACCEPT, "Load project settings from a specified file");
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);

	// Dialog layout
	vbox = gtk_vbox_new(FALSE, 8);
	gtk_widget_set_size_request(vbox, 500, -1);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 4);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(m_pDlg)->vbox), vbox, TRUE, TRUE, 0);

	// Data source selection
	m_ProjectBtn = gtk_radio_button_new_with_label(NULL, "Import settings from an existing project");
	gtk_widget_set_tooltip_text(m_ProjectBtn, "Load settings from an existing project");
	g_signal_connect(G_OBJECT(m_ProjectBtn), "toggled", G_CALLBACK(button_clicked), this);
	gtk_box_pack_start(GTK_BOX(vbox), m_ProjectBtn, FALSE, TRUE, 0);
	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(m_ProjectBtn));
	m_ProfileBtn = gtk_radio_button_new_with_label(group, "Load settings from a profile");
	gtk_widget_set_tooltip_text(m_ProfileBtn, "Load settings from a profile");
	g_signal_connect(G_OBJECT(m_ProfileBtn), "toggled", G_CALLBACK(button_clicked), this);
	gtk_box_pack_start(GTK_BOX(vbox), m_ProfileBtn, FALSE, TRUE, 0);
	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(m_ProjectBtn)); 
	m_DefaultsBtn = gtk_radio_button_new_with_label(group, "Use factory defaults");
	gtk_widget_set_tooltip_text(m_DefaultsBtn, "Reset all project settings to factory defaults");
	g_signal_connect(G_OBJECT(m_DefaultsBtn), "toggled", G_CALLBACK(button_clicked), this);
	gtk_box_pack_start(GTK_BOX(vbox), m_DefaultsBtn, FALSE, TRUE, 0);

	// Data source path
	m_FileBox = gtk_hbox_new(FALSE, 8);
	gtk_box_pack_start(GTK_BOX(vbox), m_FileBox, FALSE, TRUE, 0);
	m_SrcLabel = gtk_label_new("Load from:");
	gtk_box_pack_start(GTK_BOX(m_FileBox), m_SrcLabel, FALSE, TRUE, 0);
	m_SrcEntry = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_SrcEntry, "Path to the file which the settings shall be loaded from");
	gtk_editable_set_editable(GTK_EDITABLE(m_SrcEntry), false);
	gtk_box_pack_start(GTK_BOX(m_FileBox), m_SrcEntry, TRUE, TRUE, 0);
	m_BrowseBtn = gtk_button_new_with_label("Browse");
	gtk_widget_set_tooltip_text(m_BrowseBtn, "Browse for file in a separate dialog");
	g_signal_connect(G_OBJECT(m_BrowseBtn), "clicked", G_CALLBACK(button_clicked), this);
	gtk_box_pack_start(GTK_BOX(m_FileBox), m_BrowseBtn, FALSE, TRUE, 0);

	// Profiles
	m_TreeView = gtk_tree_view_new();
	GtkTreeViewColumn *col = gtk_tree_view_column_new();
	gtk_tree_view_append_column(GTK_TREE_VIEW(m_TreeView), col);
	GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col, renderer, "text", 0);
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(m_TreeView), FALSE);
	m_ListBox = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(m_ListBox),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(m_ListBox), 
		GTK_SHADOW_ETCHED_IN);
	gtk_container_add(GTK_CONTAINER(m_ListBox), m_TreeView);
	gtk_widget_set_size_request(m_TreeView, -1, 200);
	gtk_box_pack_start(GTK_BOX(vbox), m_ListBox, TRUE, TRUE, 0);
	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(m_TreeView));
	gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);
	g_signal_connect(G_OBJECT(selection), "changed", G_CALLBACK(tv_selection_changed), this);

	// Recent projects and profiles
	m_RecentList = gtk_recent_chooser_widget_new();
	gtk_recent_chooser_set_sort_type(GTK_RECENT_CHOOSER(m_RecentList), GTK_RECENT_SORT_MRU);
	gtk_recent_chooser_set_show_private(GTK_RECENT_CHOOSER(m_RecentList), TRUE);
	gtk_recent_chooser_set_local_only(GTK_RECENT_CHOOSER(m_RecentList), TRUE);
	gtk_recent_chooser_set_show_not_found(GTK_RECENT_CHOOSER(m_RecentList), FALSE);
	gtk_widget_set_size_request(m_RecentList, -1, 200);
	gtk_box_pack_start(GTK_BOX(vbox), m_RecentList, TRUE, TRUE, 0);
	g_signal_connect(G_OBJECT(m_RecentList), "selection-changed", G_CALLBACK(rc_selection_changed), this);

	// Search path
	m_SearchBox = gtk_hbox_new(FALSE, 8);
	gtk_box_pack_start(GTK_BOX(vbox), m_SearchBox, FALSE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(m_SearchBox), gtk_label_new("Search path:"), FALSE, TRUE, 0);
	m_PathEntry = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_PathEntry, "Path to the folder with profiles");
	gtk_editable_set_editable(GTK_EDITABLE(m_PathEntry), FALSE);
	gtk_box_pack_start(GTK_BOX(m_SearchBox), m_PathEntry, TRUE, TRUE, 0);
	m_PathBtn = gtk_button_new_with_label("Change folder");
	gtk_widget_set_tooltip_text(m_PathBtn, "Change folder with profiles");
	g_signal_connect(G_OBJECT(m_PathBtn), "clicked", G_CALLBACK(button_clicked), this);
	gtk_box_pack_start(GTK_BOX(m_SearchBox), m_PathBtn, FALSE, TRUE, 0);

	// Factory defaults options
	m_OptionsBox = gtk_table_new(1, 2, FALSE);
	gtk_widget_set_size_request(m_OptionsBox, -1, 240);
	gtk_table_set_col_spacings(GTK_TABLE(m_OptionsBox), 24);
	gtk_box_pack_start(GTK_BOX(vbox), m_OptionsBox, TRUE, TRUE, 0);
	fbox = gtk_vbox_new(FALSE, 8);
	gtk_table_attach_defaults(GTK_TABLE(m_OptionsBox), fbox, 1, 2, 0, 1);

	gtk_widget_show_all(GTK_DIALOG(m_pDlg)->vbox);
	if (no_default)
		gtk_widget_hide(m_DefaultsBtn);
}

CLoadProfileDlg::~CLoadProfileDlg()
{
	gtk_widget_destroy(m_pDlg);
	g_object_unref(m_FileList);
}

bool CLoadProfileDlg::Execute(CProfile &profile)
{
	m_Profile.Clear();

	// Path to directory with profiles
	gchar *fpath = CConfig::GetStr("ProfileDlg", "Path", NULL);
	if (fpath && g_file_test(fpath, G_FILE_TEST_EXISTS)) 
		gtk_entry_set_text(GTK_ENTRY(m_PathEntry), fpath);
	else {
		gchar *defpath = g_build_filename(get_user_data_dir(), "Profiles", NULL);
		if (force_directory(defpath))
			gtk_entry_set_text(GTK_ENTRY(m_PathEntry), defpath);
		g_free(defpath);
	}
	g_free(fpath);

	SetSelectMode(m_SelectMode);

	// Run the dialog
	if (gtk_dialog_run(GTK_DIALOG(m_pDlg))!=GTK_RESPONSE_ACCEPT)
		return false;
	gtk_widget_hide(m_pDlg);

	CConfig::SetInt("ChooseProfileDlg", "Mode", m_SelectMode);
	profile = m_Profile;
	profile.SetProjectType(m_Type);
	return true;
}

void CLoadProfileDlg::response_dialog(GtkWidget *pDlg, gint response_id, CLoadProfileDlg *pMe)
{
	if (!pMe->OnResponseDialog(response_id)) 
		g_signal_stop_emission_by_name(pDlg, "response");
}

bool CLoadProfileDlg::OnResponseDialog(gint response_id)
{
	switch (response_id)
	{
	case GTK_RESPONSE_ACCEPT:
		// Commit changes
		return OnCloseQuery();

	case GTK_RESPONSE_HELP:
		// Show context help
		g_MainWnd->ShowHelp(GTK_WINDOW(m_pDlg), IDH_LOAD_PROFILE);
		return false;
	}
	return true;
}

void CLoadProfileDlg::SetSelectMode(tSelectMode mode)
{
	if (!m_Updating) {
		m_Updating = true;
		m_SelectMode = mode;
		switch (mode)
		{
		case PROFILE:
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_ProfileBtn), TRUE);
			gtk_widget_hide(m_OptionsBox);
			gtk_widget_hide(m_RecentList);
			gtk_widget_show(m_ListBox);
			gtk_widget_show(m_FileBox);
			gtk_widget_show(m_SearchBox);
			UpdateProfiles();
			break;

		case PROJECT:
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_ProjectBtn), TRUE);
			gtk_widget_hide(m_OptionsBox);
			gtk_widget_hide(m_ListBox);
			gtk_widget_hide(m_SearchBox);
			gtk_widget_show(m_RecentList);
			gtk_widget_show(m_FileBox);
			UpdateProjects();
			break;

		case DEFAULTS:
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_DefaultsBtn), TRUE);
			gtk_widget_hide(m_SearchBox);
			gtk_widget_hide(m_FileBox);
			gtk_widget_hide(m_ListBox);
			gtk_widget_hide(m_RecentList);
			gtk_widget_show(m_OptionsBox);
			break;
		}
		m_Updating = false;
	}
}

bool CLoadProfileDlg::OnCloseQuery(void)
{
	bool retval = true;
	GError *error = NULL;

	m_Profile.Clear();
	
	if (m_SelectMode == PROFILE) {
		// Load settings from a profile
		const gchar *fpath = gtk_entry_get_text(GTK_ENTRY(m_SrcEntry));
		if (!fpath[0]) {
			ShowError(GTK_WINDOW(m_pDlg), "Please, enter name of a profile file.");
			return false;
		}
		retval = m_Profile.Import(fpath, &error)!=FALSE;
		if (error) {
			ShowError(GTK_WINDOW(m_pDlg), error->message);
			g_error_free(error);
		}
		if (retval) {
			if (CProfile::isProfileFile(fpath, NULL))
				CConfig::AddProfileToRecentList(m_Profile.ProjectType(), fpath);
			else if (CProject::isProjectFile(fpath, NULL))
				CConfig::AddProjectToRecentList(m_Profile.ProjectType(), fpath);
		}
	}
	if (m_SelectMode == PROJECT) {
		// Import settings from a project
		const gchar *fpath = gtk_entry_get_text(GTK_ENTRY(m_SrcEntry));
		if (!fpath[0]) {
			ShowError(GTK_WINDOW(m_pDlg), "Please, enter name of a project file.");
			return false;
		}
		retval = m_Profile.Import(fpath, &error)!=FALSE;
		if (error) {
			ShowError(GTK_WINDOW(m_pDlg), error->message);
			g_error_free(error);
		}
		if (retval) {
			if (CProfile::isProfileFile(fpath, NULL))
				CConfig::AddProfileToRecentList(m_Profile.ProjectType(), fpath);
			else if (CProject::isProjectFile(fpath, NULL))
				CConfig::AddProjectToRecentList(m_Profile.ProjectType(), fpath);
		}
	}
	return retval;
}

void CLoadProfileDlg::tv_selection_changed(GtkTreeSelection *widget, CLoadProfileDlg *pDlg)
{
	pDlg->OnSelectionChanged(widget);
}

void CLoadProfileDlg::OnSelectionChanged(GtkTreeSelection *widget)
{
	gchar *fname;
	GtkTreeModel *model;
	GtkTreeIter iter;

	GList *list = gtk_tree_selection_get_selected_rows(widget, &model);
	if (list && model) {
		if (gtk_tree_model_get_iter(model, &iter, (GtkTreePath*)(list->data))) {
			gtk_tree_model_get(model, &iter, COL_PATH, &fname, -1);
			gtk_entry_set_text(GTK_ENTRY(m_SrcEntry), fname);
			g_free(fname);
		}
		g_list_foreach(list, (GFunc)gtk_tree_path_free, NULL);
		g_list_free(list);
	}
}

void CLoadProfileDlg::rc_selection_changed(GtkRecentChooser *widget, CLoadProfileDlg *pDlg)
{
	pDlg->OnSelectionChanged(widget);
}

void CLoadProfileDlg::OnSelectionChanged(GtkRecentChooser *widget)
{
	gchar *uri = gtk_recent_chooser_get_current_uri(widget);
	if (uri) {
		gchar *fpath = g_filename_from_uri(uri, NULL, NULL);
		if (fpath) {
			gtk_entry_set_text(GTK_ENTRY(m_SrcEntry), fpath);
			g_free(fpath);
		}
		g_free(uri);
	}
}

void CLoadProfileDlg::button_clicked(GtkWidget *button, CLoadProfileDlg *pDlg)
{
	pDlg->OnButtonClicked(button);
}

void CLoadProfileDlg::OnButtonClicked(GtkWidget *pBtn)
{
	if (pBtn == m_ProfileBtn) {
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pBtn)))
			SetSelectMode(PROFILE);
	} else if (pBtn == m_ProjectBtn) {
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pBtn)))
			SetSelectMode(PROJECT);
	} else if (pBtn == m_DefaultsBtn) {
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pBtn)))
			SetSelectMode(DEFAULTS);
	} else if (pBtn == m_BrowseBtn) 
		BrowseForFile();
	else if (pBtn == m_PathBtn) 
		ChangeSearchPath();
}

void CLoadProfileDlg::ChangeSearchPath(void)
{
	const char *dirname = gtk_entry_get_text(GTK_ENTRY(m_PathEntry));

	GtkWidget *pPathDlg = gtk_file_chooser_dialog_new("Select path",
		GTK_WINDOW(m_pDlg), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, GTK_STOCK_CLOSE, 
		GTK_RESPONSE_REJECT, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_standard_tooltips(GTK_FILE_CHOOSER(pPathDlg));
	if (dirname && g_file_test(dirname, G_FILE_TEST_IS_DIR))
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(pPathDlg), dirname);
	if (gtk_dialog_run(GTK_DIALOG(pPathDlg)) == GTK_RESPONSE_ACCEPT) {
		char *path = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(pPathDlg));
		CConfig::SetStr("ProfileDlg", "Path", path);
		gtk_entry_set_text(GTK_ENTRY(m_PathEntry), path);
		g_free(path);
		UpdateProfiles();
	}
	gtk_widget_destroy(pPathDlg);
}

void CLoadProfileDlg::BrowseForFile(void)
{
	GtkWidget *pOpenDlg = gtk_file_chooser_dialog_new("Select a file", 
		GTK_WINDOW(m_pDlg), GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, 
		GTK_RESPONSE_REJECT, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_standard_tooltips(GTK_FILE_CHOOSER(pOpenDlg));
	const gchar *fpath = gtk_entry_get_text(GTK_ENTRY(m_SrcEntry));
	if (!gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(pOpenDlg), fpath)) {
		gchar *dir = g_path_get_dirname(fpath);
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(pOpenDlg), dir);
		g_free(dir);
	}
	if (gtk_dialog_run(GTK_DIALOG(pOpenDlg)) == GTK_RESPONSE_ACCEPT) {
		gchar *fpath = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(pOpenDlg));
		gtk_entry_set_text(GTK_ENTRY(m_SrcEntry), fpath);
		g_free(fpath);
	}
	gtk_widget_destroy(pOpenDlg);
}

void CLoadProfileDlg::UpdateProfiles(void)
{
	const char *dirpath = gtk_entry_get_text(GTK_ENTRY(m_PathEntry));
	GtkTreeIter iter;
	tProjectType type;

	gtk_tree_view_set_model(GTK_TREE_VIEW(m_TreeView), NULL);
	gtk_list_store_clear(m_FileList);
	GDir *dir = g_dir_open(dirpath, 0, NULL);
	if (dir) {
		const gchar *filename = g_dir_read_name(dir);
		while (filename) {
			if (CheckFileExtension(filename, FILE_EXTENSION_PROFILE)) {
				gchar *filepath = g_build_filename(dirpath, filename, NULL);
				if (CProfile::isProfileFile(filepath, &type) && type == m_Type) {
					gchar *name = StripFileExtension(filename);
					gtk_list_store_append(m_FileList, &iter);
					gtk_list_store_set(m_FileList, &iter, COL_DISPLAY_NAME, name, 
						COL_PATH, filepath, -1);
					g_free(name);
				}
				g_free(filepath);
			}
			filename = g_dir_read_name(dir);
		}
		g_dir_close(dir);
	}
	gtk_tree_view_set_model(GTK_TREE_VIEW(m_TreeView), GTK_TREE_MODEL(m_FileList));
}

void CLoadProfileDlg::UpdateProjects(void)
{
	GtkRecentFilter *filter = gtk_recent_filter_new();
	gtk_recent_filter_add_custom(filter, GTK_RECENT_FILTER_GROUP, (GtkRecentFilterFunc)recent_filter, this, 0);
	gtk_recent_chooser_set_filter(GTK_RECENT_CHOOSER(m_RecentList), filter);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_ProjectBtn), TRUE);
}

gboolean CLoadProfileDlg::recent_filter(const GtkRecentFilterInfo *filter_info, CLoadProfileDlg *pMe)
{
	return pMe->OnRecentFilter(filter_info);
}

gboolean CLoadProfileDlg::OnRecentFilter(const GtkRecentFilterInfo *filter_info)
{
	if (!has_group(filter_info, RECENT_GROUP_PROJECT))
		return FALSE;
	const gchar *group = ProjectTypeRecentGroup(m_Type);
	if (group && !has_group(filter_info, group))
		return FALSE;
	return TRUE;
}
