/**************************************************************

makecatalogfile_dlg.cpp (C-Munipack project)
The 'Convert files' dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <glib/gstdio.h>

#include "project.h"
#include "configuration.h"
#include "configuration.h"
#include "makecatalogfile_dlg.h"
#include "utils.h"
#include "main.h"
#include "object_dlg.h"
#include "observer_dlg.h"
#include "choosestars_dlg.h"
#include "ctxhelp.h"
#include "profile.h"

//-------------------------   PRIVATE DATA   ---------------------------

static const struct {
	const gchar *label;
	CmpackColor color;
} Types[CMPACK_SELECT_COUNT] = {
	{ NULL, CMPACK_COLOR_DEFAULT },
	{ "var", CMPACK_COLOR_RED },
	{ "comp", CMPACK_COLOR_GREEN },
	{ "check", CMPACK_COLOR_BLUE }
};

//-------------------------   MAIN WINDOW   --------------------------------

CMakeCatFileDlg::CMakeCatFileDlg(GtkWindow *pParent):m_ExportPath(NULL),
	m_ApertureIndex(-1), m_Error(NULL)
{
	GtkWidget *tbox, *frame, *hsep, *label, *scrwnd, *vbox, *tbar;

	// Dialog with buttons
	m_pDlg = gtk_dialog_new_with_buttons("Make catalog file", pParent, 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR),
		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, 
		GTK_STOCK_HELP, GTK_RESPONSE_HELP, NULL);
	gtk_dialog_widget_standard_tooltips(GTK_DIALOG(m_pDlg));
	gtk_dialog_set_tooltip_by_response(GTK_DIALOG(m_pDlg), GTK_RESPONSE_ACCEPT, "Create a new catalog file and close the dialog");
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);

	// Dialog icon
	gchar *icon = get_icon_file("catalogfile");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Path and file name
	tbox = gtk_table_new(2, 3, FALSE);
	gtk_container_set_border_width(GTK_CONTAINER(tbox), 4);
	gtk_table_set_row_spacings(GTK_TABLE(tbox), 4);
	gtk_table_set_col_spacings(GTK_TABLE(tbox), 4);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(m_pDlg)->vbox), tbox, FALSE, TRUE, 0);

	// File name
	label = gtk_label_new("File name");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
	m_FileName = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_FileName, "Name of the target file without extension (usually same as the object's designation)");
	gtk_table_attach(GTK_TABLE(tbox), m_FileName, 1, 2, 0, 1, (GtkAttachOptions)(GTK_FILL | GTK_EXPAND), GTK_FILL, 0, 0);

	// Folder
	label = gtk_label_new("Folder");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
	m_Path = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_Path, "Path to the directory where a new catalog file shall be saved to.");
	gtk_table_attach(GTK_TABLE(tbox), m_Path, 1, 2, 1, 2, (GtkAttachOptions)(GTK_FILL | GTK_EXPAND), GTK_FILL, 0, 0);
	m_PathBtn = gtk_button_new_with_label("Change folder");
	gtk_widget_set_tooltip_text(m_PathBtn, "Change the target folder");
	g_signal_connect(G_OBJECT(m_PathBtn), "clicked", G_CALLBACK(button_clicked), this);
	gtk_table_attach(GTK_TABLE(tbox), m_PathBtn, 2, 3, 1, 2, GTK_FILL, GTK_FILL, 0, 0);

	// Separator
	hsep = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(m_pDlg)->vbox), hsep, FALSE, TRUE, 4);

	// File header information
	tbox = gtk_table_new(15, 4, FALSE);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(m_pDlg)->vbox), tbox, TRUE, TRUE, 4);
	gtk_container_set_border_width(GTK_CONTAINER(tbox), 4);
	gtk_table_set_row_spacings(GTK_TABLE(tbox), 4);
	gtk_table_set_col_spacings(GTK_TABLE(tbox), 4);
	gtk_table_set_row_spacing(GTK_TABLE(tbox), 0, 12);
	gtk_table_set_row_spacing(GTK_TABLE(tbox), 3, 12);
	gtk_table_set_row_spacing(GTK_TABLE(tbox), 6, 12);
	gtk_table_set_col_spacing(GTK_TABLE(tbox), 2, 8);

	// Object
	label = gtk_label_new("Object - designation");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 0, 1, 1, 2);
	m_ObjName = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_ObjName, "Object's designation, e.g. RT And");
	gtk_entry_set_max_length(GTK_ENTRY(m_ObjName), MAX_OBJECT_LEN);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_ObjName, 1, 2, 1, 2);
	m_ObjBtn = gtk_button_new_with_label("More");
	gtk_widget_set_tooltip_text(m_ObjBtn, "Load object coordinates from presets or variable star catalogs");
	g_signal_connect(G_OBJECT(m_ObjBtn), "clicked", G_CALLBACK(button_clicked), this);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_ObjBtn, 2, 3, 1, 2);
	label = gtk_label_new("- right ascension");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 0, 1, 2, 3);
	m_RA = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_RA, "Object's right ascension in hours, minutes and seconds, e.g. 12 34 56");
	gtk_entry_set_max_length(GTK_ENTRY(m_RA), 64);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_RA, 1, 2, 2, 3);
	label = gtk_label_new("[h m s]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 2, 3, 2, 3);
	label = gtk_label_new("- declination");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 0, 1, 3, 4);
	m_Dec = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_Dec, "Object's declination in degrees, minutes and seconds, e.g. -12 34 56");
	gtk_entry_set_max_length(GTK_ENTRY(m_Dec), 64);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Dec, 1, 2, 3, 4);
	label = gtk_label_new("[\xC2\xB1""d m s]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 2, 3, 3, 4);

	// Location
	label = gtk_label_new("Location - name");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 0, 1, 4, 5);
	m_LocName = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_LocName, "Observer's location designation, e.g. Brno, Czech Republic");
	gtk_entry_set_max_length(GTK_ENTRY(m_LocName), MAX_LOCATION_LEN);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_LocName, 1, 2, 4, 5);
	m_LocBtn = gtk_button_new_with_label("More");
	gtk_widget_set_tooltip_text(m_LocBtn, "Load observer's coordinates from presets");
	g_signal_connect(G_OBJECT(m_LocBtn), "clicked", G_CALLBACK(button_clicked), this);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_LocBtn, 2, 3, 4, 5);
	label = gtk_label_new("- longitude");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 0, 1, 5, 6);
	m_Lon = gtk_entry_new_with_max_length(32);
	gtk_widget_set_tooltip_text(m_Lon, "Observer's longitude in degrees, minutes and seconds, use E or W suffix to indicate hemisphere, e.g. 12 34 56 E");
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Lon, 1, 2, 5, 6);
	label = gtk_label_new("[d m s E/W]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 2, 3, 5, 6);
	label = gtk_label_new("- latitude");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 0, 1, 6,7);
	m_Lat = gtk_entry_new_with_max_length(32);
	gtk_widget_set_tooltip_text(m_Lat, "Observer's latitude in degrees, minutes and seconds, use N or S suffix to indicate hemisphere, e.g. 56 34 12 N");
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Lat, 1, 2, 6, 7);
	label = gtk_label_new("[d m s N/S]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 2, 3, 6, 7);

	// Observer
	label = gtk_label_new("Observer's name");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 0, 1, 7, 8);
	m_Observer = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_Observer, "Observer's name");
	gtk_entry_set_max_length(GTK_ENTRY(m_Observer), MAX_OBSERVER_LEN);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Observer, 1, 3, 7, 8);

	// Equipment
	label = gtk_label_new("Telescope");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 0, 1, 8, 9);
	m_Telescope = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_Telescope, "Short description of the telescope");
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Telescope, 1, 3, 8, 9);
	label = gtk_label_new("CCD Camera");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 0, 1, 9, 10);
	m_Camera = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_Camera, "Camera brand name and model");
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Camera, 1, 3, 9, 10);
	label = gtk_label_new("Color filter");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 0, 1, 10, 11);
	m_Filter = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_Filter, "Color filter designation");
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Filter, 1, 3, 10, 11);
	label = gtk_label_new("Field of view");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 0, 1, 11, 12);
	m_FOV = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_FOV, "Size of view field, width x height");
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_FOV, 1, 3, 11, 12);
	label = gtk_label_new("Orientation");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), label, 0, 1, 12, 13);
	m_Orientation = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_Orientation, "View field orientation, e.g. top=N, left=W");
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Orientation, 1, 3, 12, 13);

	// Notes
	label = gtk_label_new("Remarks");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 4, 13, 14, GTK_FILL, GTK_FILL, 0, 0);
	m_Notes = gtk_text_view_new();
	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(m_Notes), GTK_WRAP_WORD_CHAR);
	scrwnd = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrwnd),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrwnd), 
		GTK_SHADOW_ETCHED_IN);
	gtk_container_add(GTK_CONTAINER(scrwnd), m_Notes);
	gtk_widget_set_size_request(scrwnd, -1, 140);
	gtk_table_attach_defaults(GTK_TABLE(tbox), scrwnd, 0, 4, 14, 15);

	// Catalog file preview
	vbox = gtk_vbox_new(FALSE, 0);
	gtk_table_attach_defaults(GTK_TABLE(tbox), vbox, 3, 4, 0, 13);
	gtk_widget_set_size_request(vbox, 360, -1);

	tbar = gtk_toolbar_new();
	gtk_toolbar_set_style(GTK_TOOLBAR(tbar), GTK_TOOLBAR_TEXT);
	m_EditSel = gtk_tool_button_new(NULL, "Edit selection");
	gtk_widget_set_tooltip_text(GTK_WIDGET(m_EditSel), "Change variable, comparison and check stars");
	gtk_toolbar_insert(GTK_TOOLBAR(tbar), m_EditSel, -1);
	g_signal_connect(G_OBJECT(m_EditSel), "clicked", G_CALLBACK(button_clicked), this);
	gtk_box_pack_start(GTK_BOX(vbox), tbar, FALSE, TRUE, 0);

	m_ChartData = cmpack_chart_data_new();
	m_Chart = cmpack_chart_view_new();
	frame = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(frame), m_Chart);
	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
	
	gtk_widget_show_all(GTK_DIALOG(m_pDlg)->vbox);
}

CMakeCatFileDlg::~CMakeCatFileDlg()
{
	gtk_widget_destroy(m_pDlg);
	g_object_unref(m_ChartData);
	g_free(m_ExportPath);
	g_error_free(m_Error);
}

void CMakeCatFileDlg::response_dialog(GtkDialog *pDlg, gint response_id, CMakeCatFileDlg *pMe)
{
	if (!pMe->OnResponseDialog(response_id))
		g_signal_stop_emission_by_name(pDlg, "response");
}

bool CMakeCatFileDlg::OnResponseDialog(gint response_id)
{
	switch (response_id)
	{
	case GTK_RESPONSE_ACCEPT:
		// Check input
		if (!OnCloseQuery())
			return false;
		break;

	case GTK_RESPONSE_HELP:
		// Show context help
		g_MainWnd->ShowHelp(GTK_WINDOW(m_pDlg), IDH_MAKE_CATALOG_FILE);
		return false;
	}
	return true;
}

void CMakeCatFileDlg::Execute()
{
	int res;
	char *pht_file;
	char *obsname, *telescope, *camera, *fov, *orient, *notes;
	const gchar *tmp_file;
	GtkWindow *pMainWnd = g_MainWnd->GetHandle();
	CmpackPhtInfo info;
	GtkTreePath *refpath;

	cmpack_chart_view_set_model(CMPACK_CHART_VIEW(m_Chart), NULL);
	cmpack_chart_data_clear(m_ChartData);
	m_Phot.Close();
	m_Catalog.Close();
	m_Object.Clear();
	m_Location.Clear();
	m_Selection = *g_Project->Selection();
	m_Tags = *g_Project->Tags();
	g_free(m_ExportPath);
	m_ExportPath = NULL;
	m_OutFile.Create();
	m_ApertureIndex = 0;

	m_RefType = g_Project->GetReferenceType();
	switch (m_RefType)
	{
	case REF_FRAME:
		refpath = g_Project->GetReferencePath();
		if (refpath) {
			pht_file = g_Project->GetPhotFile(refpath);
			if (m_Phot.Open(pht_file, CMPACK_OPEN_READONLY, 0, &res)) {
				UpdateChart();
				m_OutFile.Assign(m_Phot, m_ApertureIndex, NULL);
			}
			g_free(pht_file);
			gtk_tree_path_free(refpath);
		}
		break;

	case REF_CATALOG_FILE:
		// Load catalog file
		tmp_file = g_Project->GetTempCatFile()->FullPath();
		if (m_Catalog.Open(tmp_file, CMPACK_OPEN_READONLY, &res)) {
			UpdateChart();
			m_OutFile.Assign(m_Catalog, NULL);
		}
		break;

	default:
		ShowError(pMainWnd, "No reference file.");
		return;
	}

	// Selection of stars
	if (!ChooseStars()) 
		return;
	UpdateChart();

	// Restore last values
	if (g_Project->ObjectCoords()->Valid())
		m_Object = *g_Project->ObjectCoords();
	else if (g_Project->RefObjectCoords()->Valid())
		m_Object = *g_Project->RefObjectCoords();
	UpdateObjectCoords();

	if (g_Project->Profile()->Location().Valid())
		m_Location = g_Project->Profile()->Location();
	else if (g_Project->RefLocation()->Valid())
		m_Location = *g_Project->RefLocation();
	else if (CConfig::Location().Valid())
		m_Location = CConfig::Location();
	else 
		m_Location.Clear();
	UpdateLocation();

	if (m_RefType==REF_FRAME) {
		if (g_Project->RefObserver())
			obsname = g_strdup(g_Project->RefObserver());
		else
			obsname = CConfig::GetStr("MakeCatDlg", "ObsName");
		gtk_entry_set_text(GTK_ENTRY(m_Observer), obsname);
		g_free(obsname);
		telescope = CConfig::GetStr("MakeCatDlg", "Telescope");
		gtk_entry_set_text(GTK_ENTRY(m_Telescope), telescope);
		g_free(telescope);
		camera = CConfig::GetStr("MakeCatDlg", "Camera");
		gtk_entry_set_text(GTK_ENTRY(m_Camera), camera);
		g_free(camera);
		fov = CConfig::GetStr("MakeCatDlg", "FOV");
		gtk_entry_set_text(GTK_ENTRY(m_FOV), fov);
		g_free(fov);
		orient = CConfig::GetStr("MakeCatDlg", "Orientation");
		gtk_entry_set_text(GTK_ENTRY(m_Orientation), orient);
		g_free(orient);
		notes = CConfig::GetStr("MakeCatDlg", "Notes");
		gtk_text_buffer_set_text(gtk_text_view_get_buffer(GTK_TEXT_VIEW(m_Notes)), notes, -1);
		g_free(notes);
		m_Phot.GetParams(CMPACK_PI_FRAME_PARAMS, info);
		gtk_entry_set_text(GTK_ENTRY(m_Filter), info.filter);
	} else {
		gtk_entry_set_text(GTK_ENTRY(m_Observer), m_Catalog.Observer());
		gtk_entry_set_text(GTK_ENTRY(m_Telescope), m_Catalog.Telescope());
		gtk_entry_set_text(GTK_ENTRY(m_Camera), m_Catalog.Camera());
		gtk_entry_set_text(GTK_ENTRY(m_Filter), m_Catalog.Filter());
		gtk_entry_set_text(GTK_ENTRY(m_FOV), m_Catalog.FieldOfView());
		gtk_entry_set_text(GTK_ENTRY(m_Orientation), m_Catalog.Orientation());
		gtk_text_buffer_set_text(gtk_text_view_get_buffer(GTK_TEXT_VIEW(m_Notes)), m_Catalog.Notes(), -1);
	}

	// Restore last folder
	gchar *path = CConfig::GetStr("MakeCatDlg", "Folder", NULL);
	if (path && *path!='\0' && g_file_test(path, G_FILE_TEST_IS_DIR))
		gtk_entry_set_text(GTK_ENTRY(m_Path), path);
	else {
		gchar *defpath = g_build_filename(get_user_data_dir(), "Catalog files", NULL);
		if (force_directory(defpath))
			gtk_entry_set_text(GTK_ENTRY(m_Path), defpath);
		g_free(defpath);
	}
	g_free(path);

	// Suggest file name from the object name
	if (m_Object.Name()) 
		gtk_entry_set_text(GTK_ENTRY(m_FileName), m_Object.Name());
	else
		gtk_entry_set_text(GTK_ENTRY(m_FileName), "");
		
	// Edit form
	gtk_dialog_run(GTK_DIALOG(m_pDlg));
}

int CMakeCatFileDlg::MakeFileProc(CProgressDlg *sender, void *user_data)
{
	int		res;
	char	*fts_file;
	CMakeCatFileDlg *pMe = (CMakeCatFileDlg*)user_data;

	fts_file = SetFileExtension(pMe->m_ExportPath, FILE_EXTENSION_FITS);
	
	// Delete previous files
	g_unlink(pMe->m_ExportPath);
	if (fts_file)
		g_unlink(fts_file);

	// Make target file
	CCatalog dst;
	if (dst.Open(pMe->m_ExportPath, CMPACK_OPEN_CREATE, &res)) {
		dst.Assign(pMe->m_OutFile, &res);
		dst.Close();
	}

	// Copy image file
	if (res==0 && fts_file) {
		if (pMe->m_RefType==REF_FRAME) {
			GtkTreePath *refpath = g_Project->GetReferencePath();
			if (refpath) {
				gchar *tpath = g_Project->GetImageFile(refpath);
				copy_file(tpath, fts_file, false, &pMe->m_Error);
				g_free(tpath);
				gtk_tree_path_free(refpath);
			}
		} else {
			const gchar *tmp_file = g_Project->GetTempCatFile()->FullPath();
			if (tmp_file) {
				gchar *tpath = SetFileExtension(tmp_file, FILE_EXTENSION_FITS);
				copy_file(tpath, fts_file, false, &pMe->m_Error);
				g_free(tpath);
			}
		}
	}

	g_free(fts_file);
	return res;
}

void CMakeCatFileDlg::button_clicked(GtkWidget *pButton, CMakeCatFileDlg *pMe)
{
	pMe->OnButtonClicked(pButton);
}

void CMakeCatFileDlg::OnButtonClicked(GtkWidget *pButton)
{
	if (pButton == m_ObjBtn) 
		EditObjectCoords();
	else if (pButton == m_LocBtn) 
		EditLocation();
	else if (pButton == m_PathBtn) 
		ChangeCatalogPath();
	else if (pButton == GTK_WIDGET(m_EditSel))
		ChooseStars();
}

void CMakeCatFileDlg::EditObjectCoords(void)
{
	CObjectDlg dlg(GTK_WINDOW(m_pDlg));
	if (dlg.Execute(&m_Object)) 
		UpdateObjectCoords();
}

void CMakeCatFileDlg::EditLocation(void)
{
	CLocationDlg dlg(GTK_WINDOW(m_pDlg));
	if (dlg.Execute(&m_Location)) 
		UpdateLocation();
}

bool CMakeCatFileDlg::ChooseStars(void)
{
	CChooseStarsDlg dlg(GTK_WINDOW(m_pDlg));
	if (dlg.Execute(m_Selection, m_Tags, true)) {
		g_Project->SetSelection(m_Selection);
		g_Project->SetTags(m_Tags);
		UpdateChart();
		return true;
	}
	return false;
}

void CMakeCatFileDlg::UpdateObjectCoords(void)
{
	if (m_Object.Name())
		gtk_entry_set_text(GTK_ENTRY(m_ObjName), m_Object.Name());
	else
		gtk_entry_set_text(GTK_ENTRY(m_ObjName), "");
	if (m_Object.RA())
		gtk_entry_set_text(GTK_ENTRY(m_RA), m_Object.RA());
	else
		gtk_entry_set_text(GTK_ENTRY(m_RA), "");
	if (m_Object.Dec())
		gtk_entry_set_text(GTK_ENTRY(m_Dec), m_Object.Dec());
	else
		gtk_entry_set_text(GTK_ENTRY(m_Dec), "");
}

void CMakeCatFileDlg::UpdateLocation(void)
{
	if (m_Location.Name())
		gtk_entry_set_text(GTK_ENTRY(m_LocName), m_Location.Name());
	else
		gtk_entry_set_text(GTK_ENTRY(m_LocName), "");
	if (m_Location.Lon())
		gtk_entry_set_text(GTK_ENTRY(m_Lon), m_Location.Lon());
	else
		gtk_entry_set_text(GTK_ENTRY(m_Lon), "");
	if (m_Location.Lat())
		gtk_entry_set_text(GTK_ENTRY(m_Lat), m_Location.Lat());
	else
		gtk_entry_set_text(GTK_ENTRY(m_Lat), "");
}

void CMakeCatFileDlg::UpdateChart(void)
{
	cmpack_chart_view_set_model(CMPACK_CHART_VIEW(m_Chart), NULL);
	cmpack_chart_data_clear(m_ChartData);
	if (m_RefType==REF_FRAME) {
		m_Phot.Render(m_ChartData, m_ApertureIndex, false, false);
		m_OutFile.Assign(m_Phot, m_ApertureIndex, NULL);
	} else {
		m_Catalog.Render(m_ChartData, false, false);
		m_OutFile.Assign(m_Catalog, NULL);
	}
	int count = cmpack_chart_data_count(m_ChartData);
	for (int row=0; row<count; row++) {
		int star_id = (int)cmpack_chart_data_get_param(m_ChartData, row);
		int i = m_Selection.IndexOf(star_id);
		if (i>=0) {
			CmpackSelectionType type = m_Selection.GetType(i);
			cmpack_chart_data_set_color(m_ChartData, row, Types[type].color);
			cmpack_chart_data_set_topmost(m_ChartData, row, TRUE);
		} else if (m_Tags.Get(star_id)) {
			cmpack_chart_data_set_color(m_ChartData, row, CMPACK_COLOR_YELLOW);
			cmpack_chart_data_set_topmost(m_ChartData, row, TRUE);
		}
	}
	bool chart_invert = CConfig::GetBool(CConfig::NEGATIVE_CHARTS);
	cmpack_chart_view_set_negative(CMPACK_CHART_VIEW(m_Chart), chart_invert);
	cmpack_chart_view_set_model(CMPACK_CHART_VIEW(m_Chart), m_ChartData);
	cmpack_chart_view_set_auto_zoom(CMPACK_CHART_VIEW(m_Chart), TRUE);
}

bool CMakeCatFileDlg::OnCloseQuery(void)
{
	int res;
	const gchar *dir, *basename;
	gchar *notes;
	gchar *path, *filename, *fts_file;
	GtkTextIter start, end;

	dir = gtk_entry_get_text(GTK_ENTRY(m_Path));
	if (dir[0]=='\0') {
		ShowError(GTK_WINDOW(m_pDlg), "Please, specify a target directory path.");
		return false;
	}
	if (!g_file_test(dir, G_FILE_TEST_IS_DIR)) {
		GtkWidget *dialog = gtk_message_dialog_new(GTK_WINDOW(m_pDlg), 
			(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
			GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, "A folder \"%s\" does not exist.  Do you want to create it?",
			dir);
		gtk_window_set_title(GTK_WINDOW (dialog), "Confirmation");
		gtk_dialog_add_button(GTK_DIALOG (dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
		gtk_dialog_add_button(GTK_DIALOG (dialog), "_Create", GTK_RESPONSE_ACCEPT);
		gtk_dialog_set_alternative_button_order(GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT, GTK_RESPONSE_CANCEL, -1);
		gtk_dialog_set_default_response(GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
		int response = gtk_dialog_run(GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);
		if (response != GTK_RESPONSE_ACCEPT) 
			return false;
		if (!force_directory(dir)) {
			gchar *msg = g_strdup_printf("Failed to create the directory: %s", dir);
			ShowError(GTK_WINDOW(m_pDlg), msg);
			g_free(msg);
			return false;
		}
	}
	
	basename = g_basename(gtk_entry_get_text(GTK_ENTRY(m_FileName)));
	if (basename[0]=='\0') {
		ShowError(GTK_WINDOW(m_pDlg), "Please, specify a name of the target file.");
		return false;
	}
	filename = SetFileExtension(basename, FILE_EXTENSION_CATALOG);
	path = g_build_filename(dir, filename, NULL);
	g_free(filename);
	if (!ConfirmOverwrite(GTK_WINDOW(m_pDlg), path)) {
		g_free(path);
		return false;
	}

	fts_file = SetFileExtension(path, FILE_EXTENSION_FITS);
	if (!ConfirmOverwrite(GTK_WINDOW(m_pDlg), fts_file)) {
		g_free(fts_file);
		return false;
	}
	
	// Selection
	m_OutFile.SetSelection(m_Selection);
	m_OutFile.SetTags(m_Tags);

	// Object
	m_Object.SetName(gtk_entry_get_text(GTK_ENTRY(m_ObjName)));
	m_Object.SetRA(gtk_entry_get_text(GTK_ENTRY(m_RA)));
	m_Object.SetDec(gtk_entry_get_text(GTK_ENTRY(m_Dec)));
	m_OutFile.SetObject(m_Object);
	g_Project->SetObjectCoords(m_Object);
	CConfig::SetLocation(m_Location);

	// Observer
	m_Location.SetName(gtk_entry_get_text(GTK_ENTRY(m_LocName)));
	m_Location.SetLon(gtk_entry_get_text(GTK_ENTRY(m_Lon)));
	m_Location.SetLat(gtk_entry_get_text(GTK_ENTRY(m_Lat)));
	m_OutFile.SetLocation(m_Location);
	m_OutFile.SetObserver(gtk_entry_get_text(GTK_ENTRY(m_Observer)));
	CConfig::SetStr("MakeCatDlg", "ObsName", m_OutFile.Observer());

	// Other fields
	m_OutFile.SetFilter(gtk_entry_get_text(GTK_ENTRY(m_Filter)));
	m_OutFile.SetTelescope(gtk_entry_get_text(GTK_ENTRY(m_Telescope)));
	CConfig::SetStr("MakeCatDlg", "Telescope", m_OutFile.Telescope());
	m_OutFile.SetCamera(gtk_entry_get_text(GTK_ENTRY(m_Camera)));
	CConfig::SetStr("MakeCatDlg", "Camera", m_OutFile.Camera());
	m_OutFile.SetFieldOfView(gtk_entry_get_text(GTK_ENTRY(m_FOV)));
	CConfig::SetStr("MakeCatDlg", "FOV", m_OutFile.FieldOfView());
	m_OutFile.SetOrientation(gtk_entry_get_text(GTK_ENTRY(m_Orientation)));
	CConfig::SetStr("MakeCatDlg", "Orientation", m_OutFile.Orientation());

	// Notes
	GtkTextBuffer *textbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(m_Notes));
	gtk_text_buffer_get_start_iter(textbuf, &start);
	gtk_text_buffer_get_end_iter(textbuf, &end);
	notes = gtk_text_buffer_get_text(textbuf, &start, &end, FALSE);
	CConfig::SetStr("MakeCatDlg", "Notes", notes);
	m_OutFile.SetNotes(notes);
	g_free(notes);

	// Make file in separate thread
	basename = gtk_entry_get_text(GTK_ENTRY(m_FileName));
	filename = SetFileExtension(basename, FILE_EXTENSION_CATALOG);
	m_ExportPath = g_build_filename(gtk_entry_get_text(GTK_ENTRY(m_Path)), filename, NULL);
	CProgressDlg pdlg(GTK_WINDOW(m_pDlg), "Exporting files");
	pdlg.SetFileName(g_basename(m_ExportPath));
	res = pdlg.Execute(MakeFileProc, this);
	if (m_Error) {
		ShowError(GTK_WINDOW(m_pDlg), m_Error->message);
		g_error_free(m_Error);
		m_Error = NULL;
	} else if (res!=0) {
		char *msg = cmpack_formaterror(res);
		ShowError(GTK_WINDOW(m_pDlg), msg);
		cmpack_free(msg);
	}
	g_free(filename);
	g_free(m_ExportPath);
	m_ExportPath = NULL;
	if (res!=0) 
		return false;

	// Save last folder
	CConfig::SetStr("MakeCatDlg", "Folder", gtk_entry_get_text(GTK_ENTRY(m_Path)));
	return true;
}
	
void CMakeCatFileDlg::ChangeCatalogPath(void)
{
	const char *dirname = gtk_entry_get_text(GTK_ENTRY(m_Path));

	GtkWidget *pPathDlg = gtk_file_chooser_dialog_new("Select path",
		GTK_WINDOW(m_pDlg), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, GTK_STOCK_CLOSE, 
		GTK_RESPONSE_REJECT, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_standard_tooltips(GTK_FILE_CHOOSER(pPathDlg));
	if (dirname && g_file_test(dirname, G_FILE_TEST_IS_DIR))
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(pPathDlg), dirname);
	if (gtk_dialog_run(GTK_DIALOG(pPathDlg)) == GTK_RESPONSE_ACCEPT) {
		char *path = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(pPathDlg));
		gtk_entry_set_text(GTK_ENTRY(m_Path), path);
		g_free(path);
	}
	gtk_widget_destroy(pPathDlg);
}
