/**************************************************************

newproject_dlg.cpp (C-Munipack project)
New project dialog
Copyright (C) 2012 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>

#include "newproject_dlg.h"
#include "utils.h"
#include "main.h"
#include "ctxhelp.h"
#include "configuration.h"

enum tTypeColundId
{
	TCOL_ID,
	TCOL_CAPTION,
	TCOL_PIXBUF,
	TNCOLS
};

//-------------------   NEW PROJECT DIALOG   ------------------------------

CNewProjectDlg::CNewProjectDlg(GtkWindow *pParent, bool import):m_ParentDir(NULL), 
	m_FilePath(NULL), m_Import(import)
{
	GtkWidget *scrwnd;
	GtkTreeIter iter;
	GtkTreeViewColumn *col;
	GtkCellRenderer *renderer;

	m_Type = (tProjectType)CConfig::GetInt("Projects", "Type", PROJECT_REDUCE);

	// Dialog with buttons
	m_pDlg = gtk_dialog_new_with_buttons((!m_Import ? "New project" : "Import data to..."), pParent, 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR),
		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, 
		GTK_STOCK_HELP, GTK_RESPONSE_HELP, NULL);
	gtk_dialog_widget_standard_tooltips(GTK_DIALOG(m_pDlg));
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);

	// Dialog icon
	gchar *icon = get_icon_file(!m_Import ? "newproject" : "importproject");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	GtkWidget *vbox = gtk_vbox_new(FALSE, 4);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(m_pDlg)->vbox), vbox, FALSE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 4);

	// Project name and location
	GtkWidget *tbox = gtk_table_new(2, 3, FALSE);
	gtk_table_set_col_spacings(GTK_TABLE(tbox), 8);
	gtk_table_set_row_spacings(GTK_TABLE(tbox), 4);
	gtk_box_pack_start(GTK_BOX(vbox), tbox, FALSE, TRUE, 0);
	GtkWidget *label = gtk_label_new("Project name");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
	m_Name = gtk_entry_new_with_max_length(FILENAME_MAX-16);
	gtk_widget_set_tooltip_text(m_Name, "Name of a new project, it must consist of characters allowed in a file name only.");
	gtk_table_attach(GTK_TABLE(tbox), m_Name, 1, 2, 0, 1, 
		GtkAttachOptions(GTK_FILL | GTK_EXPAND), GTK_FILL, 0, 0);
	label = gtk_label_new("Location");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
	m_Path = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_Path, "Path to the directory where a new project shall be saved to.");
	gtk_table_attach(GTK_TABLE(tbox), m_Path, 1, 2, 1, 2, 
		GtkAttachOptions(GTK_FILL | GTK_EXPAND), GTK_FILL, 0, 0);
	m_PathBtn = gtk_button_new_with_label("Browse");
	gtk_widget_set_tooltip_text(m_PathBtn, "Change target directory");
	g_signal_connect(G_OBJECT(m_PathBtn), "clicked", G_CALLBACK(button_clicked), this);
	gtk_table_attach(GTK_TABLE(tbox), m_PathBtn, 2, 3, 1, 2, GTK_FILL, GTK_FILL, 0, 0);

	// Project types
	label = gtk_label_new("Project type");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, TRUE, 0);
	m_Types = gtk_list_store_new(TNCOLS, G_TYPE_INT, G_TYPE_STRING, GDK_TYPE_PIXBUF);
	for (int i=0; i<EndOfProjectTypes; i++) {
		const gchar *caption = ProjectTypeCaption((tProjectType)i);
		if (caption) {
			gtk_list_store_append(GTK_LIST_STORE(m_Types), &iter);
			gtk_list_store_set(GTK_LIST_STORE(m_Types), &iter, TCOL_CAPTION, caption, TCOL_ID, i, -1);
			if (ProjectTypeIcon((tProjectType)i)) {
				char *icon = get_icon_file(ProjectTypeIcon((tProjectType)i));
				gtk_list_store_set(GTK_LIST_STORE(m_Types), &iter, TCOL_PIXBUF, 
					gdk_pixbuf_new_from_file(icon, NULL), -1);
				g_free(icon);
			}
		}
	}
	m_TypeView = gtk_tree_view_new();
	gtk_widget_set_tooltip_text(m_TypeView, "A project type according to the type of source frames and a kind of output data");
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(m_TypeView), FALSE);
	gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(m_TypeView)), GTK_SELECTION_BROWSE);
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_resizable(col, FALSE);
	gtk_tree_view_append_column(GTK_TREE_VIEW(m_TypeView), col);
	renderer = gtk_cell_renderer_pixbuf_new();
	gtk_tree_view_column_pack_start(col, renderer, FALSE);
	gtk_tree_view_column_add_attribute(col, renderer, "pixbuf", TCOL_PIXBUF);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col, renderer, "text", TCOL_CAPTION);
	scrwnd = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrwnd), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrwnd), GTK_SHADOW_ETCHED_IN);
	gtk_container_add(GTK_CONTAINER(scrwnd), m_TypeView);
	gtk_widget_set_size_request(scrwnd, 480, 240);
	gtk_box_pack_start(GTK_BOX(vbox), scrwnd, TRUE, TRUE, 4);
	gtk_tree_view_set_model(GTK_TREE_VIEW(m_TypeView), GTK_TREE_MODEL(m_Types));

	gtk_widget_show_all(GTK_DIALOG(m_pDlg)->vbox);
}

CNewProjectDlg::~CNewProjectDlg()
{
	g_object_unref(m_Types);
	gtk_widget_destroy(m_pDlg);
	g_free(m_FilePath); 
	g_free(m_ParentDir);
}

bool CNewProjectDlg::Execute()
{
	int id;
	gboolean ok;
	GtkTreeIter iter;

	// Select project type
	ok = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(m_Types), &iter);
	while (ok) {
		gtk_tree_model_get(GTK_TREE_MODEL(m_Types), &iter, TCOL_ID, &id, -1);
		if (id==m_Type) {
			GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(m_TypeView));
			gtk_tree_selection_select_iter(selection, &iter);
			break;
		}
		ok = gtk_tree_model_iter_next(GTK_TREE_MODEL(m_Types), &iter);
	}
	
	// Restore last folder
	gchar *path = CConfig::GetStr("Projects", "Folder", NULL);
	if (path && *path!='\0' && g_file_test(path, G_FILE_TEST_IS_DIR)) 
		gtk_entry_set_text(GTK_ENTRY(m_Path), path);
	else {
		gchar *defpath = g_build_filename(get_user_data_dir(), "Projects", NULL);
		if (force_directory(defpath))
			gtk_entry_set_text(GTK_ENTRY(m_Path), defpath);
		g_free(defpath);
	}
	g_free(path);

	// Project name is empty
	gtk_entry_set_text(GTK_ENTRY(m_Name), "");

	// Run the dialog
	if (gtk_dialog_run(GTK_DIALOG(m_pDlg))!=GTK_RESPONSE_ACCEPT)
		return false;
	gtk_widget_hide(m_pDlg);

	// Save last type (last folder is saved in push button handler)
	CConfig::SetInt("Projects", "Type", m_Type);
	if (m_ParentDir) 
		CConfig::SetStr("Projects", "Folder", m_ParentDir);
	return true;
}

void CNewProjectDlg::response_dialog(GtkDialog *pDlg, gint response_id, CNewProjectDlg *pMe)
{
	if (!pMe->OnResponseDialog(response_id))
		g_signal_stop_emission_by_name(pDlg, "response");
}

bool CNewProjectDlg::OnResponseDialog(gint response_id)
{
	switch (response_id)
	{
	case GTK_RESPONSE_ACCEPT:
		// Check input
		if (!OnCloseQuery())
			return false;
		break;

	case GTK_RESPONSE_HELP:
		// Show context help
		g_MainWnd->ShowHelp(GTK_WINDOW(m_pDlg), IDH_NEW_PROJECT);
		return false;
	}
	return true;
}

void CNewProjectDlg::button_clicked(GtkWidget *pButton, CNewProjectDlg *pMe)
{
	pMe->OnButtonClicked(pButton);
}

void CNewProjectDlg::OnButtonClicked(GtkWidget *pButton)
{
	if (pButton == m_PathBtn) 
		ChangeDirPath();
}

bool CNewProjectDlg::OnCloseQuery(void)
{
	int type;
	char buf[256];
	GtkTreeModel *model;
	GtkTreeIter iter;

	g_free(m_FilePath);
	m_FilePath = NULL;
	g_free(m_ParentDir);
	m_ParentDir = NULL;

	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(m_TypeView));
	if (!gtk_tree_selection_get_selected(selection, &model, &iter)) {
		ShowError(GTK_WINDOW(m_pDlg), "Please, select a project type.");
		return false;
	}
	gtk_tree_model_get(model, &iter, TCOL_ID, &type, -1);
	m_Type = (tProjectType)type;

	// Check project name
	const gchar *name = gtk_entry_get_text(GTK_ENTRY(m_Name));
	if (name[0]=='\0') {
		ShowError(GTK_WINDOW(m_pDlg), "Please, specify a project name.");
		return false;
	}
	if (!CheckFileBaseName(name)) {
		ShowError(GTK_WINDOW(m_pDlg), "A project name must contain characters that are allowed in file name.\nDo not use: / \\ ? % * : | \" < and >");
		return false;
	}

	// Check that the selected parent directory exists
	const gchar *parent_dir = gtk_entry_get_text(GTK_ENTRY(m_Path));
	if (parent_dir[0]=='\0') {
		ShowError(GTK_WINDOW(m_pDlg), "Please, specify a target directory path.");
		return false;
	}
	if (!g_file_test(parent_dir, G_FILE_TEST_IS_DIR)) {
		GtkWidget *dialog = gtk_message_dialog_new(GTK_WINDOW(m_pDlg), 
			(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT),
			GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, "A folder \"%s\" does not exist.  Do you want to create it?",
			parent_dir);
		gtk_window_set_title(GTK_WINDOW (dialog), "Confirmation");
		gtk_dialog_add_button(GTK_DIALOG (dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
		gtk_dialog_add_button(GTK_DIALOG (dialog), "_Create", GTK_RESPONSE_ACCEPT);
		gtk_dialog_set_alternative_button_order(GTK_DIALOG(dialog), GTK_RESPONSE_ACCEPT, GTK_RESPONSE_CANCEL, -1);
		gtk_dialog_set_default_response(GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
		int response = gtk_dialog_run(GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);
		if (response != GTK_RESPONSE_ACCEPT)
			return false;
		if (!force_directory(parent_dir)) {
			gchar *msg = g_strdup_printf("Failed to create the directory: %s", parent_dir);
			ShowError(GTK_WINDOW(m_pDlg), msg);
			g_free(msg);
			return false;
		}
	}

	// Make a new project directory
	gchar *project_dir = g_build_filename(parent_dir, name, NULL);
	if (!force_directory(project_dir)) {
		gchar *msg = g_strdup_printf("Failed to create the directory: %s", project_dir);
		ShowError(GTK_WINDOW(m_pDlg), msg);
		g_free(msg);
		g_free(project_dir);
		return false;
	}
	
	// Make project file path
	sprintf(buf, "%s.%s", name, FILE_EXTENSION_PROJECT);
	gchar *fpath = g_build_filename(project_dir, buf, NULL);

	// Check if the project does not exist
	if (!ConfirmOverwrite(GTK_WINDOW(m_pDlg), fpath)) {
		g_free(fpath);
		g_free(project_dir);
		return false;
	}
	g_free(project_dir);

	// Create project file
	m_FilePath = fpath;
	m_ParentDir = g_strdup(parent_dir);
	return true;
}

void CNewProjectDlg::ChangeDirPath(void)
{
	const char *dirname = gtk_entry_get_text(GTK_ENTRY(m_Path));

	GtkWidget *pPathDlg = gtk_file_chooser_dialog_new("Select path",
		GTK_WINDOW(m_pDlg), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, GTK_STOCK_CLOSE, 
		GTK_RESPONSE_REJECT, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_standard_tooltips(GTK_FILE_CHOOSER(pPathDlg));
	if (dirname && g_file_test(dirname, G_FILE_TEST_IS_DIR))
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(pPathDlg), dirname);
	if (gtk_dialog_run(GTK_DIALOG(pPathDlg)) == GTK_RESPONSE_ACCEPT) {
		char *path = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(pPathDlg));
		gtk_entry_set_text(GTK_ENTRY(m_Path), path);
		g_free(path);
	}
	gtk_widget_destroy(pPathDlg);
}
