/**************************************************************

openproject_dlg.cpp (C-Munipack project)
The 'Open project' dialog
Copyright (C) 2012 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>

#include "opensave_dlgs.h"
#include "utils.h"
#include "main.h"
#include "ctxhelp.h"
#include "configuration.h"

//-------------------   OPEN PROJECT DIALOG   ------------------------------

COpenDlg::COpenDlg(GtkWindow *pParent, bool open_file):m_OpenFile(open_file), m_Path(NULL)
{
	GtkFileFilter *filters[2];

	// Dialog with buttons
	m_pDlg = gtk_file_chooser_dialog_new("Open project", pParent, GTK_FILE_CHOOSER_ACTION_OPEN,
		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, 
		GTK_STOCK_HELP, GTK_RESPONSE_HELP, NULL);
	gtk_file_chooser_standard_tooltips(GTK_FILE_CHOOSER(m_pDlg));
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);
	g_signal_connect(G_OBJECT(m_pDlg), "update-preview", G_CALLBACK(update_preview), this);
	g_signal_connect(G_OBJECT(m_pDlg), "selection-changed", G_CALLBACK(selection_changed), this);

	if (m_OpenFile)
		gtk_window_set_title(GTK_WINDOW(m_pDlg), "Open file");

	// File filters
	filters[0] = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filters[0], "*." FILE_EXTENSION_PROJECT);
	gtk_file_filter_set_name(filters[0], "C-Munipack projects");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(m_pDlg), filters[0]);
	filters[1] = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filters[1], "*");
	gtk_file_filter_set_name(filters[1], "All files");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(m_pDlg), filters[1]);
	if (!open_file)
		gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(m_pDlg), filters[0]);
	else
		gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(m_pDlg), filters[1]);

	// Dialog icon
	gchar *icon = get_icon_file(open_file ? "openfile" : "openproject");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Preview widget
	gtk_file_chooser_set_preview_widget(GTK_FILE_CHOOSER(m_pDlg), 
		GTK_WIDGET(g_object_ref(m_Preview.Handle())));
}

COpenDlg::~COpenDlg()
{
	gtk_widget_destroy(m_pDlg);
	g_free(m_Path);
}

void COpenDlg::response_dialog(GtkDialog *pDlg, gint response_id, COpenDlg *pMe)
{
	if (!pMe->OnResponseDialog(response_id))
		g_signal_stop_emission_by_name(pDlg, "response");
}

bool COpenDlg::OnResponseDialog(gint response_id)
{
	switch (response_id)
	{
	case GTK_RESPONSE_HELP:
		// Show context help
		g_MainWnd->ShowHelp(GTK_WINDOW(m_pDlg), IDH_OPEN_PROJECT);
		return false;
	}
	return true;
}

bool COpenDlg::Execute(void)
{
	gchar *defpath = NULL;

	g_free(m_Path);
	m_Path = NULL;
	m_Preview.Clear();

	if (m_OpenFile) {
		gchar *path = CConfig::GetStr("OpenFile", "File", NULL);
		if (path)
			defpath = g_path_get_dirname(path);
		g_free(path);
	} else {
		gchar *path = CConfig::GetStr("Projects", "Last", NULL);
		if (path)
			defpath = g_path_get_dirname(path);
		g_free(path);
	}
	if (!defpath)
		defpath = g_build_filename(get_user_data_dir(), "Projects", NULL);
	if (defpath && g_file_test(defpath, G_FILE_TEST_IS_DIR)) 
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(m_pDlg), defpath);
	g_free(defpath);

	gtk_file_chooser_set_preview_widget_active(GTK_FILE_CHOOSER(m_pDlg), 
		!CConfig::GetBool(CConfig::DISABLE_PREVIEWS));

	if (gtk_dialog_run(GTK_DIALOG(m_pDlg))==GTK_RESPONSE_ACCEPT) 
		m_Path = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(m_pDlg));
	
	return m_Path!=NULL;
}

void COpenDlg::update_preview(GtkFileChooser *widget, COpenDlg *pMe)
{
	pMe->UpdatePreview(widget);
}

void COpenDlg::UpdatePreview(GtkFileChooser *widget)
{
	if (!CConfig::GetBool(CConfig::DISABLE_PREVIEWS)) {
		gchar *file = gtk_file_chooser_get_preview_filename(widget);
		if (file) {
			m_Preview.Update(file);
			g_free(file);
		} else {
			m_Preview.Clear();
		}
	}
}

void COpenDlg::selection_changed(GtkFileChooser *widget, COpenDlg *pMe)
{
	pMe->OnSelectionChanged(widget);
}

void COpenDlg::OnSelectionChanged(GtkFileChooser *widget)
{
	UpdateControls();
}

void COpenDlg::UpdateControls(void)
{
	GtkWidget *ok_btn = get_dialog_widget_by_response(GTK_DIALOG(m_pDlg), GTK_RESPONSE_ACCEPT);
	gchar *fname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(m_pDlg));
	gtk_widget_set_sensitive(ok_btn, (fname ? g_file_test(fname, G_FILE_TEST_IS_REGULAR)!=0 : false));
	g_free(fname);
}

//-------------------   SAVE PROJECT DIALOG   ------------------------------

CSaveProjectDlg::CSaveProjectDlg(GtkWindow *pParent):m_Path(NULL)
{
	GtkFileFilter *filters[2];

	// Dialog with buttons
	m_pDlg = gtk_file_chooser_dialog_new("Save project as...", pParent, GTK_FILE_CHOOSER_ACTION_SAVE,
		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, 
		GTK_STOCK_HELP, GTK_RESPONSE_HELP, NULL);
	gtk_file_chooser_standard_tooltips(GTK_FILE_CHOOSER(m_pDlg));
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);

	// File filters
	filters[0] = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filters[0], "*." FILE_EXTENSION_PROJECT);
	gtk_file_filter_set_name(filters[0], "C-Munipack projects");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(m_pDlg), filters[0]);
	filters[1] = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filters[1], "*");
	gtk_file_filter_set_name(filters[1], "All files");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(m_pDlg), filters[1]);

	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(m_pDlg), filters[0]);

	// Dialog icon
	gchar *icon = get_icon_file("saveproject");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);
}

CSaveProjectDlg::~CSaveProjectDlg()
{
	gtk_widget_destroy(m_pDlg);
	g_free(m_Path);
}

void CSaveProjectDlg::response_dialog(GtkDialog *pDlg, gint response_id, CSaveProjectDlg *pMe)
{
	if (!pMe->OnResponseDialog(response_id))
		g_signal_stop_emission_by_name(pDlg, "response");
}

bool CSaveProjectDlg::OnResponseDialog(gint response_id)
{
	switch (response_id)
	{
	case GTK_RESPONSE_ACCEPT:
		// Check file
		if (!OnCloseQuery())
			return false;
		break;

	case GTK_RESPONSE_HELP:
		// Show context help
		g_MainWnd->ShowHelp(GTK_WINDOW(m_pDlg), IDH_SAVE_PROJECT);
		return false;
	}
	return true;
}

bool CSaveProjectDlg::OnCloseQuery(void)
{
	// Get file name, check extension
	gchar *str = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(m_pDlg));
	gchar *fpath = SetFileExtension(str, FILE_EXTENSION_PROJECT);
	g_free(str);

	// Compare paths
	if (ComparePaths(g_Project->Path(), fpath)==0) {
		ShowError(GTK_WINDOW(m_pDlg), "Cannot make copy to itself");
		g_free(fpath);
		return false;
	}

	// Ask for confirmation if the file exists already
	if (!ConfirmOverwrite(GTK_WINDOW(m_pDlg), fpath)) {
		g_free(fpath);
		return false;
	}

	g_free(fpath);
	return true;
}

bool CSaveProjectDlg::Execute(void)
{
	char buf[FILENAME_MAX+32];

	g_free(m_Path);
	m_Path = NULL;

	gchar *dirname = g_path_get_dirname(g_Project->Path());
	if (dirname && g_file_test(dirname, G_FILE_TEST_IS_DIR)) 
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(m_pDlg), dirname);
	g_free(dirname);

	sprintf(buf, "%s.%s", g_Project->Name(), FILE_EXTENSION_PROJECT);
	gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(m_pDlg), buf);

	if (gtk_dialog_run(GTK_DIALOG(m_pDlg))==GTK_RESPONSE_ACCEPT) {
		m_Path = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(m_pDlg));
		return true;
	}
	return false;
}

//-------------------   SAVE PROJECT DIALOG   ------------------------------

CSaveProfileDlg::CSaveProfileDlg(GtkWindow *pParent):m_Path(NULL)
{
	GtkFileFilter *filters[2];

	// Dialog with buttons
	m_pDlg = gtk_file_chooser_dialog_new("Save project settings as...", pParent, 
		GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, 
		GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_standard_tooltips(GTK_FILE_CHOOSER(m_pDlg));
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);

	// File filters
	filters[0] = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filters[0], "*." FILE_EXTENSION_PROFILE);
	gtk_file_filter_set_name(filters[0], "C-Munipack profiles");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(m_pDlg), filters[0]);
	filters[1] = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filters[1], "*");
	gtk_file_filter_set_name(filters[1], "All files");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(m_pDlg), filters[1]);
	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(m_pDlg), filters[0]);
}

CSaveProfileDlg::~CSaveProfileDlg()
{
	gtk_widget_destroy(m_pDlg);
	g_free(m_Path);
}

void CSaveProfileDlg::response_dialog(GtkDialog *pDlg, gint response_id, CSaveProfileDlg *pMe)
{
	if (!pMe->OnResponseDialog(response_id))
		g_signal_stop_emission_by_name(pDlg, "response");
}

bool CSaveProfileDlg::OnResponseDialog(gint response_id)
{
	switch (response_id)
	{
	case GTK_RESPONSE_ACCEPT:
		// Check file
		if (!OnCloseQuery())
			return false;
		break;
	}
	return true;
}

bool CSaveProfileDlg::OnCloseQuery(void)
{
	// Get file name, check extension
	gchar *str = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(m_pDlg));
	gchar *fpath = SetFileExtension(str, FILE_EXTENSION_PROFILE);
	g_free(str);

	// Ask for confirmation if the file exists already
	if (!ConfirmOverwrite(GTK_WINDOW(m_pDlg), fpath)) {
		g_free(fpath);
		return false;
	}

	g_free(fpath);
	return true;
}


//
// Execute the dialog
//
bool CSaveProfileDlg::Execute(void)
{
	char buf[FILENAME_MAX+32];

	g_free(m_Path);
	m_Path = NULL;

	gchar *dirname = CConfig::GetStr("ProfileDlg", "Path", NULL);
	if (dirname && g_file_test(dirname, G_FILE_TEST_IS_DIR)) 
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(m_pDlg), dirname);
	g_free(dirname);

	sprintf(buf, "%s.%s", "new profile", FILE_EXTENSION_PROFILE);
	gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(m_pDlg), buf);

	if (gtk_dialog_run(GTK_DIALOG(m_pDlg))!=GTK_RESPONSE_ACCEPT)
		return false;

	m_Path = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(m_pDlg));
	dirname = g_path_get_dirname(m_Path);
	CConfig::SetStr("ProfileDlg", "Path", dirname);
	g_free(dirname);
	return true;
}

//------------------------------   IMPORT PROJECT DIALOG   ------------------------------

CImportProjectDlg::CImportProjectDlg(GtkWindow *pParent):m_Path(NULL)
{
	// Dialog with buttons
	m_pDlg = gtk_file_chooser_dialog_new("Import data from...", pParent, 
		GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, GTK_STOCK_CLOSE, GTK_RESPONSE_REJECT, 
		GTK_STOCK_APPLY, GTK_RESPONSE_ACCEPT, GTK_STOCK_HELP, GTK_RESPONSE_HELP, NULL);
	gtk_file_chooser_standard_tooltips(GTK_FILE_CHOOSER(m_pDlg));
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);

	// Dialog icon
	gchar *icon = get_icon_file("importproject");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);
}

CImportProjectDlg::~CImportProjectDlg()
{
	gtk_widget_destroy(m_pDlg);
	g_free(m_Path);
}

bool CImportProjectDlg::Execute(void)
{
	g_free(m_Path);
	m_Path = NULL;

	// Go to last folder
	gchar *folder = CConfig::GetStr("ImportDataDlg", "Folder", NULL);
	if (folder && g_file_test(folder, G_FILE_TEST_IS_DIR)) 
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(m_pDlg), folder);
	g_free(folder);

	if (gtk_dialog_run(GTK_DIALOG(m_pDlg))==GTK_RESPONSE_ACCEPT) {
		m_Path = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(m_pDlg));
		CConfig::SetStr("ImportDataDlg", "Folder", m_Path);
		return m_Path!=NULL;
	}
	return FALSE;
}

void CImportProjectDlg::response_dialog(GtkDialog *pDlg, gint response_id, CImportProjectDlg *pMe)
{
	if (!pMe->OnResponseDialog(response_id))
		g_signal_stop_emission_by_name(pDlg, "response");
}

bool CImportProjectDlg::OnResponseDialog(gint response_id)
{
	switch (response_id)
	{
	case GTK_RESPONSE_HELP:
		// Show context help
		g_MainWnd->ShowHelp(GTK_WINDOW(m_pDlg), IDH_IMPORT_PROJECT);
		return false;
	}
	return true;
}
