/**************************************************************

openproject_dlg.h (C-Munipack project)
The 'Open project' dialog
Copyright (C) 2012 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CMPACK_OPENPROJECT_DLG_H
#define CMPACK_OPENPROJECT_DLG_H

#include <gtk/gtk.h>

#include "preview.h"

//
// Open project/file dialog
//
class COpenDlg
{
public:
	COpenDlg(GtkWindow *pParent, bool open_file);
	~COpenDlg();

	bool Execute(void);

	const gchar *Path(void) const
	{ return m_Path; }

private:
	bool		m_OpenFile;
	GtkWidget	*m_pDlg;
	CPreview	m_Preview;
	gchar		*m_Path;

	bool OnResponseDialog(gint response_id);
	void OnSelectionChanged(GtkFileChooser *chooser);
	void UpdatePreview(GtkFileChooser *chooser);
	void UpdateControls(void);

	static void update_preview(GtkFileChooser *chooser, COpenDlg *pMe);
	static void selection_changed(GtkFileChooser *chooser, COpenDlg *pMe);
	static void response_dialog(GtkDialog *pDlg, gint response_id, COpenDlg *pMe);
};

//
// Save project dialog
//
class CSaveProjectDlg
{
public:
	CSaveProjectDlg(GtkWindow *pParent);
	~CSaveProjectDlg();

	bool Execute(void);

	const gchar *Path() const
	{ return m_Path; }

private:
	GtkWidget	*m_pDlg;
	gchar		*m_Path;

	bool OnCloseQuery();
	bool OnResponseDialog(gint response_id);

	static void response_dialog(GtkDialog *pDlg, gint response_id, CSaveProjectDlg *pMe);
};


//
// Save profile dialog
//
class CSaveProfileDlg
{
public:
	CSaveProfileDlg(GtkWindow *pParent);
	~CSaveProfileDlg();

	bool Execute(void);

	const gchar *Path() const
	{ return m_Path; }

private:
	GtkWidget	*m_pDlg;
	gchar		*m_Path;

	bool OnCloseQuery();
	bool OnResponseDialog(gint response_id);

	static void response_dialog(GtkDialog *pDlg, gint response_id, CSaveProfileDlg *pMe);
};


//
// Import project dialog
//
class CImportProjectDlg
{
public:
	CImportProjectDlg(GtkWindow *pParent);
	~CImportProjectDlg();

	bool Execute(void);

	const gchar *Path(void) const
	{ return m_Path; }

private:
	GtkWidget	*m_pDlg;
	CPreview	m_Preview;
	gchar		*m_Path;

	bool OnResponseDialog(gint response_id);
	void UpdateControls(void);

	static void response_dialog(GtkDialog *pDlg, gint response_id, CImportProjectDlg *pMe);
};

#endif
