/**************************************************************

project_dlg.h (C-Munipack project)
Project settions dialog
Copyright (C) 2012 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CMPACK_PROJECT_DLG_H
#define CMPACK_PROJECT_DLG_H

#include <gtk/gtk.h>

#include "utils.h"

class CProjectDlg
{
public:
	// Section identifiers
	enum tSectionId {
		PAGE_PROJECT,
		PAGE_FILES,
		PAGE_CAMERA,
		PAGE_SOURCE_FILES,
		PAGE_CALIBRATION,
		PAGE_STAR_DETECTION,
		PAGE_PHOTOMETRY,
		PAGE_MATCHING,
		PAGE_LIGHT_CURVE,
		PAGE_MASTER_BIAS,
		PAGE_MASTER_DARK,
		PAGE_MASTER_FLAT,
		PAGE_MERGE_FRAMES,
		PAGE_FIND_VARIABLES,
		PAGE_OBSERVER,
		PAGE_DEFAULT = -1,
	};

	CProjectDlg(GtkWindow *pParent);
	~CProjectDlg();

	bool Execute(tSectionId page_id = PAGE_DEFAULT);

private:
	GtkWidget		*m_pDlg, *m_PageFrame, *m_TitleFrame, *m_PageTitle;
	GtkWidget		*m_LoadProfile, *m_SaveProfile, *m_LoadDefaults;
	GtkWidget		*m_Type, *m_Name, *m_Path, *m_Rename, *m_TypeBtn;
	GtkWidget		*m_Bias, *m_Dark, *m_Flat, *m_Ref, *m_Cat;
	GtkWidget		*m_CurrentPage, *m_PageView, *m_PageScroller;
	GtkWidget		*m_WorkFormat, *m_DataMin, *m_DataMax, *m_Binning;
	GtkWidget		*m_BLeft, *m_BRight, *m_BTop, *m_BBottom;
	GtkWidget		*m_StdCalibration, *m_AdvCalibration, *m_ArtCompStar;
	GtkWidget		*m_MBiasFormat, *m_MDarkFormat;
	GtkWidget		*m_VarFindThreshold, *m_MFlatFormat, *m_MFlatLevel, *m_KombineFormat;
	GtkWidget		*m_RNoise, *m_ADCGain, *m_FWHM, *m_Thresh;
	GtkWidget		*m_SharpMin, *m_SharpMax, *m_RoundMin, *m_RoundMax;
	GtkWidget		*m_Aperture[MAX_APERTURES], *m_SkyIn, *m_SkyOut;
	GtkWidget		*m_MatchStandard, *m_MatchSparse, *m_MatchBox1, *m_MatchBox2;
	GtkWidget		*m_MatchRS, *m_MatchIS, *m_MatchClip, *m_MatchMax;
	GtkWidget		*m_LocName, *m_LocBtn, *m_Lon, *m_Lat;
	GtkTreeStore	*m_PageList;
	GtkTreeModel	*m_FilteredPageList;
	CProfile		m_Profile;
	tProjectType	m_ProjectType;
	bool			m_Updating;
	int				m_PageId, m_DlgWidth, m_DlgHeight;

	GtkWidget *CreatePage(int id, int indent, const gchar *caption, 
		const gchar *icon, bool def_btn, GSList **stack);

	void SetData(void);
	void GetData(void);
	void UpdateControls(void);
	void ShowPage(int id);
	void LoadFromFile(void);
	void SaveToFile(void);
	void LoadDefaults(void);
	void EditLocation(void);

	bool OnResponseDialog(gint response_id);
	bool OnCloseQuery(void);
	void OnSelectionChanged(void);
	void OnSetDefaultsClicked(GtkWidget *pWidget);
	void OnButtonClicked(GtkWidget *pWidget);
	void OnButtonToggled(GtkWidget *pWidget);
	void OnComboBoxChanged(GtkWidget *pWidget);

	static gboolean SetMaxDialogSize(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data);
	static gint selection_changed(GtkTreeSelection *selection, CProjectDlg *pMe);
	static void button_clicked(GtkWidget *widget, CProjectDlg *pDlg);
	static void combo_box_changed(GtkWidget *widget, CProjectDlg *pMe);
	static void setdefaults_clicked(GtkWidget *widget, CProjectDlg *pMe);
	static void response_dialog(GtkWidget *widget, gint response_id, CProjectDlg *pMe);
	static void button_toggled(GtkWidget *widget, CProjectDlg *pMe);
	static gboolean page_visible(GtkTreeModel *model, GtkTreeIter *iter, CProjectDlg *pMe);
};

#endif
