/**************************************************************

varfind_dlg.cpp (C-Munipack project)
The 'Find variables' dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>

#include "varfind_dlg.h"
#include "showheader_dlg.h"
#include "configuration.h"
#include "proc_classes.h"
#include "utils.h"
#include "main.h"
#include "ctxhelp.h"
#include "chartexport_dlg.h"
#include "frameinfo_dlg.h"
#include "project_dlg.h"

enum tMenuId
{
	MENU_FILE = 1,
	MENU_VIEW,
	MENU_HELP
};

enum tCommandId
{
	CMD_CLOSE = 100,
	CMD_PRINT,
	CMD_REBUILD,
	CMD_SAVE_CHART,
	CMD_SAVE_MAGDEV,
	CMD_SAVE_LCURVE,
	CMD_SHOW_PROPERTIES,
	CMD_SHOW_CHART,
	CMD_SHOW_IMAGE,
	CMD_SHOW_MIXED,
	CMD_EXPORT_DATA,
	CMD_EXPORT_MAGDEV,
	CMD_EXPORT_LCURVE,
	CMD_HELP
};

enum tFieldId
{
	FIELD_FILE,
	FIELD_FILENAME,
	FIELD_DIRPATH
};

static const CMenuBar::tMenuItem FileMenu1[] = {
	{ CMenuBar::MB_ITEM,	CMD_SAVE_MAGDEV,		"Save _mag-dev curve" },
	{ CMenuBar::MB_ITEM,	CMD_SAVE_CHART,			"Save _chart" },
	{ CMenuBar::MB_ITEM,	CMD_SAVE_LCURVE,		"Save _light curve" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM,	CMD_EXPORT_DATA,		"Export varfind data" },
	{ CMenuBar::MB_ITEM,	CMD_EXPORT_MAGDEV,		"Export mag-dev curve as image" },
	{ CMenuBar::MB_ITEM,	CMD_EXPORT_LCURVE,		"Export light curve as image" },
	//{ CMenuBar::MB_ITEM,	CMD_PRINT,				"P_rint" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM,	CMD_SHOW_PROPERTIES,	"Show _Properties" },
	{ CMenuBar::MB_ITEM,	CMD_REBUILD,			"Reload" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM,	CMD_CLOSE,				"_Close" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem ViewMenu[] = {
	{ CMenuBar::MB_RADIOBTN, CMD_SHOW_IMAGE,		"_Image only" },
	{ CMenuBar::MB_RADIOBTN, CMD_SHOW_CHART,		"_Chart only" },
	{ CMenuBar::MB_RADIOBTN, CMD_SHOW_MIXED,		"Image and chart" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem FileMenu2[] = {
	{ CMenuBar::MB_ITEM,	CMD_SAVE_MAGDEV,		"Save _mag-dev curve" },
	{ CMenuBar::MB_ITEM,	CMD_SAVE_CHART,			"Save _chart" },
	{ CMenuBar::MB_ITEM,	CMD_SAVE_LCURVE,		"Save _light curve" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM,	CMD_EXPORT_DATA,		"Export varfind data" },
	{ CMenuBar::MB_ITEM,	CMD_EXPORT_MAGDEV,		"Export mag-dev curve as image" },
	{ CMenuBar::MB_ITEM,	CMD_EXPORT_LCURVE,		"Export light curve as image" },
	//{ CMenuBar::MB_ITEM,	CMD_PRINT,				"P_rint" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM,	CMD_REBUILD,			"Rebuild" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM,	CMD_CLOSE,				"_Close" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem HelpMenu[] = {
	{ CMenuBar::MB_ITEM,	CMD_HELP,				"_Show help", "help" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenu PreviewMenu1[] = {
	{ "_File",	MENU_FILE,	FileMenu1 },
	{ "_View",	MENU_VIEW,	ViewMenu },
	{ "_Help",	MENU_HELP,	HelpMenu,	CMenuBar::MBF_RIGHT },
	{ NULL }
};

static const CMenuBar::tMenu PreviewMenu2[] = {
	{ "_File",	MENU_FILE,	FileMenu2 },
	{ "_View",	MENU_VIEW,	ViewMenu },
	{ "_Help",	MENU_HELP,	HelpMenu,	CMenuBar::MBF_RIGHT },
	{ NULL }
};

//-------------------------   HELPER FUNCTIONS   --------------------------------

static gboolean foreach_all_files(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, GSList **list)
{
	if (g_Project->GetState(iter) & CFILE_MATCHING) 
		*list = g_slist_append(*list, gtk_tree_row_reference_new(model, path));
	return FALSE;
}

static void foreach_sel_files(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, GSList **list)
{
	if (g_Project->GetState(iter) & CFILE_MATCHING) 
		*list = g_slist_append(*list, gtk_tree_row_reference_new(model, path));
}

//-------------------------   VARFIND DIALOG   --------------------------------

//
// Constructor
//
CExtVarFindDlg::CExtVarFindDlg(void):m_PhotometryFile(false), m_ExportPath(NULL)
{
	// Dialog icon
	gchar *icon = get_icon_file("varfind");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Menu bar
	m_Menu.Create(PreviewMenu1, false);
	m_Menu.RegisterCallback(MenuCallback, this);
	gtk_box_pack_start(GTK_BOX(m_MainBox), m_Menu.Handle(), FALSE, FALSE, 0);

	// VarFind box
	m_VarFind.RegisterCallback(VarFindCallback, this);
	gtk_box_pack_start(GTK_BOX(m_MainBox), m_VarFind.Handle(), TRUE, TRUE, 0);

	// Show the dialog
	gtk_widget_show_all(m_MainBox);
}

//
// Denstructor
//
CExtVarFindDlg::~CExtVarFindDlg(void)
{
	g_free(m_ExportPath);
}

//
// Load catalog file
//
bool CExtVarFindDlg::LoadFile(const char *fpath)
{
	bool ok;
	int res;
	char *fts_file, *pht_file;
	
	m_Phot.Close();
	m_Frame.Close();
	m_Catalog.Close();
	m_Image.Clear();

	if (m_File.Load(fpath, &res)) {
		m_VarFind.SetApertures(*m_File.Apertures());
		ok = false;
		if (!ok) {
			pht_file = SetFileExtension(fpath, FILE_EXTENSION_CATALOG);
			if (m_Catalog.Open(pht_file, CMPACK_OPEN_READONLY)) {
				m_PhotometryFile = false;
				m_VarFind.SetCatalogFile(&m_Catalog);
				ok = true;
			}
			g_free(pht_file);
		}
		if (!ok) {
			pht_file = SetFileExtension(fpath, "mat");
			if (m_Phot.Open(pht_file, CMPACK_OPEN_READONLY)) {
				m_VarFind.SetPhotometryFile(&m_Phot);
				m_PhotometryFile = true;
				ok = true;
			}
			g_free(pht_file);
		}
		if (!ok) {
			pht_file = SetFileExtension(fpath, FILE_EXTENSION_PHOTOMETRY);
			if (m_Phot.Open(pht_file, CMPACK_OPEN_READONLY)) {
				m_VarFind.SetPhotometryFile(&m_Phot);
				m_PhotometryFile = true;
				ok = true;
			}
			g_free(pht_file);
		}
		if (ok) {
			fts_file = SetFileExtension(fpath, FILE_EXTENSION_FITS);
			if (m_Frame.Open(fts_file, CMPACK_OPEN_READONLY)) {
				if (m_Image.Load(m_Frame, CMPACK_BITPIX_AUTO))
					m_VarFind.SetImage(&m_Image);
			}
			g_free(fts_file);
		}
		if (ok) {
			UpdateMagDev();
			UpdateControls();
			return true;
		}
	}
	return false;
}


//
// Reload data from a file
//
void CExtVarFindDlg::ReloadFrameSet(void)
{
	int res;

	if (m_File.Load(m_Path, &res)) {
		m_VarFind.SetApertures(*m_File.Apertures());
		m_VarFind.UpdateChart();
		UpdateMagDev();
		UpdateLightCurve();
		UpdateControls();
	}
}


//
// Menu callback
//
void CExtVarFindDlg::MenuCallback(CCBObject *sender, int message, int wparam, void *lparam, void *cb_data)
{
	CExtVarFindDlg *pMe = (CExtVarFindDlg*)cb_data;

	switch (message)
	{
	case CMenuBar::CB_ACTIVATE:
		pMe->OnCommand(wparam);
		break;
	}
}

//
// Command handler
//
void CExtVarFindDlg::OnCommand(int cmd_id)
{
	switch (cmd_id)
	{
	// File menu
	case CMD_SHOW_PROPERTIES:
		ShowProperties();
		break;
	case CMD_REBUILD:
		ReloadFrameSet();
		break;
	case CMD_SAVE_CHART:
		m_VarFind.ExportChart();
		break;
	case CMD_SAVE_LCURVE:
		m_VarFind.SaveLightCurve();
		break;
	case CMD_EXPORT_LCURVE:
		m_VarFind.ExportLightCurve();
		break;
	case CMD_SAVE_MAGDEV:
		m_VarFind.SaveMagDevCurve();
		break;
	case CMD_EXPORT_MAGDEV:
		m_VarFind.ExportMagDevCurve();
		break;
	case CMD_CLOSE:
		Close();
		break;

	// View menu
	case CMD_SHOW_CHART:
		m_VarFind.SetDisplayMode(CVarFind::DISPLAY_CHART);
		UpdateControls();
		break;
	case CMD_SHOW_IMAGE:
		m_VarFind.SetDisplayMode(CVarFind::DISPLAY_IMAGE);
		UpdateControls();
		break;
	case CMD_SHOW_MIXED:
		m_VarFind.SetDisplayMode(CVarFind::DISPLAY_MIXED);
		UpdateControls();
		break;
		
	// Help menu
	case CMD_HELP:
		g_MainWnd->ShowHelp(GTK_WINDOW(m_pDlg), IDH_FIND_VARIABLES);
		break;
	}
}

//
// VarFind callback
//
void CExtVarFindDlg::VarFindCallback(CCBObject *sender, int message, int wparam, void *lparam, void *cb_data)
{
	CExtVarFindDlg *pMe = (CExtVarFindDlg*)cb_data;

	switch (message)
	{
	case CVarFind::CB_UPDATE_CURVE:
	case CVarFind::CB_COMPARISON_CHANGED:
		pMe->UpdateMagDev();
		pMe->UpdateLightCurve();
		pMe->UpdateControls();
		break;

	case CVarFind::CB_VARIABLE_CHANGED:
		pMe->UpdateLightCurve();
		pMe->UpdateControls();
		break;

	case CVarFind::CB_ENABLE_CTRL_QUERY:
		*((bool*)lparam) = pMe->OnEnableCtrlQuery((CVarFind::tControlId)wparam);
		break;

	case CVarFind::CB_OBJECT_VALID_QUERY:
		*((bool*)lparam) = pMe->m_File.FindObject(wparam)>=0;
		break;

	case CVarFind::CB_REMOVE_FRAMES_FROM_DATASET:
		pMe->RemoveFramesFromDataSet();
		break;
	case CVarFind::CB_REMOVE_OBJECTS_FROM_DATASET:
		pMe->RemoveObjectsFromDataSet();
		break;
	}
}

//
// Update mag-dev curve
//
void CExtVarFindDlg::UpdateMagDev(void)
{
	int res, refstar;
	CMFindProc mfind;

	const CAperture *aperture = m_VarFind.Aperture();
	if (aperture) {
		refstar = m_VarFind.ComparisonID();
		if (refstar<0)
			refstar = CMPACK_MFIND_AUTO;
		res = mfind.Init(NULL, *aperture, refstar);
		if (res==0)
			res = mfind.Execute(m_File, m_MagDev, &refstar);
		if (res==0 && refstar!=m_VarFind.ComparisonID()) 
			m_VarFind.SetComparisonStar(refstar);
	}
	m_VarFind.SetMagDev(&m_MagDev);
}

//
// Variable changed
//
void CExtVarFindDlg::UpdateLightCurve(void)
{
	int varstar, compstar;
	CSelection sel;

	varstar = m_VarFind.VariableID();
	compstar = m_VarFind.ComparisonID();
	if (varstar>=0 && compstar>=0) {
		const CAperture *aperture = m_VarFind.Aperture();
		if (aperture) {
			sel.Select(varstar, CMPACK_SELECT_VAR);
			sel.Select(compstar, CMPACK_SELECT_COMP);
			CmpackLightCurve(NULL, m_LCurve, sel, m_File, aperture->Id(), CMPACK_LCURVE_FRAME_IDS);
		}
	}
	m_VarFind.SetLightCurve(&m_LCurve);
}

//
// Enable/disable controls
//
void CExtVarFindDlg::UpdateControls(void)
{
	int varstar = m_VarFind.VariableID(), compstar = m_VarFind.ComparisonID(), 
		aperture = m_VarFind.ApertureIndex();

	m_Menu.Enable(CMD_SHOW_IMAGE, m_Frame.Valid());
	m_Menu.Enable(CMD_SHOW_MIXED, m_Frame.Valid());
	m_Menu.Check(CMD_SHOW_IMAGE, m_VarFind.DisplayMode()==CVarFind::DISPLAY_IMAGE);
	m_Menu.Check(CMD_SHOW_CHART, m_VarFind.DisplayMode()==CVarFind::DISPLAY_CHART);
	m_Menu.Check(CMD_SHOW_MIXED, m_VarFind.DisplayMode()==CVarFind::DISPLAY_MIXED);
	m_Menu.Enable(CMD_SAVE_MAGDEV, compstar>=0 && aperture>=0);
	m_Menu.Enable(CMD_EXPORT_MAGDEV, compstar>=0 && aperture>=0);
	m_Menu.Enable(CMD_SAVE_LCURVE, varstar>=0 && compstar>=0 && aperture>=0);
	m_Menu.Enable(CMD_EXPORT_LCURVE, varstar>=0 && compstar>=0 && aperture>=0);
}

//
// Show properties
//
void CExtVarFindDlg::ShowProperties(void)
{
	CVarFindInfoDlg dlg(GTK_WINDOW(m_pDlg));
	dlg.ShowModal(m_File, m_Name, m_Path);
}

//
// Is this control enabled?
//
bool CExtVarFindDlg::OnEnableCtrlQuery(CVarFind::tControlId ctrl)
{
	switch (ctrl)
	{
	case CVarFind::ID_SAVE_CHART:
		return (m_PhotometryFile ? m_Phot.Valid() : m_Catalog.Valid());
	case CVarFind::ID_SAVE_LCURVE:
		return m_VarFind.VariableID()>=0 && m_VarFind.ComparisonID()>=0 && 
			m_VarFind.ApertureIndex()>=0;
	case CVarFind::ID_REMOVE_FROM_DATASET:
		return true;
	default:
		return false;
	}
}

void CExtVarFindDlg::ExportData(void)
{
	char *filename, *folder;

	GtkWidget *pSaveDlg = gtk_file_chooser_dialog_new("Export munifind data",
		GTK_WINDOW(m_pDlg), GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, 
		GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_standard_tooltips(GTK_FILE_CHOOSER(pSaveDlg));
	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(pSaveDlg), true);

	// Restore last folder and file name
	folder = CConfig::GetStr("VarFind", "Folder", NULL);
	if (!folder)
		folder = g_path_get_dirname(g_Project->Path());
	if (folder && g_file_test(folder, G_FILE_TEST_IS_DIR)) 
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(pSaveDlg), folder);
	g_free(folder);
	filename = CConfig::GetStr("VarFind", "FileName", "munifind.dat");
	if (filename) 
		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(pSaveDlg), filename);
	g_free(filename);

	if (gtk_dialog_run(GTK_DIALOG(pSaveDlg))==GTK_RESPONSE_ACCEPT) {
		// Save last folder and file name
		folder = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(pSaveDlg));
		CConfig::SetStr("VarFind", "Folder", folder);
		g_free(folder);
		m_ExportPath = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(pSaveDlg));
		CConfig::SetStr("VarFind", "FileName", g_basename(m_ExportPath));
		gtk_widget_hide(pSaveDlg);
		// Export data in separate thread
		CProgressDlg pdlg(GTK_WINDOW(m_pDlg), "Exporting munifind data");
		pdlg.SetFileName(g_basename(m_ExportPath));
		pdlg.Execute(ExportMunifindProc, this);
		g_free(m_ExportPath);
		m_ExportPath = NULL;
	}
	gtk_widget_destroy(pSaveDlg);
}

int CExtVarFindDlg::ExportMunifindProc(CProgressDlg *sender, void *user_data)
{
	return ((CExtVarFindDlg*)user_data)->OnExportMFind(sender);
}

int CExtVarFindDlg::OnExportMFind(CProgressDlg *sender)
{
	int res = 0;
	gchar *dst_path;
	CCatalog cat2;
	CCCDFile img2;

	if (m_File.Export(m_ExportPath, 0, &res)) {
		if (m_PhotometryFile) {
			// Save reference photometry file
			dst_path = SetFileExtension(m_ExportPath, FILE_EXTENSION_CATALOG);
			if (cat2.Open(dst_path, CMPACK_OPEN_CREATE)) {
				cat2.Assign(m_Phot, m_VarFind.ApertureIndex(), &res);
				cat2.Close();
			}
			g_free(dst_path);
		} else {
			// Copy catalog file
			dst_path = SetFileExtension(m_ExportPath, FILE_EXTENSION_CATALOG);
			if (cat2.Open(dst_path, CMPACK_OPEN_CREATE)) {
				cat2.Assign(m_Catalog, &res);
				cat2.Close();
			}
			g_free(dst_path);
		}
		if (res==0 && m_Frame.Valid()) {
			// Save image file
			dst_path = SetFileExtension(m_ExportPath, FILE_EXTENSION_FITS);
			if (img2.Open(dst_path, CMPACK_OPEN_CREATE)) {
				img2.Assign(m_Frame);
				img2.Close();
			}
			g_free(dst_path);
		}
	}
	return res;
}

void CExtVarFindDlg::RemoveFramesFromDataSet(void)
{
	bool changed = false;

	GList *rows = m_VarFind.GetSelectedFrames();
	if (rows) {
		for (GList *ptr=rows; ptr!=NULL; ptr=ptr->next) {
			int frame = (gint)cmpack_graph_data_get_param(m_VarFind.LightCurveData(), (intptr_t)ptr->data);
			m_File.DeleteFrame(frame);
			changed = true;
		}
		g_list_free(rows);
	}
	if (changed) {
		UpdateMagDev();
		UpdateLightCurve();
		UpdateControls();
	}
}

void CExtVarFindDlg::RemoveObjectsFromDataSet(void)
{
	bool changed = false;

	GList *rows = m_VarFind.GetSelectedObjects();
	if (rows) {
		for (GList *ptr=rows; ptr!=NULL; ptr=ptr->next) {
			int frame = (gint)cmpack_graph_data_get_param(m_VarFind.MagDevCurveData(), (intptr_t)ptr->data);
			m_File.DeleteObject(frame);
			changed = true;
		}
		g_list_free(rows);
	}
	if (changed) {
		m_VarFind.UpdateChart();
		UpdateMagDev();
		UpdateLightCurve();
		UpdateControls();
	}
}

//---------------------------   INFO DIALOG   --------------------------------

//
// Constructor
//
CVarFindInfoDlg::CVarFindInfoDlg(GtkWindow *pParent):CInfoDlg(pParent, 0)
{
	AddHeading(FIELD_FILE, 0, 0, "File");
	AddField(FIELD_FILENAME, 0, 1, "File name", PANGO_ELLIPSIZE_MIDDLE);
	AddField(FIELD_DIRPATH, 0, 2, "Location", PANGO_ELLIPSIZE_MIDDLE);
	gtk_widget_show_all(m_Tab);
}

//
// Show dialog
//
void CVarFindInfoDlg::ShowModal(const CFrameSet &file, const gchar *name, const gchar *path)
{
	char buf[512], *dirpath;

	// Window caption
	sprintf(buf, "%s - %s", name, "properties");
	gtk_window_set_title(GTK_WINDOW(m_pDlg), buf);

	// Update properties
	SetField(FIELD_FILENAME, g_basename(path));
	dirpath = g_path_get_dirname(path);
	SetField(FIELD_DIRPATH, dirpath);
	g_free(dirpath);

	// Show dialog
	CInfoDlg::ShowModal();
}

//-------------------------   VARFIND DIALOG   --------------------------------

//
// Constructor
//
CVarFindDlg::CVarFindDlg(void):m_FileList(NULL), m_ExportPath(NULL)
{
	gchar buf[512];

	// Dialog caption
	sprintf(buf, "%s - %s", "Find variables", g_AppTitle);
	gtk_window_set_title(GTK_WINDOW(m_pDlg), buf);

	// Dialog icon
	gchar *icon = get_icon_file("varfind");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Menu bar
	m_Menu.Create(PreviewMenu2, false);
	m_Menu.RegisterCallback(MenuCallback, this);
	gtk_box_pack_start(GTK_BOX(m_MainBox), m_Menu.Handle(), FALSE, FALSE, 0);

	// VarFind box
	m_VarFind.RegisterCallback(VarFindCallback, this);
	gtk_box_pack_start(GTK_BOX(m_MainBox), m_VarFind.Handle(), TRUE, TRUE, 0);
	
	// Show the dialog
	gtk_widget_show_all(m_MainBox);
}

//
// Destructor
//
CVarFindDlg::~CVarFindDlg(void)
{
	g_slist_foreach(m_FileList, (GFunc)gtk_tree_row_reference_free, NULL);
	g_slist_free(m_FileList);
	m_FileList = NULL;
	g_free(m_ExportPath);
}

//
// Open a new dialog
//
bool CVarFindDlg::Open(bool selected_files)
{
	int res;
	char *pht_file, *fts_file;
	const char *tmp_file;
	GtkTreePath *refpath;
	GtkWindow *pMainWnd = g_MainWnd->GetHandle();

	m_FrameSet.Clear();
	m_Phot.Close();
	m_Frame.Close();
	m_Image.Clear();
	m_Catalog.Close();
	g_slist_foreach(m_FileList, (GFunc)gtk_tree_row_reference_free, NULL);
	g_slist_free(m_FileList);
	m_FileList = NULL;
	m_VarFind.SetApertures(*g_Project->RefApertures());

	// Reference file
	m_RefType = g_Project->GetReferenceType();
	switch (m_RefType)
	{
	case REF_FRAME:
		refpath = g_Project->GetReferencePath();
		if (refpath) {
			pht_file = g_Project->GetPhotFile(refpath);
			if (m_Phot.Open(pht_file, CMPACK_OPEN_READONLY, 0, &res)) {
				m_VarFind.SetPhotometryFile(&m_Phot);
				fts_file = g_Project->GetImageFile(refpath);
				if (m_Frame.Open(fts_file, CMPACK_OPEN_READONLY, NULL)) {
					if (m_Image.Load(m_Frame, CMPACK_BITPIX_AUTO)) 
						m_VarFind.SetImage(&m_Image);
				}
				g_free(fts_file);
			}
			g_free(pht_file);
			gtk_tree_path_free(refpath);
		}
		break;

	case REF_CATALOG_FILE:
		tmp_file = g_Project->GetTempCatFile()->FullPath();
		if (m_Catalog.Open(tmp_file, CMPACK_OPEN_READONLY, &res)) {
			m_VarFind.SetCatalogFile(&m_Catalog);
			fts_file = SetFileExtension(tmp_file, FILE_EXTENSION_FITS);
			if (m_Frame.Open(fts_file, CMPACK_OPEN_READONLY, NULL)) {
				if (m_Image.Load(m_Frame, CMPACK_BITPIX_AUTO)) 
					m_VarFind.SetImage(&m_Image);
			}
			g_free(fts_file);
		}
		break;

	default:
		ShowError(pMainWnd, "No reference file.");
		return false;
	}

	if (!selected_files) {
		// All files
		gtk_tree_model_foreach(g_Project->FileList(), (GtkTreeModelForeachFunc)foreach_all_files, &m_FileList);
		if (!m_FileList) {
			ShowError(pMainWnd, "There are no files in the project.");
			return false;
		}
	} else {
		// Selected files
		GtkTreeSelection *pSel = g_MainWnd->GetSelection();
		if (gtk_tree_selection_count_selected_rows(pSel)>0) 
			gtk_tree_selection_selected_foreach(pSel, (GtkTreeSelectionForeachFunc)foreach_sel_files, &m_FileList);
		if (!m_FileList) {
			ShowError(pMainWnd, "There are no selected files.");
			return false;
		}
	}

	return true;
}

//
// Dialog initialization
//
void CVarFindDlg::OnInitDialog(void)
{
	RebuildFrameSet();
}

//
// Menu callback
//
void CVarFindDlg::MenuCallback(CCBObject *sender, int message, int wparam, void *lparam, void *cb_data)
{
	CVarFindDlg *pMe = (CVarFindDlg*)cb_data;

	switch (message)
	{
	case CMenuBar::CB_ACTIVATE:
		pMe->OnCommand(wparam);
		break;
	}
}

//
// Command handler
//
void CVarFindDlg::OnCommand(int cmd_id)
{
	switch (cmd_id)
	{
	// File menu
	case CMD_SAVE_CHART:
		m_VarFind.ExportChart();
		break;
	case CMD_SAVE_LCURVE:
		m_VarFind.SaveLightCurve();
		break;
	case CMD_SAVE_MAGDEV:
		m_VarFind.SaveMagDevCurve();
		break;
	case CMD_EXPORT_LCURVE:
		m_VarFind.ExportLightCurve();
		break;
	case CMD_EXPORT_MAGDEV:
		m_VarFind.ExportMagDevCurve();
		break;
	case CMD_EXPORT_DATA:
		ExportData();
		break;
	case CMD_CLOSE:
		Close();
		break;
	case CMD_REBUILD:
		UpdateFileList();
		RebuildFrameSet();
		break;

	// View menu
	case CMD_SHOW_CHART:
		m_VarFind.SetDisplayMode(CVarFind::DISPLAY_CHART);
		UpdateControls();
		break;
	case CMD_SHOW_IMAGE:
		m_VarFind.SetDisplayMode(CVarFind::DISPLAY_IMAGE);
		UpdateControls();
		break;
	case CMD_SHOW_MIXED:
		m_VarFind.SetDisplayMode(CVarFind::DISPLAY_MIXED);
		UpdateControls();
		break;
		
	// Help menu
	case CMD_HELP:
		g_MainWnd->ShowHelp(GTK_WINDOW(m_pDlg), IDH_FIND_VARIABLES);
		break;
	}
}

//
// Update frame list
//
void CVarFindDlg::UpdateFileList(void)
{
	g_slist_foreach(m_FileList, (GFunc)gtk_tree_row_reference_free, NULL);
	g_slist_free(m_FileList);
	m_FileList = NULL;

	g_Project->Lock();
	GtkTreeModel *frames = g_Project->FileList();
	gtk_tree_model_foreach(frames, (GtkTreeModelForeachFunc)foreach_all_files, &m_FileList);
	g_Project->Unlock();
}

//
// VarFind callback
//
void CVarFindDlg::VarFindCallback(CCBObject *sender, int message, int wparam, void *lparam, void *cb_data)
{
	CVarFindDlg *pMe = (CVarFindDlg*)cb_data;

	switch (message)
	{
	case CVarFind::CB_APERTURE_CHANGED:
		pMe->RebuildFrameSet();
		break;

	case CVarFind::CB_COMPARISON_CHANGED:
		pMe->UpdateMagDev();
		pMe->UpdateLightCurve();
		pMe->UpdateControls();
		break;

	case CVarFind::CB_VARIABLE_CHANGED:
		pMe->UpdateLightCurve();
		pMe->UpdateControls();
		break;

	case CVarFind::CB_ENABLE_CTRL_QUERY:
		*((bool*)lparam) = pMe->OnEnableCtrlQuery((CVarFind::tControlId)wparam);
		break;

	case CVarFind::CB_OBJECT_VALID_QUERY:
		*((bool*)lparam) = pMe->m_FrameSet.FindObject(wparam)>=0;
		break;

	case CVarFind::CB_REMOVE_FRAMES_FROM_DATASET:
		pMe->RemoveFramesFromDataSet();
		break;
	case CVarFind::CB_DELETE_FRAMES_FROM_PROJECT:
		pMe->DeleteFramesFromProject();
		break;
	case CVarFind::CB_SHOW_FRAME_INFO:
		pMe->ShowFrameInfo();
		break;
	case CVarFind::CB_SHOW_FRAME_PREVIEW:
		pMe->ShowFramePreview();
		break;
	case CVarFind::CB_REMOVE_OBJECTS_FROM_DATASET:
		pMe->RemoveObjectsFromDataSet();
		break;
	}
}

//
// Aperture change, rebuild data
//
void CVarFindDlg::RebuildFrameSet(void)
{
	int res;
	const CAperture *aper = m_VarFind.Aperture();

	m_FrameSet.Clear();
	if (aper) {
		if (m_RefType==REF_FRAME)
			m_FrameSet.Init(*aper, m_Phot);
		else
			m_FrameSet.Init(*aper, m_Catalog);
		g_Project->SetInt("LightCurve", "Aperture", aper->Id());

		CProgressDlg pDlg(GTK_WINDOW(m_pDlg), "Processing files");
		pDlg.SetMinMax(0, g_slist_length(m_FileList));
		res = pDlg.Execute(ExecuteProc, this);
		if (res!=0 && !pDlg.Cancelled()) {
			char *msg = cmpack_formaterror(res);
			ShowError(GTK_WINDOW(m_pDlg), msg);
			cmpack_free(msg);
		}
	}

	m_VarFind.UpdateChart();
	UpdateMagDev();
	UpdateLightCurve();
	UpdateControls();
}

//
// Comparison changed
//
void CVarFindDlg::UpdateMagDev(void)
{
	int res, refstar;
	CMFindProc mfind;

	const CAperture *aperture = m_VarFind.Aperture();
	if (aperture) {
		refstar = m_VarFind.ComparisonID();
		if (refstar<0)
			refstar = CMPACK_MFIND_AUTO;
		res = mfind.Init(NULL, *aperture, refstar);
		if (res==0)
			res = mfind.Execute(m_FrameSet, m_MagDev, &refstar);
		if (res==0 && refstar!=m_VarFind.ComparisonID()) 
			m_VarFind.SetComparisonStar(refstar);
	}
	m_VarFind.SetMagDev(&m_MagDev);
}

//
// Variable changed
//
void CVarFindDlg::UpdateLightCurve(void)
{
	int varstar, compstar;
	CSelection sel;

	varstar = m_VarFind.VariableID();
	compstar = m_VarFind.ComparisonID();
	if (varstar>=0 && compstar>=0) {
		const CAperture *aperture = m_VarFind.Aperture();
		if (aperture) {
			sel.Select(varstar, CMPACK_SELECT_VAR);
			sel.Select(compstar, CMPACK_SELECT_COMP);
			CmpackLightCurve(NULL, m_LCurve, sel, m_FrameSet, aperture->Id(), CMPACK_LCURVE_FRAME_IDS);
			g_Project->SetSelection(sel);
			g_Project->SetTags(CTags());
		}
	}
	m_VarFind.SetLightCurve(&m_LCurve);
}

//
// Enable/disable controls
//
void CVarFindDlg::UpdateControls(void)
{
	int varstar = m_VarFind.VariableID(), compstar = m_VarFind.ComparisonID(), 
		aperture = m_VarFind.ApertureIndex();

	m_Menu.Enable(CMD_SHOW_IMAGE, m_Frame.Valid());
	m_Menu.Enable(CMD_SHOW_MIXED, m_Frame.Valid());
	m_Menu.Check(CMD_SHOW_IMAGE, m_VarFind.DisplayMode()==CVarFind::DISPLAY_IMAGE);
	m_Menu.Check(CMD_SHOW_CHART, m_VarFind.DisplayMode()==CVarFind::DISPLAY_CHART);
	m_Menu.Check(CMD_SHOW_MIXED, m_VarFind.DisplayMode()==CVarFind::DISPLAY_MIXED);
	m_Menu.Enable(CMD_EXPORT_DATA, aperture>=0);
	m_Menu.Enable(CMD_SAVE_MAGDEV, compstar>=0 && aperture>=0);
	m_Menu.Enable(CMD_SAVE_LCURVE, varstar>=0 && compstar>=0 && aperture>=0);
	m_Menu.Enable(CMD_EXPORT_MAGDEV, compstar>=0 && aperture>=0);
	m_Menu.Enable(CMD_EXPORT_LCURVE, varstar>=0 && compstar>=0 && aperture>=0);
}

//
// Is this control enabled?
//
bool CVarFindDlg::OnEnableCtrlQuery(CVarFind::tControlId ctrl)
{
	switch (ctrl)
	{
	case CVarFind::ID_SAVE_CHART:
		return (m_RefType==REF_FRAME ? m_Phot.Valid() : m_Catalog.Valid());
	case CVarFind::ID_SAVE_LCURVE:
		return m_VarFind.VariableID()>=0 && m_VarFind.ComparisonID()>=0 && 
			m_VarFind.ApertureIndex()>=0;
	case CVarFind::ID_SHOW_FRAME_PREVIEW:
	case CVarFind::ID_SHOW_FRAME_INFO:
	case CVarFind::ID_REMOVE_FROM_DATASET:
	case CVarFind::ID_DELETE_FROM_PROJECT:
		return true;
	default:
		return false;
	}
}

int CVarFindDlg::ExecuteProc(CProgressDlg *sender, void *userdata)
{
	return ((CVarFindDlg*)userdata)->OnProcessFiles(sender);
}

int CVarFindDlg::OnProcessFiles(CProgressDlg *sender)
{
	int res, infiles;
	GtkTreePath *path;
	CListProc list;

	// Make listing context
	res = list.Init(sender, &m_FrameSet, false);
	if (res==0) {
		// Process files
		infiles = 0;
		for (GSList *node = m_FileList; node != NULL; node = node->next) {
			path = gtk_tree_row_reference_get_path((GtkTreeRowReference*)node->data);
			if (path) {
				res = list.Add(path);
				gtk_tree_path_free(path);
				if (res!=0)
					break;
			}
			if (sender) {
				if (sender->Cancelled())
					break;
				sender->SetProgress(infiles++);
			}
		}
	}

	return res;
}

void CVarFindDlg::ExportData(void)
{
	char *filename, *folder;

	GtkWidget *pSaveDlg = gtk_file_chooser_dialog_new("Export munifind data",
		GTK_WINDOW(m_pDlg), GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, 
		GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_standard_tooltips(GTK_FILE_CHOOSER(pSaveDlg));
	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(pSaveDlg), true);

	// Restore last folder and file name
	folder = CConfig::GetStr("VarFind", "Folder", NULL);
	if (!folder)
		folder = g_path_get_dirname(g_Project->Path());
	if (folder && g_file_test(folder, G_FILE_TEST_IS_DIR)) 
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(pSaveDlg), folder);
	g_free(folder);
	filename = CConfig::GetStr("VarFind", "FileName", "munifind.dat");
	if (filename) 
		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(pSaveDlg), filename);
	g_free(filename);

	if (gtk_dialog_run(GTK_DIALOG(pSaveDlg))==GTK_RESPONSE_ACCEPT) {
		// Save last folder and file name
		folder = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(pSaveDlg));
		CConfig::SetStr("VarFind", "Folder", folder);
		g_free(folder);
		m_ExportPath = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(pSaveDlg));
		CConfig::SetStr("VarFind", "FileName", g_basename(m_ExportPath));
		gtk_widget_hide(pSaveDlg);
		// Export data in separate thread
		CProgressDlg pdlg(GTK_WINDOW(m_pDlg), "Exporting munifind data");
		pdlg.SetFileName(g_basename(m_ExportPath));
		pdlg.Execute(ExportMunifindProc, this);
		g_free(m_ExportPath);
		m_ExportPath = NULL;
	}
	gtk_widget_destroy(pSaveDlg);
}

int CVarFindDlg::ExportMunifindProc(CProgressDlg *sender, void *user_data)
{
	return ((CVarFindDlg*)user_data)->OnExportMFind(sender);
}

int CVarFindDlg::OnExportMFind(CProgressDlg *sender)
{
	int res = 0;
	gchar *dst_path;
	CCatalog cat2;
	CCCDFile img2;

	if (m_FrameSet.Export(m_ExportPath, 0, &res)) {
		if (m_RefType==REF_FRAME) {
			// Save reference CCD image
			dst_path = SetFileExtension(m_ExportPath, FILE_EXTENSION_CATALOG);
			if (cat2.Open(dst_path, CMPACK_OPEN_CREATE)) {
				cat2.Assign(m_Phot, m_VarFind.ApertureIndex(), &res);
				cat2.Close();
			}
			g_free(dst_path);
		} else {
			// Copy catalog file
			dst_path = SetFileExtension(m_ExportPath, FILE_EXTENSION_CATALOG);
			if (cat2.Open(dst_path, CMPACK_OPEN_CREATE)) {
				cat2.Assign(m_Catalog, &res);
				cat2.Close();
			}
			g_free(dst_path);
		}
		if (res==0 && m_Frame.Valid()) {
			// Save CCD frame 
			dst_path = SetFileExtension(m_ExportPath, FILE_EXTENSION_FITS);
			if (img2.Open(dst_path, CMPACK_OPEN_CREATE)) {
				img2.Assign(m_Frame);
				img2.Close();
			}
			g_free(dst_path);
		}
	}
	return res;
}

void CVarFindDlg::RemoveFramesFromDataSet(void)
{
	bool changed = false;

	GList *rows = m_VarFind.GetSelectedFrames();
	if (rows) {
		for (GList *ptr=rows; ptr!=NULL; ptr=ptr->next) {
			int frame = (gint)cmpack_graph_data_get_param(m_VarFind.LightCurveData(), (intptr_t)ptr->data);
			m_FrameSet.DeleteFrame(frame);
			changed = true;
		}
		g_list_free(rows);
	}
	if (changed) {
		UpdateMagDev();
		UpdateLightCurve();
		UpdateControls();
	}
}

void CVarFindDlg::RemoveObjectsFromDataSet(void)
{
	bool changed = false;

	GList *rows = m_VarFind.GetSelectedObjects();
	if (rows) {
		for (GList *ptr=rows; ptr!=NULL; ptr=ptr->next) {
			int object = (gint)cmpack_graph_data_get_param(m_VarFind.MagDevCurveData(), (intptr_t)ptr->data);
			m_FrameSet.DeleteObject(object);
			changed = true;
		}
		g_list_free(rows);
	}
	if (changed) {
		m_VarFind.UpdateChart();
		UpdateMagDev();
		UpdateLightCurve();
		UpdateControls();
	}
}

void CVarFindDlg::DeleteFramesFromProject(void)
{
	bool ok, selected_ref;
	int frame, count;

	GList *rows = m_VarFind.GetSelectedFrames();
	if (rows) {
		count = g_list_length(rows);
		selected_ref = false;
		for (GList *ptr=rows; ptr!=NULL; ptr=ptr->next) {
			frame = (gint)cmpack_graph_data_get_param(m_VarFind.LightCurveData(), (intptr_t)ptr->data);
			if (frame == g_Project->GetReferenceFrame()) {
				selected_ref = true;
				break;
			}
		}
		if (selected_ref) {
			if (count==1) 
				ShowError(GTK_WINDOW(m_pDlg), "The selected frame is a reference frame. It it not allowed to remove it.");
			else
				ShowError(GTK_WINDOW(m_pDlg), "The selection includes a reference frame. It is not allowed to remove it.");
			ok = false;
		} else {
			if (count==1) 
				ok = ShowConfirmation(GTK_WINDOW(m_pDlg), "Do you want to remove the selected frame from the project?");
			else 
				ok = ShowConfirmation(GTK_WINDOW(m_pDlg), "Do you want to remove the selected frames from the project?");
		}
		if (ok) {
			for (GList *ptr=rows; ptr!=NULL; ptr=ptr->next) {
				frame = (gint)cmpack_graph_data_get_param(m_VarFind.LightCurveData(), (intptr_t)ptr->data);
				m_FrameSet.DeleteFrame(frame);
				g_Project->RemoveFrame(frame);
			}
			UpdateMagDev();
			UpdateLightCurve();
			UpdateControls();
		}
		g_list_free(rows);
	}
}

void CVarFindDlg::ShowFramePreview(void)
{
	int frame = m_VarFind.SelectedFrameID();
	if (frame>=0) {
		GtkTreePath *path = g_Project->GetFilePath(frame);
		if (path) {
			g_MainWnd->ShowFramePreview(path);
			gtk_tree_path_free(path);
		}
	}
}

void CVarFindDlg::ShowFrameInfo(void)
{
	int frame = m_VarFind.SelectedFrameID();
	if (frame>=0) {
		GtkTreePath *path = g_Project->GetFilePath(frame);
		if (path) {
			CFrameInfoDlg dlg(GTK_WINDOW(m_pDlg));
			dlg.Show(path);
			gtk_tree_path_free(path);
		}
	}
}
